@@ -127,7 +127,7 @@ class CIB12(ConfigBase):
         (rc, output) = self.Factory.rsh(self.Factory.target,
             r"""awk -v RS="}" """
             r"""'/^(\s*nodelist\s*{)?\s*node\s*{.*(ring0_addr|name):\s*%s(\s+|$)/"""
-            r"""{gsub(/.*nodeid:\s*/,"");gsub(/\s+.*$/,"");print}'%s"""
+            r"""{gsub(/.*nodeid:\s*/,"");gsub(/\s+.*$/,"");print}' %s"""
             % (node_name, CTSvars.COROSYNC_CONF), None)
 
         if rc == 0 and len(output) == 1:
@@ -512,7 +512,7 @@ if __name__ == '__main__':
         "--stonith", "rhcs",
     ]
     env = CTS.CtsLab(args)
-    cm = CM_corosync.crm_corosync(env)
+    cm = CM_corosync.crm_corosync()
     CibFactory = ConfigFactory(cm)
     cib = CibFactory.createConfig("pacemaker-3.0")
     print(cib.contents())
