@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2021 the Pacemaker project contributors
+ * Copyright 2013-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,23 +15,52 @@
 #include <crm/crm.h>
 #include <crm/cluster.h>
 #include <crm/cluster/election_internal.h>
+#include <crm/common/messages_internal.h>
 #include <crm/cib/internal.h>
 
+/*
+ * Legacy attrd (all pre-1.1.11 Pacemaker versions, plus all versions when used
+ * with the no-longer-supported CMAN or corosync-plugin stacks) is unversioned.
+ *
+ * With atomic attrd, each attrd will send ATTRD_PROTOCOL_VERSION with every
+ * peer request and reply. As of Pacemaker 2.0.0, at start-up each attrd will
+ * also set a private attribute for itself with its version, so any attrd can
+ * determine the minimum version supported by all peers.
+ *
+ * Protocol  Pacemaker  Significant changes
+ * --------  ---------  -------------------
+ *     1       1.1.11   PCMK__ATTRD_CMD_UPDATE (PCMK__XA_ATTR_NAME only),
+ *                      PCMK__ATTRD_CMD_PEER_REMOVE, PCMK__ATTRD_CMD_REFRESH,
+ *                      PCMK__ATTRD_CMD_FLUSH, PCMK__ATTRD_CMD_SYNC,
+ *                      PCMK__ATTRD_CMD_SYNC_RESPONSE
+ *     1       1.1.13   PCMK__ATTRD_CMD_UPDATE (with PCMK__XA_ATTR_PATTERN),
+ *                      PCMK__ATTRD_CMD_QUERY
+ *     1       1.1.15   PCMK__ATTRD_CMD_UPDATE_BOTH,
+ *                      PCMK__ATTRD_CMD_UPDATE_DELAY
+ *     2       1.1.17   PCMK__ATTRD_CMD_CLEAR_FAILURE
+ *     3       2.1.1    PCMK__ATTRD_CMD_SYNC_RESPONSE indicates remote nodes
+ *     4       2.2.0    Multiple attributes can be updated in a single IPC
+ *                      message
+ */
+#define ATTRD_PROTOCOL_VERSION "4"
+
+#define attrd_send_ack(client, id, flags) \
+    pcmk__ipc_send_ack((client), (id), (flags), "ack", ATTRD_PROTOCOL_VERSION, CRM_EX_INDETERMINATE)
+
 void attrd_init_mainloop(void);
 void attrd_run_mainloop(void);
 
 void attrd_set_requesting_shutdown(void);
 void attrd_clear_requesting_shutdown(void);
-gboolean attrd_requesting_shutdown(void);
-gboolean attrd_shutting_down(void);
+bool attrd_requesting_shutdown(void);
+bool attrd_shutting_down(void);
 void attrd_shutdown(int nsig);
-void attrd_init_ipc(qb_ipcs_service_t **ipcs,
-                    qb_ipcs_msg_process_fn dispatch_fn);
+void attrd_init_ipc(void);
 void attrd_ipc_fini(void);
 
 void attrd_cib_disconnect(void);
 
-gboolean attrd_value_needs_expansion(const char *value);
+bool attrd_value_needs_expansion(const char *value);
 int attrd_expand_value(const char *value, const char *old_value);
 
 /* regular expression to clear failures of all resources */
@@ -115,24 +144,42 @@ typedef struct attribute_value_s {
 extern crm_cluster_t *attrd_cluster;
 extern GHashTable *attributes;
 
-#define attrd_send_ack(client, id, flags) \
-    pcmk__ipc_send_ack((client), (id), (flags), "ack", CRM_EX_INDETERMINATE)
-
 #define CIB_OP_TIMEOUT_S 120
 
-void write_attributes(bool all, bool ignore_delay);
+int attrd_cluster_connect(void);
+void attrd_peer_update(const crm_node_t *peer, xmlNode *xml, const char *host,
+                       bool filter);
+void attrd_peer_sync(crm_node_t *peer, xmlNode *xml);
+void attrd_peer_remove(const char *host, bool uncache, const char *source);
+void attrd_peer_clear_failure(pcmk__request_t *request);
+void attrd_peer_sync_response(const crm_node_t *peer, bool peer_won,
+                              xmlNode *xml);
+
 void attrd_broadcast_protocol(void);
-void attrd_peer_message(crm_node_t *client, xmlNode *msg);
-void attrd_client_peer_remove(pcmk__client_t *client, xmlNode *xml);
-void attrd_client_clear_failure(xmlNode *xml);
-void attrd_client_update(xmlNode *xml);
-void attrd_client_refresh(void);
-void attrd_client_query(pcmk__client_t *client, uint32_t id, uint32_t flags,
-                        xmlNode *query);
+xmlNode *attrd_client_peer_remove(pcmk__request_t *request);
+xmlNode *attrd_client_clear_failure(pcmk__request_t *request);
+xmlNode *attrd_client_update(pcmk__request_t *request);
+xmlNode *attrd_client_refresh(pcmk__request_t *request);
+xmlNode *attrd_client_query(pcmk__request_t *request);
+gboolean attrd_send_message(crm_node_t * node, xmlNode * data);
 
-void free_attribute(gpointer data);
+xmlNode *attrd_add_value_xml(xmlNode *parent, const attribute_t *a,
+                             const attribute_value_t *v, bool force_write);
+void attrd_clear_value_seen(void);
+void attrd_free_attribute(gpointer data);
+void attrd_free_attribute_value(gpointer data);
+attribute_t *attrd_populate_attribute(xmlNode *xml, const char *attr);
 
-gboolean attrd_election_cb(gpointer user_data);
-void attrd_peer_change_cb(enum crm_status_type type, crm_node_t *peer, const void *data);
+void attrd_write_attribute(attribute_t *a, bool ignore_delay);
+void attrd_write_attributes(bool all, bool ignore_delay);
+void attrd_write_or_elect_attribute(attribute_t *a);
+
+extern int minimum_protocol_version;
+void attrd_update_minimum_protocol_ver(const char *value);
+
+mainloop_timer_t *attrd_add_timer(const char *id, int timeout_ms, attribute_t *attr);
+
+void attrd_unregister_handlers(void);
+void attrd_handle_request(pcmk__request_t *request);
 
 #endif /* PACEMAKER_ATTRD__H */
