@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -18,7 +18,7 @@
 #include <stdint.h>     // uint32_t, uint64_t, UINT64_C()
 #include <errno.h>
 #include <fcntl.h>
-#include <inttypes.h>  /* U64T ~ PRIu64 */
+#include <inttypes.h>   // PRIu64
 
 #include <crm/crm.h>
 #include <crm/cib.h>
@@ -51,7 +51,7 @@ qb_ipcs_service_t *ipcs_shm = NULL;
 
 void send_cib_replace(const xmlNode * sync_request, const char *host);
 static void cib_process_request(xmlNode *request, gboolean privileged,
-                                pcmk__client_t *cib_client);
+                                const pcmk__client_t *cib_client);
 
 
 static int cib_process_command(xmlNode *request, xmlNode **reply,
@@ -166,8 +166,8 @@ cib_common_callback_worker(uint32_t id, uint32_t flags, xmlNode * op_request,
 
         crm_element_value_int(op_request, F_CIB_NOTIFY_ACTIVATE, &on_off);
 
-        crm_debug("Setting %s callbacks for %s (%s): %s",
-                  type, cib_client->name, cib_client->id, on_off ? "on" : "off");
+        crm_debug("Setting %s callbacks %s for client %s",
+                  type, (on_off? "on" : "off"), pcmk__client_name(cib_client));
 
         if (pcmk__str_eq(type, T_CIB_POST_NOTIFY, pcmk__str_casei)) {
             bit = cib_notify_post;
@@ -196,7 +196,7 @@ cib_common_callback_worker(uint32_t id, uint32_t flags, xmlNode * op_request,
             }
         }
 
-        pcmk__ipc_send_ack(cib_client, id, flags, "ack", status);
+        pcmk__ipc_send_ack(cib_client, id, flags, "ack", NULL, status);
         return;
     }
 
@@ -218,7 +218,7 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
 
     if (op_request == NULL) {
         crm_trace("Invalid message from %p", c);
-        pcmk__ipc_send_ack(cib_client, id, flags, "nack", CRM_EX_PROTOCOL);
+        pcmk__ipc_send_ack(cib_client, id, flags, "nack", NULL, CRM_EX_PROTOCOL);
         return 0;
 
     } else if(cib_client == NULL) {
@@ -236,7 +236,7 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
         const char *value = crm_element_value(op_request, F_CIB_CLIENTNAME);
 
         if (value == NULL) {
-            cib_client->name = crm_itoa(cib_client->pid);
+            cib_client->name = pcmk__itoa(cib_client->pid);
         } else {
             cib_client->name = strdup(value);
             if (crm_is_daemon_name(value)) {
@@ -250,18 +250,17 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
         const char *qmax = cib_config_lookup("cluster-ipc-limit");
 
         if (pcmk__set_client_queue_max(cib_client, qmax)) {
-            crm_trace("IPC threshold for %s[%u] is now %u",
-                      cib_client->name, cib_client->pid, cib_client->queue_max);
+            crm_trace("IPC threshold for client %s[%u] is now %u",
+                      pcmk__client_name(cib_client), cib_client->pid,
+                      cib_client->queue_max);
         }
     }
 
     crm_xml_add(op_request, F_CIB_CLIENTID, cib_client->id);
     crm_xml_add(op_request, F_CIB_CLIENTNAME, cib_client->name);
 
-#if ENABLE_ACL
     CRM_LOG_ASSERT(cib_client->user != NULL);
     pcmk__update_acl_user(op_request, F_CIB_USER, cib_client->user);
-#endif
 
     cib_common_callback_worker(id, flags, op_request, cib_client, privileged);
     free_xml(op_request);
@@ -277,7 +276,7 @@ int sync_our_cib(xmlNode * request, gboolean all);
 static gboolean
 cib_digester_cb(gpointer data)
 {
-    if (cib_is_master) {
+    if (based_is_primary) {
         char buffer[32];
         xmlNode *ping = create_xml_node(NULL, "ping");
 
@@ -285,7 +284,7 @@ cib_digester_cb(gpointer data)
         free(ping_digest);
         ping_digest = NULL;
         ping_modified_since = FALSE;
-        snprintf(buffer, 32, "%" U64T, ping_seq);
+        snprintf(buffer, 32, "%" PRIu64, ping_seq);
         crm_trace("Requesting peer digests (%s)", buffer);
 
         crm_xml_add(ping, F_TYPE, "cib");
@@ -310,8 +309,17 @@ process_ping_reply(xmlNode *reply)
     const char *seq_s = crm_element_value(pong, F_CIB_PING_ID);
     const char *digest = crm_element_value(pong, XML_ATTR_DIGEST);
 
-    if (seq_s) {
-        seq = (uint64_t) crm_parse_ll(seq_s, NULL);
+    if (seq_s == NULL) {
+        crm_debug("Ignoring ping reply with no " F_CIB_PING_ID);
+        return;
+
+    } else {
+        long long seq_ll;
+
+        if (pcmk__scan_ll(seq_s, &seq_ll, 0LL) != pcmk_rc_ok) {
+            return;
+        }
+        seq = (uint64_t) seq_ll;
     }
 
     if(digest == NULL) {
@@ -384,17 +392,19 @@ do_local_notify(xmlNode * notify_src, const char *client_id,
             rid = client_obj->request_id;
             client_obj->request_id = 0;
 
-            crm_trace("Sending response %d to %s %s",
-                      rid, client_obj->name,
-                      from_peer ? "(originator of delegated request)" : "");
+            crm_trace("Sending response %d to client %s%s",
+                      rid, pcmk__client_name(client_obj),
+                      (from_peer? " (originator of delegated request)" : ""));
         } else {
-            crm_trace("Sending response [call %d] to %s %s",
-                      call_id, client_obj->name, from_peer ? "(originator of delegated request)" : "");
+            crm_trace("Sending response (call %d) to client %s%s",
+                      call_id, pcmk__client_name(client_obj),
+                      (from_peer? " (originator of delegated request)" : ""));
         }
 
     } else {
-        crm_trace("Sending event %d to %s %s",
-                  call_id, client_obj->name, from_peer ? "(originator of delegated request)" : "");
+        crm_trace("Sending event %d to client %s%s",
+                  call_id, pcmk__client_name(client_obj),
+                  (from_peer? " (originator of delegated request)" : ""));
     }
 
     switch (PCMK__CLIENT_TYPE(client_obj)) {
@@ -405,9 +415,10 @@ do_local_notify(xmlNode * notify_src, const char *client_id,
                                              : crm_ipc_server_event));
 
                 if (rc != pcmk_rc_ok) {
-                    crm_warn("%s reply to %s failed: %s " CRM_XS " rc=%d",
+                    crm_warn("%s reply to client %s failed: %s " CRM_XS " rc=%d",
                              (sync_reply? "Synchronous" : "Asynchronous"),
-                             client_obj->name, pcmk_rc_str(rc), rc);
+                             pcmk__client_name(client_obj), pcmk_rc_str(rc),
+                             rc);
                 }
             }
             break;
@@ -418,9 +429,9 @@ do_local_notify(xmlNode * notify_src, const char *client_id,
             pcmk__remote_send_xml(client_obj->remote, notify_src);
             break;
         default:
-            crm_err("Unknown transport for %s " CRM_XS " flags=0x%llx",
-                    pcmk__client_name(client_obj),
-                    (unsigned long long) client_obj->flags);
+            crm_err("Unknown transport for client %s "
+                    CRM_XS " flags=%#016" PRIx64,
+                    pcmk__client_name(client_obj), client_obj->flags);
     }
 }
 
@@ -443,12 +454,12 @@ check_local_notify(int bcast_id)
         return;
     }
 
-    notify = g_hash_table_lookup(local_notify_queue, GINT_TO_POINTER(bcast_id));
+    notify = pcmk__intkey_table_lookup(local_notify_queue, bcast_id);
 
     if (notify) {
         do_local_notify(notify->notify_src, notify->client_id, notify->sync_reply,
                         notify->from_peer);
-        g_hash_table_remove(local_notify_queue, GINT_TO_POINTER(bcast_id));
+        pcmk__intkey_table_remove(local_notify_queue, bcast_id);
     }
 }
 
@@ -464,18 +475,15 @@ queue_local_notify(xmlNode * notify_src, const char *client_id, gboolean sync_re
     notify->from_peer = from_peer;
 
     if (!local_notify_queue) {
-        local_notify_queue = g_hash_table_new_full(g_direct_hash,
-                                                   g_direct_equal, NULL,
-                                                   local_notify_destroy_callback);
+        local_notify_queue = pcmk__intkey_table(local_notify_destroy_callback);
     }
-
-    g_hash_table_insert(local_notify_queue, GINT_TO_POINTER(cib_local_bcast_num), notify);
+    pcmk__intkey_table_insert(local_notify_queue, cib_local_bcast_num, notify);
     // cppcheck doesn't know notify will get freed when hash table is destroyed
     // cppcheck-suppress memleak
 }
 
 static void
-parse_local_options_v1(pcmk__client_t *cib_client, int call_type,
+parse_local_options_v1(const pcmk__client_t *cib_client, int call_type,
                        int call_options, const char *host, const char *op,
                        gboolean *local_notify, gboolean *needs_reply,
                        gboolean *process, gboolean *needs_forward)
@@ -489,15 +497,18 @@ parse_local_options_v1(pcmk__client_t *cib_client, int call_type,
     }
 
     if (host == NULL && (call_options & cib_scope_local)) {
-        crm_trace("Processing locally scoped %s op from %s", op, cib_client->name);
+        crm_trace("Processing locally scoped %s op from client %s",
+                  op, pcmk__client_name(cib_client));
         *local_notify = TRUE;
 
-    } else if (host == NULL && cib_is_master) {
-        crm_trace("Processing master %s op locally from %s", op, cib_client->name);
+    } else if ((host == NULL) && based_is_primary) {
+        crm_trace("Processing %s op locally from client %s as primary",
+                  op, pcmk__client_name(cib_client));
         *local_notify = TRUE;
 
     } else if (pcmk__str_eq(host, cib_our_uname, pcmk__str_casei)) {
-        crm_trace("Processing locally addressed %s op from %s", op, cib_client->name);
+        crm_trace("Processing locally addressed %s op from client %s",
+                  op, pcmk__client_name(cib_client));
         *local_notify = TRUE;
 
     } else if (stand_alone) {
@@ -506,21 +517,23 @@ parse_local_options_v1(pcmk__client_t *cib_client, int call_type,
         *process = TRUE;
 
     } else {
-        crm_trace("%s op from %s needs to be forwarded to %s",
-                  op, cib_client->name, host ? host : "the master instance");
+        crm_trace("%s op from %s needs to be forwarded to client %s",
+                  op, pcmk__client_name(cib_client),
+                  pcmk__s(host, "the primary instance"));
         *needs_forward = TRUE;
         *process = FALSE;
     }
 }
 
 static void
-parse_local_options_v2(pcmk__client_t *cib_client, int call_type,
+parse_local_options_v2(const pcmk__client_t *cib_client, int call_type,
                        int call_options, const char *host, const char *op,
                        gboolean *local_notify, gboolean *needs_reply,
                        gboolean *process, gboolean *needs_forward)
 {
     if (cib_op_modifies(call_type)) {
-        if (pcmk__strcase_any_of(op, CIB_OP_MASTER, CIB_OP_SLAVE, NULL)) {
+        if (pcmk__str_any_of(op, PCMK__CIB_REQUEST_PRIMARY,
+                             PCMK__CIB_REQUEST_SECONDARY, NULL)) {
             /* Always handle these locally */
             *process = TRUE;
             *needs_reply = FALSE;
@@ -533,8 +546,9 @@ parse_local_options_v2(pcmk__client_t *cib_client, int call_type,
             *needs_reply = TRUE;
             *needs_forward = TRUE;
             *process = FALSE;
-            crm_trace("%s op from %s needs to be forwarded to %s",
-                      op, cib_client->name, host ? host : "the master instance");
+            crm_trace("%s op from %s needs to be forwarded to client %s",
+                      op, pcmk__client_name(cib_client),
+                      pcmk__s(host, "the primary instance"));
             return;
         }
     }
@@ -546,23 +560,27 @@ parse_local_options_v2(pcmk__client_t *cib_client, int call_type,
     *needs_forward = FALSE;
 
     if (stand_alone) {
-        crm_trace("Processing %s op from %s (stand-alone)", op, cib_client->name);
+        crm_trace("Processing %s op from client %s (stand-alone)",
+                  op, pcmk__client_name(cib_client));
 
     } else if (host == NULL) {
-        crm_trace("Processing unaddressed %s op from %s", op, cib_client->name);
+        crm_trace("Processing unaddressed %s op from client %s",
+                  op, pcmk__client_name(cib_client));
 
     } else if (pcmk__str_eq(host, cib_our_uname, pcmk__str_casei)) {
-        crm_trace("Processing locally addressed %s op from %s", op, cib_client->name);
+        crm_trace("Processing locally addressed %s op from client %s",
+                  op, pcmk__client_name(cib_client));
 
     } else {
-        crm_trace("%s op from %s needs to be forwarded to %s", op, cib_client->name, host);
+        crm_trace("%s op from %s needs to be forwarded to client %s",
+                  op, pcmk__client_name(cib_client), host);
         *needs_forward = TRUE;
         *process = FALSE;
     }
 }
 
 static void
-parse_local_options(pcmk__client_t *cib_client, int call_type,
+parse_local_options(const pcmk__client_t *cib_client, int call_type,
                     int call_options, const char *host, const char *op,
                     gboolean *local_notify, gboolean *needs_reply,
                     gboolean *process, gboolean *needs_forward)
@@ -586,11 +604,10 @@ parse_peer_options_v1(int call_type, xmlNode * request,
     const char *delegated = NULL;
     const char *originator = crm_element_value(request, F_ORIG);
     const char *reply_to = crm_element_value(request, F_CIB_ISREPLY);
-    const char *update = crm_element_value(request, F_CIB_GLOBAL_UPDATE);
 
     gboolean is_reply = pcmk__str_eq(reply_to, cib_our_uname, pcmk__str_casei);
 
-    if (crm_is_true(update)) {
+    if (pcmk__xe_attr_is_true(request, F_CIB_GLOBAL_UPDATE)) {
         *needs_reply = FALSE;
         if (is_reply) {
             *local_notify = TRUE;
@@ -604,7 +621,7 @@ parse_peer_options_v1(int call_type, xmlNode * request,
 
     op = crm_element_value(request, F_CIB_OPERATION);
     crm_trace("Processing %s request sent by %s", op, originator);
-    if (pcmk__str_eq(op, "cib_shutdown_req", pcmk__str_casei)) {
+    if (pcmk__str_eq(op, PCMK__CIB_REQUEST_SHUTDOWN, pcmk__str_none)) {
         /* Always process these */
         *local_notify = FALSE;
         if (reply_to == NULL || is_reply) {
@@ -639,24 +656,25 @@ parse_peer_options_v1(int call_type, xmlNode * request,
         *needs_reply = TRUE;
         return TRUE;
 
-    } else if (host == NULL && cib_is_master == TRUE) {
-        crm_trace("Processing %s request sent to master instance from %s", op, originator);
+    } else if ((host == NULL) && based_is_primary) {
+        crm_trace("Processing %s request sent to primary instance from %s",
+                  op, originator);
         return TRUE;
     }
 
     delegated = crm_element_value(request, F_CIB_DELEGATED);
     if (delegated != NULL) {
-        crm_trace("Ignoring msg for master instance");
+        crm_trace("Ignoring message for primary instance");
 
     } else if (host != NULL) {
         /* this is for a specific instance and we're not it */
-        crm_trace("Ignoring msg for instance on %s", crm_str(host));
+        crm_trace("Ignoring msg for instance on %s", host);
 
-    } else if (reply_to == NULL && cib_is_master == FALSE) {
-        /* this is for the master instance and we're not it */
-        crm_trace("Ignoring reply to %s", crm_str(reply_to));
+    } else if ((reply_to == NULL) && !based_is_primary) {
+        // This is for the primary instance, and we're not it
+        crm_trace("Ignoring reply for primary instance");
 
-    } else if (pcmk__str_eq(op, "cib_shutdown_req", pcmk__str_casei)) {
+    } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_SHUTDOWN, pcmk__str_none)) {
         if (reply_to != NULL) {
             crm_debug("Processing %s from %s", op, originator);
             *needs_reply = FALSE;
@@ -684,29 +702,30 @@ parse_peer_options_v2(int call_type, xmlNode * request,
     const char *op = crm_element_value(request, F_CIB_OPERATION);
     const char *originator = crm_element_value(request, F_ORIG);
     const char *reply_to = crm_element_value(request, F_CIB_ISREPLY);
-    const char *update = crm_element_value(request, F_CIB_GLOBAL_UPDATE);
 
     gboolean is_reply = pcmk__str_eq(reply_to, cib_our_uname, pcmk__str_casei);
 
-    if(pcmk__str_eq(op, CIB_OP_REPLACE, pcmk__str_casei)) {
+    if (pcmk__str_eq(op, PCMK__CIB_REQUEST_REPLACE, pcmk__str_none)) {
         /* sync_our_cib() sets F_CIB_ISREPLY */
         if (reply_to) {
             delegated = reply_to;
         }
         goto skip_is_reply;
 
-    } else if(pcmk__str_eq(op, CIB_OP_SYNC, pcmk__str_casei)) {
+    } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_SYNC_TO_ALL,
+                            pcmk__str_none)) {
+        // Nothing to do
 
     } else if (is_reply && pcmk__str_eq(op, CRM_OP_PING, pcmk__str_casei)) {
         process_ping_reply(request);
         return FALSE;
 
-    } else if (pcmk__str_eq(op, CIB_OP_UPGRADE, pcmk__str_casei)) {
+    } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_UPGRADE, pcmk__str_none)) {
         /* Only the DC (node with the oldest software) should process
          * this operation if F_CIB_SCHEMA_MAX is unset
          *
          * If the DC is happy it will then send out another
-         * CIB_OP_UPGRADE which will tell all nodes to do the actual
+         * PCMK__CIB_REQUEST_UPGRADE which will tell all nodes to do the actual
          * upgrade.
          *
          * Except this time F_CIB_SCHEMA_MAX will be set which puts a
@@ -717,14 +736,14 @@ parse_peer_options_v2(int call_type, xmlNode * request,
 
         crm_trace("Parsing %s operation%s for %s with max=%s and upgrade_rc=%s",
                   op, (is_reply? " reply" : ""),
-                  (cib_is_master? "master" : "slave"),
+                  (based_is_primary? "primary" : "secondary"),
                   (max? max : "none"), (upgrade_rc? upgrade_rc : "none"));
 
         if (upgrade_rc != NULL) {
             // Our upgrade request was rejected by DC, notify clients of result
             crm_xml_add(request, F_CIB_RC, upgrade_rc);
 
-        } else if ((max == NULL) && cib_is_master) {
+        } else if ((max == NULL) && based_is_primary) {
             /* We are the DC, check if this upgrade is allowed */
             goto skip_is_reply;
 
@@ -737,7 +756,7 @@ parse_peer_options_v2(int call_type, xmlNode * request,
             return FALSE;
         }
 
-    } else if (crm_is_true(update)) {
+    } else if (pcmk__xe_attr_is_true(request, F_CIB_GLOBAL_UPDATE)) {
         crm_info("Detected legacy %s global update from %s", op, originator);
         send_sync_request(NULL);
         legacy_mode = TRUE;
@@ -747,7 +766,7 @@ parse_peer_options_v2(int call_type, xmlNode * request,
         crm_trace("Ignoring legacy %s reply sent from %s to local clients", op, originator);
         return FALSE;
 
-    } else if (pcmk__str_eq(op, "cib_shutdown_req", pcmk__str_casei)) {
+    } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_SHUTDOWN, pcmk__str_none)) {
         /* Legacy handling */
         crm_debug("Legacy handling of %s message from %s", op, originator);
         *local_notify = FALSE;
@@ -783,7 +802,7 @@ parse_peer_options_v2(int call_type, xmlNode * request,
 
     } else if (host != NULL) {
         /* this is for a specific instance and we're not it */
-        crm_trace("Ignoring %s operation for instance on %s", op, crm_str(host));
+        crm_trace("Ignoring %s operation for instance on %s", op, host);
         return FALSE;
 
     } else if(is_reply == FALSE && pcmk__str_eq(op, CRM_OP_PING, pcmk__str_casei)) {
@@ -816,7 +835,7 @@ parse_peer_options(int call_type, xmlNode * request,
 }
 
 static void
-forward_request(xmlNode * request, pcmk__client_t *cib_client, int call_options)
+forward_request(xmlNode *request, int call_options)
 {
     const char *op = crm_element_value(request, F_CIB_OPERATION);
     const char *host = crm_element_value(request, F_CIB_HOST);
@@ -828,7 +847,7 @@ forward_request(xmlNode * request, pcmk__client_t *cib_client, int call_options)
         send_cluster_message(crm_get_peer(0, host), crm_msg_cib, request, FALSE);
 
     } else {
-        crm_trace("Forwarding %s op to master instance", op);
+        crm_trace("Forwarding %s op to primary instance", op);
         send_cluster_message(NULL, crm_msg_cib, request, FALSE);
     }
 
@@ -874,8 +893,8 @@ send_peer_reply(xmlNode * msg, xmlNode * result_diff, const char *originator, gb
                   diff_add_admin_epoch, diff_add_epoch, diff_add_updates, digest);
 
         crm_xml_add(msg, F_CIB_ISREPLY, originator);
-        crm_xml_add(msg, F_CIB_GLOBAL_UPDATE, XML_BOOLEAN_TRUE);
-        crm_xml_add(msg, F_CIB_OPERATION, CIB_OP_APPLY_DIFF);
+        pcmk__xe_set_bool_attr(msg, F_CIB_GLOBAL_UPDATE, true);
+        crm_xml_add(msg, F_CIB_OPERATION, PCMK__CIB_REQUEST_APPLY_PATCH);
         crm_xml_add(msg, F_CIB_USER, CRM_DAEMON_USER);
 
         if (format == 1) {
@@ -900,14 +919,14 @@ send_peer_reply(xmlNode * msg, xmlNode * result_diff, const char *originator, gb
  * \internal
  * \brief Handle an IPC or CPG message containing a request
  *
- * \param[in] request            Request XML
+ * \param[in,out] request        Request XML
  * \param[in] privileged         Whether privileged commands may be run
  *                               (see cib_server_ops[] definition)
  * \param[in] cib_client         IPC client that sent request (or NULL if CPG)
  */
 static void
 cib_process_request(xmlNode *request, gboolean privileged,
-                    pcmk__client_t *cib_client)
+                    const pcmk__client_t *cib_client)
 {
     int call_type = 0;
     int call_options = 0;
@@ -917,7 +936,6 @@ cib_process_request(xmlNode *request, gboolean privileged,
     gboolean needs_reply = TRUE;    // Whether to build a reply
     gboolean local_notify = FALSE;  // Whether to notify (local) requester
     gboolean needs_forward = FALSE; // Whether to forward request somewhere else
-    gboolean global_update = crm_is_true(crm_element_value(request, F_CIB_GLOBAL_UPDATE));
 
     xmlNode *op_reply = NULL;
     xmlNode *result_diff = NULL;
@@ -945,7 +963,7 @@ cib_process_request(xmlNode *request, gboolean privileged,
         target = "local host";
 
     } else {
-        target = "master";
+        target = "primary";
     }
 
     if (cib_client == NULL) {
@@ -987,7 +1005,7 @@ cib_process_request(xmlNode *request, gboolean privileged,
         const char *section = crm_element_value(request, F_CIB_SECTION);
         int log_level = LOG_INFO;
 
-        if (pcmk__str_eq(op, CRM_OP_NOOP, pcmk__str_casei)) {
+        if (pcmk__str_eq(op, PCMK__CIB_REQUEST_NOOP, pcmk__str_none)) {
             log_level = LOG_DEBUG;
         }
 
@@ -995,11 +1013,11 @@ cib_process_request(xmlNode *request, gboolean privileged,
                    "Forwarding %s operation for section %s to %s (origin=%s/%s/%s)",
                    op,
                    section ? section : "'all'",
-                   host ? host : cib_legacy_mode() ? "master" : "all",
+                   pcmk__s(host, (cib_legacy_mode() ? "primary" : "all")),
                    originator ? originator : "local",
                    client_name, call_id);
 
-        forward_request(request, cib_client, call_options);
+        forward_request(request, call_options);
         return;
     }
 
@@ -1034,7 +1052,7 @@ cib_process_request(xmlNode *request, gboolean privileged,
         if (!is_update) {
             level = LOG_TRACE;
 
-        } else if (global_update) {
+        } else if (pcmk__xe_attr_is_true(request, F_CIB_GLOBAL_UPDATE)) {
             switch (rc) {
                 case pcmk_ok:
                     level = LOG_INFO;
@@ -1080,8 +1098,8 @@ cib_process_request(xmlNode *request, gboolean privileged,
                   local_notify?" with local notification":"");
 
     } else if (!needs_reply || stand_alone) {
-        // This was a non-originating slave update
-        crm_trace("Completed slave update");
+        // This was a non-originating secondary update
+        crm_trace("Completed update as secondary");
 
     } else if (cib_legacy_mode() &&
                rc == pcmk_ok && result_diff != NULL && !(call_options & cib_inhibit_bcast)) {
@@ -1147,6 +1165,23 @@ cib_process_request(xmlNode *request, gboolean privileged,
     return;
 }
 
+static char *
+calculate_section_digest(const char *xpath, xmlNode * xml_obj)
+{
+    xmlNode *xml_section = NULL;
+
+    if (xml_obj == NULL) {
+        return NULL;
+    }
+
+    xml_section = get_xpath_object(xpath, xml_obj, LOG_TRACE);
+    if (xml_section == NULL) {
+        return NULL;
+    }
+    return calculate_xml_versioned_digest(xml_section, FALSE, TRUE, CRM_FEATURE_SET); 
+
+}
+
 static int
 cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gboolean privileged)
 {
@@ -1172,6 +1207,13 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
 
     static mainloop_timer_t *digest_timer = NULL;
 
+    char *current_nodes_digest = NULL;
+    char *current_alerts_digest = NULL;
+    char *current_status_digest = NULL;
+    uint32_t change_section = cib_change_section_nodes
+                              |cib_change_section_alerts
+                              |cib_change_section_status;
+
     CRM_ASSERT(cib_status == pcmk_ok);
 
     if(digest_timer == NULL) {
@@ -1210,8 +1252,7 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
     }
 
     /* Handle a valid write action */
-    global_update = crm_is_true(crm_element_value(request, F_CIB_GLOBAL_UPDATE));
-    if (global_update) {
+    if (pcmk__xe_attr_is_true(request, F_CIB_GLOBAL_UPDATE)) {
         /* legacy code */
         manage_counters = FALSE;
         cib__set_call_options(call_options, "call", cib_force_diff);
@@ -1237,6 +1278,14 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
             cib__clear_call_options(call_options, "call", cib_zero_copy);
         }
 
+        /* Calculate the hash value of the section before the change. */
+        if (pcmk__str_eq(PCMK__CIB_REQUEST_REPLACE, op, pcmk__str_none)) {
+            current_nodes_digest = calculate_section_digest("//" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_NODES, current_cib);
+            current_alerts_digest = calculate_section_digest("//" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_ALERTS, current_cib);
+            current_status_digest = calculate_section_digest("//" XML_TAG_CIB "/" XML_CIB_TAG_STATUS, current_cib);
+            crm_trace("current-digest %s:%s:%s", current_nodes_digest, current_alerts_digest, current_status_digest);
+        }
+
         /* result_cib must not be modified after cib_perform_op() returns */
         rc = cib_perform_op(op, call_options, cib_op_func(call_type), FALSE,
                             section, request, input, manage_counters, &config_changed,
@@ -1259,7 +1308,7 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
         /* Always write to disk for replace ops,
          * this also negates the need to detect ordering changes
          */
-        if (pcmk__str_eq(CIB_OP_REPLACE, op, pcmk__str_none)) {
+        if (pcmk__str_eq(PCMK__CIB_REQUEST_REPLACE, op, pcmk__str_none)) {
             config_changed = TRUE;
         }
     }
@@ -1280,24 +1329,53 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
             cib_read_config(config_hash, result_cib);
         }
 
-        if (pcmk__str_eq(CIB_OP_REPLACE, op, pcmk__str_none)) {
-            if (section == NULL) {
-                send_r_notify = TRUE;
+        if (pcmk__str_eq(PCMK__CIB_REQUEST_REPLACE, op, pcmk__str_none)) {
+            char *result_nodes_digest = NULL;
+            char *result_alerts_digest = NULL;
+            char *result_status_digest = NULL;
 
-            } else if (pcmk__str_eq(section, XML_TAG_CIB, pcmk__str_casei)) {
-                send_r_notify = TRUE;
+            /* Calculate the hash value of the changed section. */
+            result_nodes_digest = calculate_section_digest("//" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_NODES, result_cib);
+            result_alerts_digest = calculate_section_digest("//" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_ALERTS, result_cib);
+            result_status_digest = calculate_section_digest("//" XML_TAG_CIB "/" XML_CIB_TAG_STATUS, result_cib);
+            crm_trace("result-digest %s:%s:%s", result_nodes_digest, result_alerts_digest, result_status_digest);
 
-            } else if (pcmk__str_eq(section, XML_CIB_TAG_NODES, pcmk__str_casei)) {
-                send_r_notify = TRUE;
-
-            } else if (pcmk__str_eq(section, XML_CIB_TAG_STATUS, pcmk__str_casei)) {
-                send_r_notify = TRUE;
-
-            } else if (pcmk__str_eq(section, XML_CIB_TAG_CONFIGURATION, pcmk__str_casei)) {
-                send_r_notify = TRUE;
+            if (pcmk__str_eq(current_nodes_digest, result_nodes_digest,
+                             pcmk__str_none)) {
+                change_section =
+                    pcmk__clear_flags_as(__func__, __LINE__, LOG_TRACE,
+                                         "CIB change section",
+                                         "change_section", change_section,
+                                         cib_change_section_nodes, "nodes");
             }
 
-        } else if (pcmk__str_eq(CIB_OP_ERASE, op, pcmk__str_none)) {
+            if (pcmk__str_eq(current_alerts_digest, result_alerts_digest,
+                             pcmk__str_none)) {
+                change_section =
+                    pcmk__clear_flags_as(__func__, __LINE__, LOG_TRACE,
+                                         "CIB change section",
+                                         "change_section", change_section,
+                                         cib_change_section_alerts, "alerts");
+            }
+
+            if (pcmk__str_eq(current_status_digest, result_status_digest,
+                             pcmk__str_none)) {
+                change_section =
+                    pcmk__clear_flags_as(__func__, __LINE__, LOG_TRACE,
+                                         "CIB change section",
+                                         "change_section", change_section,
+                                         cib_change_section_status, "status");
+            }
+
+            if (change_section != cib_change_section_none) {
+                send_r_notify = TRUE;
+            }
+            
+            free(result_nodes_digest);
+            free(result_alerts_digest);
+            free(result_status_digest);
+
+        } else if (pcmk__str_eq(PCMK__CIB_REQUEST_ERASE, op, pcmk__str_none)) {
             send_r_notify = TRUE;
         }
 
@@ -1334,7 +1412,7 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
     if (send_r_notify) {
         const char *origin = crm_element_value(request, F_ORIG);
 
-        cib_replace_notify(origin, the_cib, rc, *cib_diff);
+        cib_replace_notify(origin, the_cib, rc, *cib_diff, change_section);
     }
 
     xml_log_patchset(LOG_TRACE, "cib:diff", *cib_diff);
@@ -1369,6 +1447,10 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
         cib_op_cleanup(call_type, call_options, &input, &output);
     }
 
+    free(current_nodes_digest);
+    free(current_alerts_digest);
+    free(current_status_digest);
+
     crm_trace("done");
     return rc;
 }
@@ -1397,7 +1479,7 @@ cib_peer_callback(xmlNode * msg, void *private_data)
         crm_xml_add(msg, F_CIB_CLIENTNAME, originator);
     }
 
-    /* crm_log_xml_trace("Peer[inbound]", msg); */
+    /* crm_log_xml_trace(msg, "Peer[inbound]"); */
     cib_process_request(msg, TRUE, NULL);
     return;
 
@@ -1423,7 +1505,8 @@ disconnect_remote_client(gpointer key, gpointer value, gpointer user_data)
 {
     pcmk__client_t *a_client = value;
 
-    crm_err("Disconnecting %s... Not implemented", crm_str(a_client->name));
+    crm_err("Can't disconnect client %s: Not implemented",
+            pcmk__client_name(a_client));
 }
 
 void
@@ -1508,7 +1591,7 @@ initiate_exit(void)
 
     leaving = create_xml_node(NULL, "exit-notification");
     crm_xml_add(leaving, F_TYPE, "cib");
-    crm_xml_add(leaving, F_CIB_OPERATION, "cib_shutdown_req");
+    crm_xml_add(leaving, F_CIB_OPERATION, PCMK__CIB_REQUEST_SHUTDOWN);
 
     send_cluster_message(NULL, crm_msg_cib, leaving, TRUE);
     free_xml(leaving);
