@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,6 +13,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <inttypes.h>           // PRIx64
 
 #include <stdlib.h>
 #include <errno.h>
@@ -47,6 +48,7 @@ cib_notify_send_one(gpointer key, gpointer value, gpointer user_data)
 {
     const char *type = NULL;
     gboolean do_send = FALSE;
+    int rc = pcmk_rc_ok;
 
     pcmk__client_t *client = value;
     struct cib_notification_s *update = user_data;
@@ -85,22 +87,26 @@ cib_notify_send_one(gpointer key, gpointer value, gpointer user_data)
     if (do_send) {
         switch (PCMK__CLIENT_TYPE(client)) {
             case pcmk__client_ipc:
-                if (pcmk__ipc_send_iov(client, update->iov,
-                                       crm_ipc_server_event) != pcmk_rc_ok) {
-                    crm_warn("Notification of client %s/%s failed", client->name, client->id);
+                rc = pcmk__ipc_send_iov(client, update->iov,
+                                        crm_ipc_server_event);
+                if (rc != pcmk_rc_ok) {
+                    crm_warn("Could not notify client %s: %s " CRM_XS " id=%s",
+                             pcmk__client_name(client), pcmk_rc_str(rc),
+                             client->id);
                 }
                 break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
             case pcmk__client_tls:
 #endif
             case pcmk__client_tcp:
-                crm_debug("Sent %s notification to client %s/%s", type, client->name, client->id);
+                crm_debug("Sent %s notification to client %s (id %s)",
+                          type, pcmk__client_name(client), client->id);
                 pcmk__remote_send_xml(client->remote, update->msg);
                 break;
             default:
-                crm_err("Unknown transport for %s " CRM_XS " flags=0x%llx",
-                        pcmk__client_name(client),
-                        (unsigned long long) client->flags);
+                crm_err("Unknown transport for client %s "
+                        CRM_XS " flags=%#016" PRIx64,
+                        pcmk__client_name(client), client->flags);
         }
     }
 }
@@ -228,7 +234,8 @@ attach_cib_generation(xmlNode * msg, const char *field, xmlNode * a_cib)
 }
 
 void
-cib_replace_notify(const char *origin, xmlNode * update, int result, xmlNode * diff)
+cib_replace_notify(const char *origin, xmlNode *update, int result,
+                   xmlNode *diff, uint32_t change_section)
 {
     xmlNode *replace_msg = NULL;
 
@@ -254,17 +261,21 @@ cib_replace_notify(const char *origin, xmlNode * update, int result, xmlNode * d
     if (add_updates != del_updates) {
         crm_info("Replaced: %d.%d.%d -> %d.%d.%d from %s",
                  del_admin_epoch, del_epoch, del_updates,
-                 add_admin_epoch, add_epoch, add_updates, crm_str(origin));
+                 add_admin_epoch, add_epoch, add_updates,
+                 pcmk__s(origin, "unspecified peer"));
     } else if (diff != NULL) {
         crm_info("Local-only Replace: %d.%d.%d from %s",
-                 add_admin_epoch, add_epoch, add_updates, crm_str(origin));
+                 add_admin_epoch, add_epoch, add_updates,
+                 pcmk__s(origin, "unspecified peer"));
     }
 
     replace_msg = create_xml_node(NULL, "notify-replace");
     crm_xml_add(replace_msg, F_TYPE, T_CIB_NOTIFY);
     crm_xml_add(replace_msg, F_SUBTYPE, T_CIB_REPLACE_NOTIFY);
-    crm_xml_add(replace_msg, F_CIB_OPERATION, CIB_OP_REPLACE);
+    crm_xml_add(replace_msg, F_CIB_OPERATION, PCMK__CIB_REQUEST_REPLACE);
     crm_xml_add_int(replace_msg, F_CIB_RC, result);
+    crm_xml_add_ll(replace_msg, F_CIB_CHANGE_SECTION,
+                   (long long) change_section);
     attach_cib_generation(replace_msg, "cib-replace-generation", update);
 
     crm_log_xml_trace(replace_msg, "CIB Replaced");
