@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,6 +15,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <inttypes.h>           // PRIx64
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
@@ -37,7 +38,6 @@
 /* #undef HAVE_GNUTLS_GNUTLS_H */
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
-#  undef KEYFILE
 #  include <gnutls/gnutls.h>
 #endif
 
@@ -264,7 +264,7 @@ remote_auth_timeout_cb(gpointer data)
 
     client->remote->auth_timeout = 0;
 
-    if (client->remote->authenticated == TRUE) {
+    if (pcmk_is_set(client->flags, pcmk__client_authenticated)) {
         return FALSE;
     }
 
@@ -360,7 +360,8 @@ cib_remote_connection_destroy(gpointer user_data)
         return;
     }
 
-    crm_trace("Cleaning up after client disconnect: %s/%s", crm_str(client->name), client->id);
+    crm_trace("Cleaning up after client %s disconnect",
+              pcmk__client_name(client));
 
     num_clients--;
     crm_trace("Num unfree'd clients: %d", num_clients);
@@ -375,7 +376,8 @@ cib_remote_connection_destroy(gpointer user_data)
                 void *sock_ptr = gnutls_transport_get_ptr(*client->remote->tls_session);
 
                 csock = GPOINTER_TO_INT(sock_ptr);
-                if (client->remote->tls_handshake_complete) {
+                if (pcmk_is_set(client->flags,
+                                pcmk__client_tls_handshake_complete)) {
                     gnutls_bye(*client->remote->tls_session, GNUTLS_SHUT_WR);
                 }
                 gnutls_deinit(*client->remote->tls_session);
@@ -385,9 +387,9 @@ cib_remote_connection_destroy(gpointer user_data)
             break;
 #endif
         default:
-            crm_warn("Unknown transport for %s " CRM_XS " flags=0x%llx",
-                     pcmk__client_name(client),
-                     (unsigned long long) client->flags);
+            crm_warn("Unknown transport for client %s "
+                     CRM_XS " flags=%#016" PRIx64,
+                     pcmk__client_name(client), client->flags);
     }
 
     if (csock > 0) {
@@ -443,9 +445,7 @@ cib_handle_remote_msg(pcmk__client_t *client, xmlNode *command)
     crm_xml_add(command, F_TYPE, T_CIB);
     crm_xml_add(command, F_CIB_CLIENTID, client->id);
     crm_xml_add(command, F_CIB_CLIENTNAME, client->name);
-#if ENABLE_ACL
     crm_xml_add(command, F_CIB_USER, client->user);
-#endif
 
     if (crm_element_value(command, F_CIB_CALLID) == NULL) {
         char *call_uuid = crm_generate_uuid();
@@ -469,15 +469,19 @@ cib_remote_msg(gpointer data)
     xmlNode *command = NULL;
     pcmk__client_t *client = data;
     int rc;
-    int timeout = client->remote->authenticated ? -1 : 1000;
+    int timeout = 1000;
 
-    crm_trace("Remote %s message received for %s",
+    if (pcmk_is_set(client->flags, pcmk__client_authenticated)) {
+        timeout = -1;
+    }
+
+    crm_trace("Remote %s message received for client %s",
               pcmk__client_type_str(PCMK__CLIENT_TYPE(client)),
               pcmk__client_name(client));
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
     if ((PCMK__CLIENT_TYPE(client) == pcmk__client_tls)
-        && !(client->remote->tls_handshake_complete)) {
+        && !pcmk_is_set(client->flags, pcmk__client_tls_handshake_complete)) {
 
         int rc = pcmk__read_handshake_data(client);
 
@@ -491,7 +495,7 @@ cib_remote_msg(gpointer data)
         }
 
         crm_debug("TLS handshake with remote CIB client completed");
-        client->remote->tls_handshake_complete = TRUE;
+        pcmk__set_client_flags(client, pcmk__client_tls_handshake_complete);
         if (client->remote->auth_timeout) {
             g_source_remove(client->remote->auth_timeout);
         }
@@ -507,12 +511,10 @@ cib_remote_msg(gpointer data)
     rc = pcmk__read_remote_message(client->remote, timeout);
 
     /* must pass auth before we will process anything else */
-    if (client->remote->authenticated == FALSE) {
+    if (!pcmk_is_set(client->flags, pcmk__client_authenticated)) {
         xmlNode *reg;
-
-#if ENABLE_ACL
         const char *user = NULL;
-#endif
+
         command = pcmk__remote_message_xml(client->remote);
         if (cib_remote_auth(command) == FALSE) {
             free_xml(command);
@@ -520,17 +522,15 @@ cib_remote_msg(gpointer data)
         }
 
         crm_notice("Remote CIB client connection accepted");
-        client->remote->authenticated = TRUE;
+        pcmk__set_client_flags(client, pcmk__client_authenticated);
         g_source_remove(client->remote->auth_timeout);
         client->remote->auth_timeout = 0;
         client->name = crm_element_value_copy(command, "name");
 
-#if ENABLE_ACL
         user = crm_element_value(command, "user");
         if (user) {
             client->user = strdup(user);
         }
-#endif
 
         /* send ACK */
         reg = create_xml_node(NULL, "cib_result");
