@@ -244,7 +244,7 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
         xmlNode *update = NULL;
         int flags = node_update_peer;
         int alive = node_alive(node);
-        crm_action_t *down = match_down_event(node->uuid);
+        pcmk__graph_action_t *down = match_down_event(node->uuid);
 
         crm_trace("Alive=%d, appeared=%d, down=%d",
                   alive, appeared, (down? down->id : -1));
@@ -289,13 +289,19 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
             }
 
         } else if (appeared == FALSE) {
-            crm_warn("Stonith/shutdown of node %s was not expected",
-                     node->uname);
+            if (transition_graph == NULL || transition_graph->id == -1) {
+                crm_info("Stonith/shutdown of node %s is unknown to the "
+                         "current DC", node->uname);
+            } else {
+                crm_warn("Stonith/shutdown of node %s was not expected",
+                         node->uname);
+            }
             if (!is_remote) {
                 crm_update_peer_join(__func__, node, crm_join_none);
                 check_join_state(fsa_state, __func__);
             }
-            abort_transition(INFINITY, tg_restart, "Node failure", NULL);
+            abort_transition(INFINITY, pcmk__graph_restart, "Node failure",
+                             NULL);
             fail_incompletable_actions(transition_graph, node->uuid);
 
         } else {
@@ -311,7 +317,7 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
 
             /* Trigger resource placement on newly integrated nodes */
             if (appeared) {
-                abort_transition(INFINITY, tg_restart,
+                abort_transition(INFINITY, pcmk__graph_restart,
                                  "Pacemaker Remote node integrated", NULL);
             }
         }
