@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -161,7 +161,8 @@ cib_fencing_updated(xmlNode *msg, int call_id, int rc, xmlNode *output,
         crm_err("Fencing update %d for %s: failed - %s (%d)",
                 call_id, (char *)user_data, pcmk_strerror(rc), rc);
         crm_log_xml_warn(msg, "Failed update");
-        abort_transition(INFINITY, tg_shutdown, "CIB update failed", NULL);
+        abort_transition(INFINITY, pcmk__graph_shutdown, "CIB update failed",
+                         NULL);
 
     } else {
         crm_info("Fencing update %d for %s: complete", call_id, (char *)user_data);
@@ -169,7 +170,8 @@ cib_fencing_updated(xmlNode *msg, int call_id, int rc, xmlNode *output,
 }
 
 static void
-send_stonith_update(crm_action_t *action, const char *target, const char *uuid)
+send_stonith_update(pcmk__graph_action_t *action, const char *target,
+                    const char *uuid)
 {
     int rc = pcmk_ok;
     crm_node_t *peer = NULL;
@@ -245,14 +247,14 @@ send_stonith_update(crm_action_t *action, const char *target, const char *uuid)
  * \param[in] reason  Log this stonith action XML as abort reason (or NULL)
  */
 static void
-abort_for_stonith_failure(enum transition_action abort_action,
-                          const char *target, xmlNode *reason)
+abort_for_stonith_failure(enum pcmk__graph_next abort_action,
+                          const char *target, const xmlNode *reason)
 {
     /* If stonith repeatedly fails, we eventually give up on starting a new
      * transition for that reason.
      */
-    if ((abort_action != tg_stop) && too_many_st_failures(target)) {
-        abort_action = tg_stop;
+    if ((abort_action != pcmk__graph_wait) && too_many_st_failures(target)) {
+        abort_action = pcmk__graph_wait;
     }
     abort_transition(INFINITY, abort_action, "Stonith failed", reason);
 }
@@ -308,7 +310,7 @@ remove_stonith_cleanup(const char *target)
  * \brief Purge all entries from the stonith cleanup list
  */
 void
-purge_stonith_cleanup()
+purge_stonith_cleanup(void)
 {
     if (stonith_cleanup_list) {
         GList *iter = NULL;
@@ -329,7 +331,7 @@ purge_stonith_cleanup()
  * \brief Send stonith updates for all entries in cleanup list, then purge it
  */
 void
-execute_stonith_cleanup()
+execute_stonith_cleanup(void)
 {
     GList *iter;
 
@@ -359,7 +361,7 @@ static crm_trigger_t *stonith_reconnect = NULL;
 static char *te_client_id = NULL;
 
 static gboolean
-fail_incompletable_stonith(crm_graph_t *graph)
+fail_incompletable_stonith(pcmk__graph_t *graph)
 {
     GList *lpc = NULL;
     const char *task = NULL;
@@ -371,22 +373,23 @@ fail_incompletable_stonith(crm_graph_t *graph)
 
     for (lpc = graph->synapses; lpc != NULL; lpc = lpc->next) {
         GList *lpc2 = NULL;
-        synapse_t *synapse = (synapse_t *) lpc->data;
+        pcmk__graph_synapse_t *synapse = (pcmk__graph_synapse_t *) lpc->data;
 
         if (pcmk_is_set(synapse->flags, pcmk__synapse_confirmed)) {
             continue;
         }
 
         for (lpc2 = synapse->actions; lpc2 != NULL; lpc2 = lpc2->next) {
-            crm_action_t *action = (crm_action_t *) lpc2->data;
+            pcmk__graph_action_t *action = (pcmk__graph_action_t *) lpc2->data;
 
-            if (action->type != action_type_crm || pcmk_is_set(action->flags, pcmk__graph_action_confirmed)) {
+            if ((action->type != pcmk__cluster_graph_action)
+                || pcmk_is_set(action->flags, pcmk__graph_action_confirmed)) {
                 continue;
             }
 
             task = crm_element_value(action->xml, XML_LRM_ATTR_TASK);
             if (task && pcmk__str_eq(task, CRM_OP_FENCE, pcmk__str_casei)) {
-                crm__set_graph_action_flags(action, pcmk__graph_action_failed);
+                pcmk__set_graph_action_flags(action, pcmk__graph_action_failed);
                 last_action = action->xml;
                 pcmk__update_graph(graph, action);
                 crm_notice("Failing action %d (%s): fencer terminated",
@@ -397,7 +400,7 @@ fail_incompletable_stonith(crm_graph_t *graph)
 
     if (last_action != NULL) {
         crm_warn("Fencer failure resulted in unrunnable actions");
-        abort_for_stonith_failure(tg_restart, NULL, last_action);
+        abort_for_stonith_failure(pcmk__graph_restart, NULL, last_action);
         return TRUE;
     }
 
@@ -424,9 +427,7 @@ tengine_stonith_connection_destroy(stonith_t *st, stonith_event_t *e)
         if (stonith_api->state != stonith_disconnected) {
             stonith_api->cmds->disconnect(st);
         }
-        stonith_api->cmds->remove_notification(stonith_api, T_STONITH_NOTIFY_DISCONNECT);
-        stonith_api->cmds->remove_notification(stonith_api, T_STONITH_NOTIFY_FENCE);
-        stonith_api->cmds->remove_notification(stonith_api, T_STONITH_NOTIFY_HISTORY_SYNCED);
+        stonith_api->cmds->remove_notification(stonith_api, NULL);
     }
 
     if (AM_I_DC) {
@@ -435,39 +436,69 @@ tengine_stonith_connection_destroy(stonith_t *st, stonith_event_t *e)
     }
 }
 
+/*!
+ * \internal
+ * \brief Handle an event notification from the fencing API
+ *
+ * \param[in] st     Fencing API connection (ignored)
+ * \param[in] event  Fencing API event notification
+ */
 static void
-tengine_stonith_notify(stonith_t *st, stonith_event_t *st_event)
+handle_fence_notification(stonith_t *st, stonith_event_t *event)
 {
+    bool succeeded = true;
+    const char *executioner = "the cluster";
+    const char *client = "a client";
+    const char *reason = NULL;
+    int exec_status;
+
     if (te_client_id == NULL) {
         te_client_id = crm_strdup_printf("%s.%lu", crm_system_name,
                                          (unsigned long) getpid());
     }
 
-    if (st_event == NULL) {
+    if (event == NULL) {
         crm_err("Notify data not found");
         return;
     }
 
-    crmd_alert_fencing_op(st_event);
+    if (event->executioner != NULL) {
+        executioner = event->executioner;
+    }
+    if (event->client_origin != NULL) {
+        client = event->client_origin;
+    }
 
-    if ((st_event->result == pcmk_ok) && pcmk__str_eq("on", st_event->action, pcmk__str_casei)) {
-        crm_notice("%s was successfully unfenced by %s (at the request of %s)",
-                   st_event->target,
-                   st_event->executioner? st_event->executioner : "<anyone>",
-                   st_event->origin);
-                /* TODO: Hook up st_event->device */
+    exec_status = stonith__event_execution_status(event);
+    if ((stonith__event_exit_status(event) != CRM_EX_OK)
+        || (exec_status != PCMK_EXEC_DONE)) {
+        succeeded = false;
+        if (exec_status == PCMK_EXEC_DONE) {
+            exec_status = PCMK_EXEC_ERROR;
+        }
+    }
+    reason = stonith__event_exit_reason(event);
+
+    crmd_alert_fencing_op(event);
+
+    if (pcmk__str_eq("on", event->action, pcmk__str_none)) {
+        // Unfencing doesn't need special handling, just a log message
+        if (succeeded) {
+            crm_notice("%s was unfenced by %s at the request of %s@%s",
+                       event->target, executioner, client, event->origin);
+        } else {
+            crm_err("Unfencing of %s by %s failed (%s%s%s) with exit status %d",
+                    event->target, executioner,
+                    pcmk_exec_status_str(exec_status),
+                    ((reason == NULL)? "" : ": "),
+                    ((reason == NULL)? "" : reason),
+                    stonith__event_exit_status(event));
+        }
         return;
+    }
 
-    } else if (pcmk__str_eq("on", st_event->action, pcmk__str_casei)) {
-        crm_err("Unfencing of %s by %s failed: %s (%d)",
-                st_event->target,
-                st_event->executioner? st_event->executioner : "<anyone>",
-                pcmk_strerror(st_event->result), st_event->result);
-        return;
-
-    } else if ((st_event->result == pcmk_ok)
-               && pcmk__str_eq(st_event->target, fsa_our_uname, pcmk__str_none)) {
-
+    if (succeeded
+        && pcmk__str_eq(event->target, fsa_our_uname, pcmk__str_casei)) {
         /* We were notified of our own fencing. Most likely, either fencing was
          * misconfigured, or fabric fencing that doesn't cut cluster
          * communication is in use.
@@ -478,44 +509,41 @@ tengine_stonith_notify(stonith_t *st, stonith_event_t *st_event)
          * our subsequent election votes as "not part of our cluster".
          */
         crm_crit("We were allegedly just fenced by %s for %s!",
-                 st_event->executioner? st_event->executioner : "the cluster",
-                 st_event->origin); /* Dumps blackbox if enabled */
+                 executioner, event->origin); // Dumps blackbox if enabled
         if (fence_reaction_panic) {
             pcmk__panic(__func__);
         } else {
             crm_exit(CRM_EX_FATAL);
         }
-        return;
+        return; // Should never get here
     }
 
-    /* Update the count of stonith failures for this target, in case we become
+    /* Update the count of fencing failures for this target, in case we become
      * DC later. The current DC has already updated its fail count in
      * tengine_stonith_callback().
      */
-    if (!AM_I_DC && pcmk__str_eq(st_event->operation, T_STONITH_NOTIFY_FENCE, pcmk__str_casei)) {
-        if (st_event->result == pcmk_ok) {
-            st_fail_count_reset(st_event->target);
+    if (!AM_I_DC) {
+        if (succeeded) {
+            st_fail_count_reset(event->target);
         } else {
-            st_fail_count_increment(st_event->target);
+            st_fail_count_increment(event->target);
         }
     }
 
-    crm_notice("Peer %s was%s terminated (%s) by %s on behalf of %s: %s "
-               CRM_XS " initiator=%s ref=%s",
-               st_event->target, st_event->result == pcmk_ok ? "" : " not",
-               st_event->action,
-               st_event->executioner ? st_event->executioner : "<anyone>",
-               (st_event->client_origin? st_event->client_origin : "<unknown>"),
-               pcmk_strerror(st_event->result),
-               st_event->origin, st_event->id);
+    crm_notice("Peer %s was%s terminated (%s) by %s on behalf of %s@%s: "
+               "%s%s%s%s " CRM_XS " event=%s",
+               event->target, (succeeded? "" : " not"),
+               event->action, executioner, client, event->origin,
+               (succeeded? "OK" : pcmk_exec_status_str(exec_status)),
+               ((reason == NULL)? "" : " ("),
+               ((reason == NULL)? "" : reason),
+               ((reason == NULL)? "" : ")"),
+               event->id);
 
-    if (st_event->result == pcmk_ok) {
-        crm_node_t *peer = pcmk__search_known_node_cache(0, st_event->target,
+    if (succeeded) {
+        crm_node_t *peer = pcmk__search_known_node_cache(0, event->target,
                                                          CRM_GET_PEER_ANY);
         const char *uuid = NULL;
-        gboolean we_are_executioner = pcmk__str_eq(st_event->executioner,
-                                                   fsa_our_uname,
-                                                   pcmk__str_casei);
 
         if (peer == NULL) {
             return;
@@ -523,10 +551,9 @@ tengine_stonith_notify(stonith_t *st, stonith_event_t *st_event)
 
         uuid = crm_peer_uuid(peer);
 
-        crm_trace("target=%s dc=%s", st_event->target, fsa_our_dc);
-        if(AM_I_DC) {
+        if (AM_I_DC) {
             /* The DC always sends updates */
-            send_stonith_update(NULL, st_event->target, uuid);
+            send_stonith_update(NULL, event->target, uuid);
 
             /* @TODO Ideally, at this point, we'd check whether the fenced node
              * hosted any guest nodes, and call remote_node_down() for them.
@@ -536,31 +563,33 @@ tengine_stonith_notify(stonith_t *st, stonith_event_t *st_event)
              * on the scheduler creating fence pseudo-events for the guests.
              */
 
-            if (st_event->client_origin
-                && !pcmk__str_eq(st_event->client_origin, te_client_id, pcmk__str_casei)) {
-
-                /* Abort the current transition graph if it wasn't us
-                 * that invoked stonith to fence someone
+            if (!pcmk__str_eq(client, te_client_id, pcmk__str_casei)) {
+                /* Abort the current transition if it wasn't the cluster that
+                 * initiated fencing.
                  */
-                crm_info("External fencing operation from %s fenced %s", st_event->client_origin, st_event->target);
-                abort_transition(INFINITY, tg_restart, "External Fencing Operation", NULL);
+                crm_info("External fencing operation from %s fenced %s",
+                         client, event->target);
+                abort_transition(INFINITY, pcmk__graph_restart,
+                                 "External Fencing Operation", NULL);
             }
 
             /* Assume it was our leader if we don't currently have one */
-        } else if (pcmk__str_eq(fsa_our_dc, st_event->target, pcmk__str_null_matches | pcmk__str_casei)
+        } else if (pcmk__str_eq(fsa_our_dc, event->target,
+                                pcmk__str_null_matches|pcmk__str_casei)
                    && !pcmk_is_set(peer->flags, crm_remote_node)) {
 
             crm_notice("Fencing target %s %s our leader",
-                       st_event->target, (fsa_our_dc? "was" : "may have been"));
+                       event->target, (fsa_our_dc? "was" : "may have been"));
 
             /* Given the CIB resyncing that occurs around elections,
              * have one node update the CIB now and, if the new DC is different,
              * have them do so too after the election
              */
-            if (we_are_executioner) {
-                send_stonith_update(NULL, st_event->target, uuid);
+            if (pcmk__str_eq(event->executioner, fsa_our_uname,
+                             pcmk__str_casei)) {
+                send_stonith_update(NULL, event->target, uuid);
             }
-            add_stonith_cleanup(st_event->target);
+            add_stonith_cleanup(event->target);
         }
 
         /* If the target is a remote node, and we host its connection,
@@ -569,7 +598,7 @@ tengine_stonith_notify(stonith_t *st, stonith_event_t *st_event)
          * so the failure might not otherwise be detected until the next poke.
          */
         if (pcmk_is_set(peer->flags, crm_remote_node)) {
-            remote_ra_fail(st_event->target);
+            remote_ra_fail(event->target);
         }
 
         crmd_peer_down(peer, TRUE);
@@ -632,7 +661,7 @@ te_connect_stonith(gpointer user_data)
                                                  tengine_stonith_connection_destroy);
         stonith_api->cmds->register_notification(stonith_api,
                                                  T_STONITH_NOTIFY_FENCE,
-                                                 tengine_stonith_notify);
+                                                 handle_fence_notification);
         stonith_api->cmds->register_notification(stonith_api,
                                                  T_STONITH_NOTIFY_HISTORY_SYNCED,
                                                  tengine_stonith_history_synced);
@@ -648,7 +677,7 @@ te_connect_stonith(gpointer user_data)
     \brief Schedule fencer connection attempt in main loop
 */
 void
-controld_trigger_fencer_connect()
+controld_trigger_fencer_connect(void)
 {
     if (stonith_reconnect == NULL) {
         stonith_reconnect = mainloop_add_trigger(G_PRIORITY_LOW,
@@ -669,9 +698,7 @@ controld_disconnect_fencer(bool destroy)
         if (stonith_api->state != stonith_disconnected) {
             stonith_api->cmds->disconnect(stonith_api);
         }
-        stonith_api->cmds->remove_notification(stonith_api, T_STONITH_NOTIFY_DISCONNECT);
-        stonith_api->cmds->remove_notification(stonith_api, T_STONITH_NOTIFY_FENCE);
-        stonith_api->cmds->remove_notification(stonith_api, T_STONITH_NOTIFY_HISTORY_SYNCED);
+        stonith_api->cmds->remove_notification(stonith_api, NULL);
     }
     if (destroy) {
         if (stonith_api) {
@@ -713,46 +740,65 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
     char *uuid = NULL;
     int stonith_id = -1;
     int transition_id = -1;
-    crm_action_t *action = NULL;
-    int call_id = data->call_id;
-    int rc = data->rc;
-    char *userdata = data->userdata;
+    pcmk__graph_action_t *action = NULL;
+    const char *target = NULL;
 
-    CRM_CHECK(userdata != NULL, return);
-    crm_notice("Stonith operation %d/%s: %s (%d)", call_id, (char *)userdata,
-               pcmk_strerror(rc), rc);
-
-    if (AM_I_DC == FALSE) {
+    if ((data == NULL) || (data->userdata == NULL)) {
+        crm_err("Ignoring fence operation %d result: "
+                "No transition key given (bug?)",
+                ((data == NULL)? -1 : data->call_id));
         return;
     }
 
-    /* crm_info("call=%d, optype=%d, node_name=%s, result=%d, node_list=%s, action=%s", */
-    /*       op->call_id, op->optype, op->node_name, op->op_result, */
-    /*       (char *)op->node_list, op->private_data); */
+    if (!AM_I_DC) {
+        const char *reason = stonith__exit_reason(data);
 
-    /* filter out old STONITH actions */
-    CRM_CHECK(decode_transition_key(userdata, &uuid, &transition_id, &stonith_id, NULL),
+        if (reason == NULL) {
+           reason = pcmk_exec_status_str(stonith__execution_status(data));
+        }
+        crm_notice("Result of fence operation %d: %d (%s) " CRM_XS " key=%s",
+                   data->call_id, stonith__exit_status(data), reason,
+                   (const char *) data->userdata);
+        return;
+    }
+
+    CRM_CHECK(decode_transition_key(data->userdata, &uuid, &transition_id,
+                                    &stonith_id, NULL),
               goto bail);
 
-    if (transition_graph->complete || stonith_id < 0 || !pcmk__str_eq(uuid, te_uuid, pcmk__str_casei)
-        || transition_graph->id != transition_id) {
-        crm_info("Ignoring STONITH action initiated outside of the current transition");
+    if (transition_graph->complete || (stonith_id < 0)
+        || !pcmk__str_eq(uuid, te_uuid, pcmk__str_none)
+        || (transition_graph->id != transition_id)) {
+        crm_info("Ignoring fence operation %d result: "
+                 "Not from current transition " CRM_XS
+                 " complete=%s action=%d uuid=%s (vs %s) transition=%d (vs %d)",
+                 data->call_id, pcmk__btoa(transition_graph->complete),
+                 stonith_id, uuid, te_uuid, transition_id, transition_graph->id);
         goto bail;
     }
 
     action = controld_get_action(stonith_id);
     if (action == NULL) {
-        crm_err("Stonith action not matched");
+        crm_err("Ignoring fence operation %d result: "
+                "Action %d not found in transition graph (bug?) "
+                CRM_XS " uuid=%s transition=%d",
+                data->call_id, stonith_id, uuid, transition_id);
         goto bail;
     }
 
-    stop_te_timer(action->timer);
-    if (rc == pcmk_ok) {
-        const char *target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
+    target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
+    if (target == NULL) {
+        crm_err("Ignoring fence operation %d result: No target given (bug?)",
+                data->call_id);
+        goto bail;
+    }
+
+    stop_te_timer(action);
+    if (stonith__exit_status(data) == CRM_EX_OK) {
         const char *uuid = crm_element_value(action->xml, XML_LRM_ATTR_TARGET_UUID);
         const char *op = crm_meta_value(action->params, "stonith_action");
 
-        crm_info("Stonith operation %d for %s passed", call_id, target);
+        crm_info("Fence operation %d for %s succeeded", data->call_id, target);
         if (!(pcmk_is_set(action->flags, pcmk__graph_action_confirmed))) {
             te_action_confirmed(action, NULL);
             if (pcmk__str_eq("on", op, pcmk__str_casei)) {
@@ -785,30 +831,42 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
 
             } else if (!(pcmk_is_set(action->flags, pcmk__graph_action_sent_update))) {
                 send_stonith_update(action, target, uuid);
-                crm__set_graph_action_flags(action, pcmk__graph_action_sent_update);
+                pcmk__set_graph_action_flags(action,
+                                             pcmk__graph_action_sent_update);
             }
         }
         st_fail_count_reset(target);
 
     } else {
-        const char *target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
-        enum transition_action abort_action = tg_restart;
+        enum pcmk__graph_next abort_action = pcmk__graph_restart;
+        int status = stonith__execution_status(data);
+        const char *reason = stonith__exit_reason(data);
 
-        crm__set_graph_action_flags(action, pcmk__graph_action_failed);
-        crm_notice("Stonith operation %d for %s failed (%s): aborting transition.",
-                   call_id, target, pcmk_strerror(rc));
+        if (reason == NULL) {
+            if (status == PCMK_EXEC_DONE) {
+                reason = "Agent returned error";
+            } else {
+                reason = pcmk_exec_status_str(status);
+            }
+        }
+        pcmk__set_graph_action_flags(action, pcmk__graph_action_failed);
 
         /* If no fence devices were available, there's no use in immediately
          * checking again, so don't start a new transition in that case.
          */
-        if (rc == -ENODEV) {
-            crm_warn("No devices found in cluster to fence %s, giving up",
-                     target);
-            abort_action = tg_stop;
+        if (status == PCMK_EXEC_NO_FENCE_DEVICE) {
+            crm_warn("Fence operation %d for %s failed: %s "
+                     "(aborting transition and giving up for now)",
+                     data->call_id, target, reason);
+            abort_action = pcmk__graph_wait;
+        } else {
+            crm_notice("Fence operation %d for %s failed: %s "
+                       "(aborting transition)", data->call_id, target, reason);
         }
 
         /* Increment the fail count now, so abort_for_stonith_failure() can
-         * check it. Non-DC nodes will increment it in tengine_stonith_notify().
+         * check it. Non-DC nodes will increment it in
+         * handle_fence_notification().
          */
         st_fail_count_increment(target);
         abort_for_stonith_failure(abort_action, target, NULL);
@@ -818,7 +876,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
     trigger_graph();
 
   bail:
-    free(userdata);
+    free(data->userdata);
     free(uuid);
     return;
 }
@@ -838,8 +896,18 @@ fence_with_delay(const char *target, const char *type, const char *delay)
                                                type, timeout_sec, 0, delay_i);
 }
 
-gboolean
-te_fence_node(crm_graph_t *graph, crm_action_t *action)
+/*!
+ * \internal
+ * \brief Execute a fencing action from a transition graph
+ *
+ * \param[in] graph   Transition graph being executed (ignored)
+ * \param[in] action  Fencing action to execute
+ *
+ * \return Standard Pacemaker return code
+ */
+int
+controld_execute_fence_action(pcmk__graph_t *graph,
+                              pcmk__graph_action_t *action)
 {
     int rc = 0;
     const char *id = NULL;
@@ -862,7 +930,7 @@ te_fence_node(crm_graph_t *graph, crm_action_t *action)
 
     if (invalid_action) {
         crm_log_xml_warn(action->xml, "BadAction");
-        return FALSE;
+        return EPROTO;
     }
 
     priority_delay = crm_meta_value(action->params, XML_CONFIG_ATTR_PRIORITY_FENCING_DELAY);
@@ -883,8 +951,7 @@ te_fence_node(crm_graph_t *graph, crm_action_t *action)
                                          (int) (transition_graph->stonith_timeout / 1000),
                                          st_opt_timeout_updates, transition_key,
                                          "tengine_stonith_callback", tengine_stonith_callback);
-
-    return TRUE;
+    return pcmk_rc_ok;
 }
 
 bool
