@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -249,7 +249,7 @@ s_crmd_fsa(enum crmd_fsa_cause cause)
 
     if ((fsa_message_queue != NULL) || (fsa_actions != A_NOTHING)
         || do_fsa_stall) {
-        crm_debug("Exiting the FSA: queue=%d, fsa_actions=0x%llx, stalled=%s",
+        crm_debug("Exiting the FSA: queue=%d, fsa_actions=%#llx, stalled=%s",
                   g_list_length(fsa_message_queue),
                   (unsigned long long) fsa_actions, pcmk__btoa(do_fsa_stall));
     } else {
@@ -436,7 +436,7 @@ s_crmd_fsa_actions(fsa_data_t * fsa_data)
 
             /* Error checking and reporting */
         } else {
-            crm_err("Action %s not supported "CRM_XS" 0x%llx",
+            crm_err("Action %s not supported "CRM_XS" %#llx",
                     fsa_action2string(fsa_actions),
                     (unsigned long long) fsa_actions);
             register_fsa_error_adv(C_FSA_INTERNAL, I_ERROR, fsa_data, NULL,
@@ -581,12 +581,14 @@ do_state_transition(enum crmd_fsa_state cur_state,
 
     switch (next_state) {
         case S_PENDING:
-            fsa_cib_conn->cmds->set_slave(fsa_cib_conn, cib_scope_local);
-            /* fall through */
-        case S_ELECTION:
-            crm_trace("Resetting our DC to NULL on transition to %s", fsa_state2string(next_state));
+            fsa_cib_conn->cmds->set_secondary(fsa_cib_conn, cib_scope_local);
             update_dc(NULL);
             break;
+
+        case S_ELECTION:
+            update_dc(NULL);
+            break;
+
         case S_NOT_DC:
             election_trigger->counter = 0;
             purge_stonith_cleanup();
@@ -600,6 +602,7 @@ do_state_transition(enum crmd_fsa_state cur_state,
                 crm_err("Reached S_NOT_DC without a DC" " being recorded");
             }
             break;
+
         case S_RECOVERY:
             clear_recovery_bit = FALSE;
             break;
