@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,7 +15,7 @@
 #  include <crm/common/xml.h>
 #  include <crm/common/mainloop.h>
 #  include <crm/cluster.h>
-#  include <crm/cluster/election.h>
+#  include <crm/cluster/election_internal.h>
 #  include <crm/common/ipc_internal.h>
 
 /*! States the controller can be in */
@@ -426,7 +426,7 @@ enum crmd_fsa_input {
 
 #  define R_IN_RECOVERY     0x80000000ULL
 
-#define CRM_DIRECT_NACK_RC (99) // Deprecated (see PCMK_LRM_OP_INVALID)
+#define CRM_DIRECT_NACK_RC (99) // Deprecated (see PCMK_EXEC_INVALID)
 
 enum crmd_fsa_cause {
     C_UNKNOWN = 0,
@@ -505,7 +505,7 @@ extern char *fsa_our_uuid;
 extern char *fsa_pe_ref;        // Last invocation of the scheduler
 extern char *fsa_our_dc;
 extern char *fsa_our_dc_version;
-extern GListPtr fsa_message_queue;
+extern GList *fsa_message_queue;
 
 extern char *fsa_cluster_name;
 
@@ -518,7 +518,7 @@ extern gboolean ever_had_quorum;
 // These should be moved elsewhere
 void do_update_cib_nodes(gboolean overwrite, const char *caller);
 int crmd_cib_smart_opt(void);
-xmlNode *controld_query_executor_state(const char *node_name);
+xmlNode *controld_query_executor_state(void);
 
 const char *fsa_input2string(enum crmd_fsa_input input);
 const char *fsa_state2string(enum crmd_fsa_state state);
