@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -16,6 +16,8 @@
 
 #include <pacemaker-controld.h>
 
+extern pcmk__output_t *logger_out;
+
 int reannounce_count = 0;
 void join_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *user_data);
 
@@ -32,12 +34,12 @@ extern ha_msg_input_t *copy_ha_msg_input(ha_msg_input_t * orig);
  * \param[in] msg  A join message from the DC
  */
 static void
-update_dc_expected(xmlNode *msg)
+update_dc_expected(const xmlNode *msg)
 {
-    if (fsa_our_dc && crm_is_true(crm_element_value(msg, F_CRM_DC_LEAVING))) {
+    if (fsa_our_dc && pcmk__xe_attr_is_true(msg, F_CRM_DC_LEAVING)) {
         crm_node_t *dc_node = crm_get_peer(0, fsa_our_dc);
 
-        crm_update_peer_expected(__func__, dc_node, CRMD_JOINSTATE_DOWN);
+        pcmk__update_peer_expected(__func__, dc_node, CRMD_JOINSTATE_DOWN);
     }
 }
 
@@ -195,14 +197,16 @@ set_join_state(const char * start_state)
     if (pcmk__str_eq(start_state, "standby", pcmk__str_casei)) {
         crm_notice("Forcing node %s to join in %s state per configured environment",
                    fsa_our_uname, start_state);
-        update_attr_delegate(fsa_cib_conn, cib_sync_call, XML_CIB_TAG_NODES, fsa_our_uuid,
-                             NULL, NULL, NULL, "standby", "on", TRUE, NULL, NULL);
+        cib__update_node_attr(logger_out, fsa_cib_conn, cib_sync_call,
+                              XML_CIB_TAG_NODES, fsa_our_uuid, NULL, NULL,
+                              NULL, "standby", "on", NULL, NULL);
 
     } else if (pcmk__str_eq(start_state, "online", pcmk__str_casei)) {
         crm_notice("Forcing node %s to join in %s state per configured environment",
                    fsa_our_uname, start_state);
-        update_attr_delegate(fsa_cib_conn, cib_sync_call, XML_CIB_TAG_NODES, fsa_our_uuid,
-                             NULL, NULL, NULL, "standby", "off", TRUE, NULL, NULL);
+        cib__update_node_attr(logger_out, fsa_cib_conn, cib_sync_call,
+                              XML_CIB_TAG_NODES, fsa_our_uuid, NULL, NULL,
+                              NULL, "standby", "off", NULL, NULL);
 
     } else if (pcmk__str_eq(start_state, "default", pcmk__str_casei)) {
         crm_debug("Not forcing a starting state on node %s", fsa_our_uname);
@@ -225,11 +229,10 @@ do_cl_join_finalize_respond(long long action,
     gboolean was_nack = TRUE;
     static gboolean first_join = TRUE;
     ha_msg_input_t *input = fsa_typed_data(fsa_dt_ha_msg);
-    const char *start_state = pcmk__env_option("node_start_state");
+    const char *start_state = pcmk__env_option(PCMK__ENV_NODE_START_STATE);
 
     int join_id = -1;
     const char *op = crm_element_value(input->msg, F_CRM_TASK);
-    const char *ack_nack = crm_element_value(input->msg, CRM_OP_JOIN_ACKNAK);
     const char *welcome_from = crm_element_value(input->msg, F_CRM_HOST_FROM);
 
     if (!pcmk__str_eq(op, CRM_OP_JOIN_ACKNAK, pcmk__str_casei)) {
@@ -238,7 +241,7 @@ do_cl_join_finalize_respond(long long action,
     }
 
     /* calculate if it was an ack or a nack */
-    if (crm_is_true(ack_nack)) {
+    if (pcmk__xe_attr_is_true(input->msg, CRM_OP_JOIN_ACKNAK)) {
         was_nack = FALSE;
     }
 
@@ -264,8 +267,12 @@ do_cl_join_finalize_respond(long long action,
 
     update_dc_expected(input->msg);
 
+    /* record the node's feature set as a transient attribute */
+    update_attrd(fsa_our_uname, CRM_ATTR_FEATURE_SET, CRM_FEATURE_SET, NULL,
+                 FALSE);
+
     /* send our status section to the DC */
-    tmp1 = controld_query_executor_state(fsa_our_uname);
+    tmp1 = controld_query_executor_state();
     if (tmp1 != NULL) {
         xmlNode *reply = create_request(CRM_OP_JOIN_CONFIRM, tmp1, fsa_our_dc,
                                         CRM_SYSTEM_DC, CRM_SYSTEM_CRMD, NULL);
