@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -138,7 +138,7 @@ create_node_state_update(crm_node_t *node, int flags, xmlNode *parent,
     node_state = create_xml_node(parent, XML_CIB_TAG_STATE);
 
     if (pcmk_is_set(node->flags, crm_remote_node)) {
-        crm_xml_add(node_state, XML_NODE_IS_REMOTE, XML_BOOLEAN_TRUE);
+        pcmk__xe_set_bool_attr(node_state, XML_NODE_IS_REMOTE, true);
     }
 
     set_uuid(node_state, XML_ATTR_UUID, node);
@@ -152,8 +152,8 @@ create_node_state_update(crm_node_t *node, int flags, xmlNode *parent,
     crm_xml_add(node_state, XML_ATTR_UNAME, node->uname);
 
     if ((flags & node_update_cluster) && node->state) {
-        crm_xml_add_boolean(node_state, XML_NODE_IN_CLUSTER,
-                            pcmk__str_eq(node->state, CRM_NODE_MEMBER, pcmk__str_casei));
+        pcmk__xe_set_bool_attr(node_state, XML_NODE_IN_CLUSTER,
+                               pcmk__str_eq(node->state, CRM_NODE_MEMBER, pcmk__str_casei));
     }
 
     if (!pcmk_is_set(node->flags, crm_remote_node)) {
@@ -292,8 +292,6 @@ node_list_update_callback(xmlNode * msg, int call_id, int rc, xmlNode * output,
     }
 }
 
-#define NODE_PATH_MAX 512
-
 void
 populate_cib_nodes(enum node_update_flags flags, const char *source)
 {
@@ -304,21 +302,26 @@ populate_cib_nodes(enum node_update_flags flags, const char *source)
 
 #if SUPPORT_COROSYNC
     if (!pcmk_is_set(flags, node_update_quick) && is_corosync_cluster()) {
-        from_hashtable = corosync_initialize_nodelist(NULL, FALSE, node_list);
+        from_hashtable = pcmk__corosync_add_nodes(node_list);
     }
 #endif
 
     if (from_hashtable) {
         GHashTableIter iter;
         crm_node_t *node = NULL;
+        GString *xpath = NULL;
 
         g_hash_table_iter_init(&iter, crm_peer_cache);
         while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
             xmlNode *new_node = NULL;
 
-            crm_trace("Creating node entry for %s/%s", node->uname, node->uuid);
-            if(node->uuid && node->uname) {
-                char xpath[NODE_PATH_MAX];
+            if ((node->uuid != NULL) && (node->uname != NULL)) {
+                crm_trace("Creating node entry for %s/%s", node->uname, node->uuid);
+                if (xpath == NULL) {
+                    xpath = g_string_sized_new(512);
+                } else {
+                    g_string_truncate(xpath, 0);
+                }
 
                 /* We need both to be valid */
                 new_node = create_xml_node(node_list, XML_CIB_TAG_NODE);
@@ -326,17 +329,24 @@ populate_cib_nodes(enum node_update_flags flags, const char *source)
                 crm_xml_add(new_node, XML_ATTR_UNAME, node->uname);
 
                 /* Search and remove unknown nodes with the conflicting uname from CIB */
-                snprintf(xpath, NODE_PATH_MAX,
-                         "/" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_NODES
-                         "/" XML_CIB_TAG_NODE "[@uname='%s'][@id!='%s']",
-                         node->uname, node->uuid);
+                pcmk__g_strcat(xpath,
+                               "/" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION
+                               "/" XML_CIB_TAG_NODES "/" XML_CIB_TAG_NODE
+                               "[@" XML_ATTR_UNAME "='", node->uname, "']"
+                               "[@" XML_ATTR_ID "!='", node->uuid, "']", NULL);
 
-                call_id = fsa_cib_conn->cmds->query(fsa_cib_conn, xpath, NULL,
+                call_id = fsa_cib_conn->cmds->query(fsa_cib_conn,
+                                                    (const char *) xpath->str,
+                                                    NULL,
                                                     cib_scope_local | cib_xpath);
                 fsa_register_cib_callback(call_id, FALSE, strdup(node->uuid),
                                           search_conflicting_node_callback);
             }
         }
+
+        if (xpath != NULL) {
+            g_string_free(xpath, TRUE);
+        }
     }
 
     crm_trace("Populating <nodes> section from %s", from_hashtable ? "hashtable" : "cluster");
@@ -430,9 +440,11 @@ crm_update_quorum(gboolean quorum, gboolean force_update)
              * nodes are joining around the same time, so the one that brings us
              * to quorum doesn't cause all the remaining ones to be fenced.
              */
-            abort_after_delay(INFINITY, tg_restart, "Quorum gained", 5000);
+            abort_after_delay(INFINITY, pcmk__graph_restart, "Quorum gained",
+                              5000);
         } else {
-            abort_transition(INFINITY, tg_restart, "Quorum lost", NULL);
+            abort_transition(INFINITY, pcmk__graph_restart, "Quorum lost",
+                             NULL);
         }
     }
     fsa_has_quorum = quorum;
