@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -22,7 +22,7 @@
 
 #include <pacemaker-controld.h>
 
-GListPtr fsa_message_queue = NULL;
+GList *fsa_message_queue = NULL;
 extern void crm_shutdown(int nsig);
 
 static enum crmd_fsa_input handle_message(xmlNode *msg,
@@ -174,7 +174,7 @@ void
 fsa_dump_queue(int log_level)
 {
     int offset = 0;
-    GListPtr lpc = NULL;
+    GList *lpc = NULL;
 
     for (lpc = fsa_message_queue; lpc != NULL; lpc = lpc->next) {
         fsa_data_t *data = (fsa_data_t *) lpc->data;
@@ -326,12 +326,12 @@ gboolean
 relay_message(xmlNode * msg, gboolean originated_locally)
 {
     int dest = 1;
-    int is_for_dc = 0;
-    int is_for_dcib = 0;
-    int is_for_te = 0;
-    int is_for_crm = 0;
-    int is_for_cib = 0;
-    int is_local = 0;
+    bool is_for_dc = false;
+    bool is_for_dcib = false;
+    bool is_for_te = false;
+    bool is_for_crm = false;
+    bool is_for_cib = false;
+    bool is_local = false;
     const char *host_to = crm_element_value(msg, F_CRM_HOST_TO);
     const char *sys_to = crm_element_value(msg, F_CRM_SYS_TO);
     const char *sys_from = crm_element_value(msg, F_CRM_SYS_FROM);
@@ -348,18 +348,17 @@ relay_message(xmlNode * msg, gboolean originated_locally)
         return TRUE;
 
     } else if (pcmk__str_eq(task, CRM_OP_HELLO, pcmk__str_casei)) {
-        /* quietly ignore */
         crm_trace("No routing needed for hello message %s", ref);
         return TRUE;
 
     } else if (!pcmk__str_eq(type, T_CRM, pcmk__str_casei)) {
-        crm_warn("Cannot route message %s: Type is '%s' not '" T_CRM "'",
-                 ref, (type? type : "missing"));
+        crm_warn("Received invalid message %s: type '%s' not '" T_CRM "'",
+                 ref, pcmk__s(type, ""));
         crm_log_xml_warn(msg, "[bad message type]");
         return TRUE;
 
     } else if (sys_to == NULL) {
-        crm_warn("Cannot route message %s: No subsystem specified", ref);
+        crm_warn("Received invalid message %s: no subsystem", ref);
         crm_log_xml_warn(msg, "[no subsystem]");
         return TRUE;
     }
@@ -370,10 +369,10 @@ relay_message(xmlNode * msg, gboolean originated_locally)
     is_for_cib = (strcasecmp(CRM_SYSTEM_CIB, sys_to) == 0);
     is_for_crm = (strcasecmp(CRM_SYSTEM_CRMD, sys_to) == 0);
 
-    is_local = 0;
+    is_local = false;
     if (pcmk__str_empty(host_to)) {
         if (is_for_dc || is_for_te) {
-            is_local = 0;
+            is_local = false;
 
         } else if (is_for_crm) {
             if (pcmk__strcase_any_of(task, CRM_OP_NODE_INFO,
@@ -383,24 +382,24 @@ relay_message(xmlNode * msg, gboolean originated_locally)
                  * client may not know the local node name. Always handle these
                  * requests locally.
                  */
-                is_local = 1;
+                is_local = true;
             } else {
                 is_local = !originated_locally;
             }
 
         } else {
-            is_local = 1;
+            is_local = true;
         }
 
     } else if (pcmk__str_eq(fsa_our_uname, host_to, pcmk__str_casei)) {
-        is_local = 1;
+        is_local = true;
     } else if (is_for_crm && pcmk__str_eq(task, CRM_OP_LRM_DELETE, pcmk__str_casei)) {
         xmlNode *msg_data = get_message_xml(msg, F_CRM_DATA);
         const char *mode = crm_element_value(msg_data, PCMK__XA_MODE);
 
         if (pcmk__str_eq(mode, XML_TAG_CIB, pcmk__str_casei)) {
             // Local delete of an offline node's resource history
-            is_local = 1;
+            is_local = true;
         }
     }
 
@@ -456,7 +455,7 @@ relay_message(xmlNode * msg, gboolean originated_locally)
 #endif
 
         if (host_to) {
-            node_to = crm_find_peer(0, host_to);
+            node_to = pcmk__search_cluster_node_cache(0, host_to);
             if (node_to == NULL) {
                 crm_warn("Cannot route message %s: Unknown node %s",
                          ref, host_to);
@@ -479,21 +478,16 @@ authorize_version(xmlNode *message_data, const char *field,
                   const char *client_name, const char *ref, const char *uuid)
 {
     const char *version = crm_element_value(message_data, field);
+    long long version_num;
 
-    if (pcmk__str_empty(version)) {
-        crm_warn("IPC hello from %s rejected: No protocol %s",
+    if ((pcmk__scan_ll(version, &version_num, -1LL) != pcmk_rc_ok)
+        || (version_num < 0LL)) {
+
+        crm_warn("Rejected IPC hello from %s: '%s' is not a valid protocol %s "
                  CRM_XS " ref=%s uuid=%s",
-                 client_name, field, (ref? ref : "none"), uuid);
+                 client_name, ((version == NULL)? "" : version),
+                 field, (ref? ref : "none"), uuid);
         return false;
-    } else {
-        int version_num = crm_parse_int(version, NULL);
-
-        if (version_num < 0) {
-            crm_warn("IPC hello from %s rejected: Protocol %s '%s' "
-                     "not recognized", CRM_XS " ref=%s uuid=%s",
-                     client_name, field, version, (ref? ref : "none"), uuid);
-            return false;
-        }
     }
     return true;
 }
@@ -507,14 +501,14 @@ authorize_version(xmlNode *message_data, const char *field,
  * that nothing further needs to be done with the message. If the message is not
  * a hello, just return true to indicate it needs further processing.
  *
- * \param[in] client_msg     XML of IPC message
- * \param[in] curr_client    If IPC is not proxied, client that sent message
- * \param[in] proxy_session  If IPC is proxied, the session ID
+ * \param[in]     client_msg     XML of IPC message
+ * \param[in,out] curr_client    If IPC is not proxied, client that sent message
+ * \param[in]     proxy_session  If IPC is proxied, the session ID
  *
  * \return true if message needs further processing, false if it doesn't
  */
 bool
-controld_authorize_ipc_message(xmlNode *client_msg, pcmk__client_t *curr_client,
+controld_authorize_ipc_message(const xmlNode *client_msg, pcmk__client_t *curr_client,
                                const char *proxy_session)
 {
     xmlNode *message_data = NULL;
@@ -672,16 +666,12 @@ handle_lrm_delete(xmlNode *stored_msg)
         rsc_id = ID(rsc_xml);
         from_sys = crm_element_value(stored_msg, F_CRM_SYS_FROM);
         node = crm_element_value(msg_data, XML_LRM_ATTR_TARGET);
-#if ENABLE_ACL
         user_name = pcmk__update_acl_user(stored_msg, F_CRM_USER, NULL);
-#endif
         crm_debug("Handling " CRM_OP_LRM_DELETE " for %s on %s locally%s%s "
                   "(clearing CIB resource history only)", rsc_id, node,
                   (user_name? " for user " : ""), (user_name? user_name : ""));
-#if ENABLE_ACL
         rc = controld_delete_resource_history(rsc_id, node, user_name,
                                               cib_dryrun|cib_sync_call);
-#endif
         if (rc == pcmk_rc_ok) {
             rc = controld_delete_resource_history(rsc_id, node, user_name,
                                                   crmd_cib_smart_opt());
@@ -708,7 +698,7 @@ handle_lrm_delete(xmlNode *stored_msg)
             op = lrmd_new_event(rsc_id, CRMD_ACTION_DELETE, 0);
             op->type = lrmd_event_exec_complete;
             op->user_data = strdup(transition? transition : FAKE_TE_ID);
-            op->params = crm_str_table_new();
+            op->params = pcmk__strkey_table(free, free);
             g_hash_table_insert(op->params, strdup(XML_ATTR_CRM_VERSION),
                                 strdup(CRM_FEATURE_SET));
             controld_rc2event(op, rc);
@@ -728,20 +718,23 @@ handle_lrm_delete(xmlNode *stored_msg)
  * \return Next FSA input
  */
 static enum crmd_fsa_input
-handle_remote_state(xmlNode *msg)
+handle_remote_state(const xmlNode *msg)
 {
     const char *remote_uname = ID(msg);
-    const char *remote_is_up = crm_element_value(msg, XML_NODE_IN_CLUSTER);
     crm_node_t *remote_peer;
+    bool remote_is_up = false;
+    int rc = pcmk_rc_ok;
 
-    CRM_CHECK(remote_uname && remote_is_up, return I_NULL);
+    rc = pcmk__xe_get_bool_attr(msg, XML_NODE_IN_CLUSTER, &remote_is_up);
+
+    CRM_CHECK(remote_uname && rc == pcmk_rc_ok, return I_NULL);
 
     remote_peer = crm_remote_peer_get(remote_uname);
     CRM_CHECK(remote_peer, return I_NULL);
 
-    crm_update_peer_state(__func__, remote_peer,
-                          crm_is_true(remote_is_up)?
-                          CRM_NODE_MEMBER : CRM_NODE_LOST, 0);
+    pcmk__update_peer_state(__func__, remote_peer,
+                            remote_is_up ? CRM_NODE_MEMBER : CRM_NODE_LOST,
+                            0);
     return I_NULL;
 }
 
@@ -753,10 +746,11 @@ handle_remote_state(xmlNode *msg)
  * \return Next FSA input
  */
 static enum crmd_fsa_input
-handle_ping(xmlNode *msg)
+handle_ping(const xmlNode *msg)
 {
     const char *value = NULL;
     xmlNode *ping = NULL;
+    xmlNode *reply = NULL;
 
     // Build reply
 
@@ -774,11 +768,11 @@ handle_ping(xmlNode *msg)
     crm_xml_add(ping, XML_PING_ATTR_STATUS, "ok");
 
     // Send reply
-    msg = create_reply(msg, ping);
+    reply = create_reply(msg, ping);
     free_xml(ping);
-    if (msg) {
-        (void) relay_message(msg, TRUE);
-        free_xml(msg);
+    if (reply != NULL) {
+        (void) relay_message(reply, TRUE);
+        free_xml(reply);
     }
 
     // Nothing further to do
@@ -788,10 +782,12 @@ handle_ping(xmlNode *msg)
 /*!
  * \brief Handle a PCMK__CONTROLD_CMD_NODES message
  *
+ * \param[in] request  Message XML
+ *
  * \return Next FSA input
  */
 static enum crmd_fsa_input
-handle_node_list(xmlNode *request)
+handle_node_list(const xmlNode *request)
 {
     GHashTableIter iter;
     crm_node_t *node = NULL;
@@ -829,20 +825,21 @@ handle_node_list(xmlNode *request)
  * \return Next FSA input
  */
 static enum crmd_fsa_input
-handle_node_info_request(xmlNode *msg)
+handle_node_info_request(const xmlNode *msg)
 {
     const char *value = NULL;
     crm_node_t *node = NULL;
     int node_id = 0;
     xmlNode *reply = NULL;
+    xmlNode *reply_data = NULL;
 
     // Build reply
 
-    reply = create_xml_node(NULL, XML_CIB_TAG_NODE);
-    crm_xml_add(reply, XML_PING_ATTR_SYSFROM, CRM_SYSTEM_CRMD);
+    reply_data = create_xml_node(NULL, XML_CIB_TAG_NODE);
+    crm_xml_add(reply_data, XML_PING_ATTR_SYSFROM, CRM_SYSTEM_CRMD);
 
     // Add whether current partition has quorum
-    crm_xml_add_boolean(reply, XML_ATTR_HAVE_QUORUM, fsa_has_quorum);
+    pcmk__xe_set_bool_attr(reply_data, XML_ATTR_HAVE_QUORUM, fsa_has_quorum);
 
     // Check whether client requested node info by ID and/or name
     crm_element_value_int(msg, XML_ATTR_ID, &node_id);
@@ -856,22 +853,22 @@ handle_node_info_request(xmlNode *msg)
         value = fsa_our_uname;
     }
 
-    node = crm_find_peer_full(node_id, value, CRM_GET_PEER_ANY);
+    node = pcmk__search_node_caches(node_id, value, CRM_GET_PEER_ANY);
     if (node) {
-        crm_xml_add_int(reply, XML_ATTR_ID, node->id);
-        crm_xml_add(reply, XML_ATTR_UUID, node->uuid);
-        crm_xml_add(reply, XML_ATTR_UNAME, node->uname);
-        crm_xml_add(reply, XML_NODE_IS_PEER, node->state);
-        crm_xml_add_boolean(reply, XML_NODE_IS_REMOTE,
-                            node->flags & crm_remote_node);
+        crm_xml_add_int(reply_data, XML_ATTR_ID, node->id);
+        crm_xml_add(reply_data, XML_ATTR_UUID, node->uuid);
+        crm_xml_add(reply_data, XML_ATTR_UNAME, node->uname);
+        crm_xml_add(reply_data, XML_NODE_IS_PEER, node->state);
+        pcmk__xe_set_bool_attr(reply_data, XML_NODE_IS_REMOTE,
+                               pcmk_is_set(node->flags, crm_remote_node));
     }
 
     // Send reply
-    msg = create_reply(msg, reply);
-    free_xml(reply);
-    if (msg) {
-        (void) relay_message(msg, TRUE);
-        free_xml(msg);
+    reply = create_reply(msg, reply_data);
+    free_xml(reply_data);
+    if (reply != NULL) {
+        (void) relay_message(reply, TRUE);
+        free_xml(reply);
     }
 
     // Nothing further to do
@@ -980,9 +977,9 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
 
     if (strcmp(op, CRM_OP_SHUTDOWN_REQ) == 0) {
         const char *from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
-        crm_node_t *node = crm_find_peer(0, from);
+        crm_node_t *node = pcmk__search_cluster_node_cache(0, from);
 
-        crm_update_peer_expected(__func__, node, CRMD_JOINSTATE_DOWN);
+        pcmk__update_peer_expected(__func__, node, CRMD_JOINSTATE_DOWN);
         if(AM_I_DC == FALSE) {
             return I_NULL; /* Done */
         }
@@ -1003,8 +1000,7 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
             return handle_shutdown_self_ack(stored_msg);
 
         } else if (strcmp(op, CRM_OP_SHUTDOWN_REQ) == 0) {
-            /* a slave wants to shut down */
-            /* create cib fragment and add to message */
+            // Another controller wants to shut down its node
             return handle_shutdown_request(stored_msg);
 
         } else if (strcmp(op, CRM_OP_REMOTE_STATE) == 0) {
@@ -1025,7 +1021,7 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
     } else if (strcmp(op, CRM_OP_THROTTLE) == 0) {
         throttle_update(stored_msg);
         if (AM_I_DC && transition_graph != NULL) {
-            if (transition_graph->complete == FALSE) {
+            if (!transition_graph->complete) {
                 crm_debug("The throttle changed. Trigger a graph.");
                 trigger_graph();
             }
@@ -1068,7 +1064,7 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
         return handle_lrm_delete(stored_msg);
 
     } else if ((strcmp(op, CRM_OP_LRM_FAIL) == 0)
-               || (strcmp(op, CRM_OP_LRM_REFRESH) == 0)
+               || (strcmp(op, CRM_OP_LRM_REFRESH) == 0) // @COMPAT
                || (strcmp(op, CRM_OP_REPROBE) == 0)) {
 
         crm_xml_add(stored_msg, F_CRM_SYS_TO, CRM_SYSTEM_LRMD);
@@ -1186,7 +1182,6 @@ handle_shutdown_request(xmlNode * stored_msg)
      */
 
     char *now_s = NULL;
-    time_t now = time(NULL);
     const char *host_from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
 
     if (host_from == NULL) {
@@ -1197,7 +1192,7 @@ handle_shutdown_request(xmlNode * stored_msg)
     crm_info("Creating shutdown request for %s (state=%s)", host_from, fsa_state2string(fsa_state));
     crm_log_xml_trace(stored_msg, "message");
 
-    now_s = crm_itoa(now);
+    now_s = pcmk__ttoa(time(NULL));
     update_attrd(host_from, XML_CIB_ATTR_SHUTDOWN, now_s, NULL, FALSE);
     free(now_s);
 
@@ -1205,13 +1200,14 @@ handle_shutdown_request(xmlNode * stored_msg)
     return I_NULL;
 }
 
-/* msg is deleted by the time this returns */
-extern gboolean process_te_message(xmlNode * msg, xmlNode * xml_data);
-
 static void
 send_msg_via_ipc(xmlNode * msg, const char *sys)
 {
-    pcmk__client_t *client_channel = pcmk__find_client_by_id(sys);
+    pcmk__client_t *client_channel = NULL;
+
+    CRM_CHECK(sys != NULL, return);
+
+    client_channel = pcmk__find_client_by_id(sys);
 
     if (crm_element_value(msg, F_CRM_HOST_FROM) == NULL) {
         crm_xml_add(msg, F_CRM_HOST_FROM, fsa_our_uname);
@@ -1221,12 +1217,12 @@ send_msg_via_ipc(xmlNode * msg, const char *sys)
         /* Transient clients such as crmadmin */
         pcmk__ipc_send_xml(client_channel, 0, msg, crm_ipc_server_event);
 
-    } else if (sys != NULL && strcmp(sys, CRM_SYSTEM_TENGINE) == 0) {
+    } else if (pcmk__str_eq(sys, CRM_SYSTEM_TENGINE, pcmk__str_none)) {
         xmlNode *data = get_message_xml(msg, F_CRM_DATA);
 
         process_te_message(msg, data);
 
-    } else if (sys != NULL && strcmp(sys, CRM_SYSTEM_LRMD) == 0) {
+    } else if (pcmk__str_eq(sys, CRM_SYSTEM_LRMD, pcmk__str_none)) {
         fsa_data_t fsa_data;
         ha_msg_input_t fsa_input;
 
@@ -1243,11 +1239,11 @@ send_msg_via_ipc(xmlNode * msg, const char *sys)
 
         do_lrm_invoke(A_LRM_INVOKE, C_IPC_MESSAGE, fsa_state, I_MESSAGE, &fsa_data);
 
-    } else if (sys != NULL && crmd_is_proxy_session(sys)) {
+    } else if (crmd_is_proxy_session(sys)) {
         crmd_proxy_send(sys, msg);
 
     } else {
-        crm_debug("Unknown Sub-system (%s)... discarding message.", crm_str(sys));
+        crm_info("Received invalid request: unknown subsystem '%s'", sys);
     }
 }
 
@@ -1279,15 +1275,15 @@ send_remote_state_message(const char *node_name, gboolean node_up)
         xmlNode *msg = create_request(CRM_OP_REMOTE_STATE, NULL, fsa_our_dc,
                                       CRM_SYSTEM_DC, CRM_SYSTEM_CRMD, NULL);
 
-        crm_info("Notifying DC %s of pacemaker_remote node %s %s",
+        crm_info("Notifying DC %s of Pacemaker Remote node %s %s",
                  fsa_our_dc, node_name, (node_up? "coming up" : "going down"));
         crm_xml_add(msg, XML_ATTR_ID, node_name);
-        crm_xml_add_boolean(msg, XML_NODE_IN_CLUSTER, node_up);
+        pcmk__xe_set_bool_attr(msg, XML_NODE_IN_CLUSTER, node_up);
         send_cluster_message(crm_get_peer(0, fsa_our_dc), crm_msg_crmd, msg,
                              TRUE);
         free_xml(msg);
     } else {
-        crm_debug("No DC to notify of pacemaker_remote node %s %s",
+        crm_debug("No DC to notify of Pacemaker Remote node %s %s",
                   node_name, (node_up? "coming up" : "going down"));
     }
 }
