@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,15 +15,15 @@
 #include <pacemaker-controld.h>
 
 gboolean
-stop_te_timer(crm_action_timer_t * timer)
+stop_te_timer(pcmk__graph_action_t *action)
 {
-    if (timer == NULL) {
+    if (action == NULL) {
         return FALSE;
     }
-    if (timer->source_id != 0) {
+    if (action->timer != 0) {
         crm_trace("Stopping action timer");
-        g_source_remove(timer->source_id);
-        timer->source_id = 0;
+        g_source_remove(action->timer);
+        action->timer = 0;
     } else {
         crm_trace("Action timer was already stopped");
         return FALSE;
@@ -34,8 +34,6 @@ stop_te_timer(crm_action_timer_t * timer)
 gboolean
 te_graph_trigger(gpointer user_data)
 {
-    enum transition_status graph_rc = -1;
-
     if (transition_graph == NULL) {
         crm_debug("Nothing to do");
         return TRUE;
@@ -56,33 +54,32 @@ te_graph_trigger(gpointer user_data)
             break;
     }
 
-    if (transition_graph->complete == FALSE) {
+    if (!transition_graph->complete) {
+        enum pcmk__graph_status graph_rc;
         int limit = transition_graph->batch_limit;
 
         transition_graph->batch_limit = throttle_get_total_job_limit(limit);
-        graph_rc = run_graph(transition_graph);
+        graph_rc = pcmk__execute_graph(transition_graph);
         transition_graph->batch_limit = limit; /* Restore the configured value */
 
-        /* significant overhead... */
-        /* print_graph(LOG_TRACE, transition_graph); */
-
-        if (graph_rc == transition_active) {
+        if (graph_rc == pcmk__graph_active) {
             crm_trace("Transition not yet complete");
             return TRUE;
 
-        } else if (graph_rc == transition_pending) {
+        } else if (graph_rc == pcmk__graph_pending) {
             crm_trace("Transition not yet complete - no actions fired");
             return TRUE;
         }
 
-        if (graph_rc != transition_complete) {
-            crm_warn("Transition failed: %s", transition_status(graph_rc));
-            print_graph(LOG_NOTICE, transition_graph);
+        if (graph_rc != pcmk__graph_complete) {
+            crm_warn("Transition failed: %s",
+                     pcmk__graph_status2text(graph_rc));
+            pcmk__log_graph(LOG_NOTICE, transition_graph);
         }
     }
 
     crm_debug("Transition %d is now complete", transition_graph->id);
-    transition_graph->complete = TRUE;
+    transition_graph->complete = true;
     notify_crmd(transition_graph);
 
     return TRUE;
@@ -99,7 +96,7 @@ static struct abort_timer_s {
     bool aborted;
     guint id;
     int priority;
-    enum transition_action action;
+    enum pcmk__graph_next action;
     const char *text;
 } abort_timer = { 0, };
 
@@ -121,7 +118,7 @@ abort_timer_popped(gpointer data)
  * \param[in] abort_text  Must be literal string
  */
 void
-abort_after_delay(int abort_priority, enum transition_action abort_action,
+abort_after_delay(int abort_priority, enum pcmk__graph_next abort_action,
                   const char *abort_text, guint delay_ms)
 {
     if (abort_timer.id) {
@@ -135,15 +132,58 @@ abort_after_delay(int abort_priority, enum transition_action abort_action,
     abort_timer.id = g_timeout_add(delay_ms, abort_timer_popped, NULL);
 }
 
+static const char *
+abort2text(enum pcmk__graph_next abort_action)
+{
+    switch (abort_action) {
+        case pcmk__graph_done:      return "done";
+        case pcmk__graph_wait:      return "stop";
+        case pcmk__graph_restart:   return "restart";
+        case pcmk__graph_shutdown:  return "shutdown";
+    }
+    return "unknown";
+}
+
+static bool
+update_abort_priority(pcmk__graph_t *graph, int priority,
+                      enum pcmk__graph_next action, const char *abort_reason)
+{
+    bool change = FALSE;
+
+    if (graph == NULL) {
+        return change;
+    }
+
+    if (graph->abort_priority < priority) {
+        crm_debug("Abort priority upgraded from %d to %d", graph->abort_priority, priority);
+        graph->abort_priority = priority;
+        if (graph->abort_reason != NULL) {
+            crm_debug("'%s' abort superseded by %s", graph->abort_reason, abort_reason);
+        }
+        graph->abort_reason = abort_reason;
+        change = TRUE;
+    }
+
+    if (graph->completion_action < action) {
+        crm_debug("Abort action %s superseded by %s: %s",
+                  abort2text(graph->completion_action), abort2text(action), abort_reason);
+        graph->completion_action = action;
+        change = TRUE;
+    }
+
+    return change;
+}
+
 void
-abort_transition_graph(int abort_priority, enum transition_action abort_action,
-                       const char *abort_text, xmlNode * reason, const char *fn, int line)
+abort_transition_graph(int abort_priority, enum pcmk__graph_next abort_action,
+                       const char *abort_text, const xmlNode *reason,
+                       const char *fn, int line)
 {
     int add[] = { 0, 0, 0 };
     int del[] = { 0, 0, 0 };
     int level = LOG_INFO;
-    xmlNode *diff = NULL;
-    xmlNode *change = NULL;
+    const xmlNode *diff = NULL;
+    const xmlNode *change = NULL;
 
     CRM_CHECK(transition_graph != NULL, return);
 
@@ -155,8 +195,9 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
         case S_ILLEGAL:
         case S_STOPPING:
         case S_TERMINATE:
-            crm_info("Abort %s suppressed: state=%s (complete=%d)",
-                     abort_text, fsa_state2string(fsa_state), transition_graph->complete);
+            crm_info("Abort %s suppressed: state=%s (%scomplete)",
+                     abort_text, fsa_state2string(fsa_state),
+                     (transition_graph->complete? "" : "in"));
             return;
         default:
             break;
@@ -165,14 +206,14 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
     abort_timer.aborted = TRUE;
     controld_expect_sched_reply(NULL);
 
-    if (transition_graph->complete == FALSE) {
+    if (!transition_graph->complete) {
         if(update_abort_priority(transition_graph, abort_priority, abort_action, abort_text)) {
             level = LOG_NOTICE;
         }
     }
 
     if(reason) {
-        xmlNode *search = NULL;
+        const xmlNode *search = NULL;
 
         for(search = reason; search; search = search->parent) {
             if (pcmk__str_eq(XML_TAG_DIFF, TYPE(search), pcmk__str_casei)) {
@@ -198,14 +239,16 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                    pcmk__btoa(transition_graph->complete));
 
     } else if(change == NULL) {
-        char *local_path = xml_get_path(reason);
+        GString *local_path = pcmk__element_xpath(reason);
+        CRM_ASSERT(local_path != NULL);
 
         do_crm_log(level, "Transition %d aborted by %s.%s: %s "
                    CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                    transition_graph->id, TYPE(reason), ID(reason), abort_text,
-                   add[0], add[1], add[2], fn, line, local_path,
+                   add[0], add[1], add[2], fn, line,
+                   (const char *) local_path->str,
                    pcmk__btoa(transition_graph->complete));
-        free(local_path);
+        g_string_free(local_path, TRUE);
 
     } else {
         const char *kind = NULL;
