@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2020 the Pacemaker project contributors
+ * Copyright 2013-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -62,7 +62,7 @@ load2str(enum throttle_state_e mode)
     }
 }
 
-#if SUPPORT_PROCFS
+#if HAVE_LINUX_PROCFS
 /*!
  * \internal
  * \brief Return name of /proc file containing the CIB daemon's load statistics
@@ -250,7 +250,8 @@ throttle_load_avg(float *load)
  * \return Throttle mode corresponding to load value
  */
 static enum throttle_state_e
-throttle_check_thresholds(float load, const char *desc, float thresholds[4])
+throttle_check_thresholds(float load, const char *desc,
+                          const float thresholds[4])
 {
     if (load > thresholds[3]) {
         crm_notice("Extreme %s detected: %f", desc, load);
@@ -294,14 +295,14 @@ throttle_handle_load(float load, const char *desc, int cores)
 
     return throttle_check_thresholds(load, desc, thresholds);
 }
-#endif
+#endif // HAVE_LINUX_PROCFS
 
 static enum throttle_state_e
 throttle_mode(void)
 {
     enum throttle_state_e mode = throttle_none;
 
-#if SUPPORT_PROCFS
+#if HAVE_LINUX_PROCFS
     unsigned int cores;
     float load;
     float thresholds[4];
@@ -351,7 +352,7 @@ throttle_mode(void)
         }
         crm_debug("Current load is %f across %u core(s)", load, cores);
     }
-#endif // SUPPORT_PROCFS
+#endif // HAVE_LINUX_PROCFS
     return mode;
 }
 
@@ -405,14 +406,14 @@ throttle_set_load_target(float target)
 void
 throttle_update_job_max(const char *preference)
 {
-    long long max = -1;
+    long long max = 0LL;
     const char *env_limit = getenv("PCMK_node_action_limit");
 
     if (env_limit != NULL) {
         preference = env_limit; // Per-node override
     }
-    if (preference) {
-        max = crm_parse_ll(preference, NULL);
+    if (preference != NULL) {
+        pcmk__scan_ll(preference, &max, 0LL);
     }
     if (max > 0) {
         throttle_job_max = (int) max;
@@ -426,8 +427,7 @@ void
 throttle_init(void)
 {
     if(throttle_records == NULL) {
-        throttle_records = g_hash_table_new_full(
-            crm_str_hash, g_str_equal, NULL, throttle_record_free);
+        throttle_records = pcmk__strkey_table(NULL, throttle_record_free);
         throttle_timer = mainloop_timer_add("throttle", 30 * 1000, TRUE, throttle_timer_cb, NULL);
     }
 
