@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -724,17 +724,9 @@ update_dc(xmlNode * msg)
         }
     }
 
-    free(fsa_our_dc_version);
-    fsa_our_dc_version = NULL;
-
     fsa_our_dc = NULL;          /* Free'd as last_dc */
-
-    if (welcome_from != NULL) {
-        fsa_our_dc = strdup(welcome_from);
-    }
-    if (dc_version != NULL) {
-        fsa_our_dc_version = strdup(dc_version);
-    }
+    pcmk__str_update(&fsa_our_dc, welcome_from);
+    pcmk__str_update(&fsa_our_dc_version, dc_version);
 
     if (pcmk__str_eq(fsa_our_dc, last_dc, pcmk__str_casei)) {
         /* do nothing */
@@ -742,11 +734,12 @@ update_dc(xmlNode * msg)
     } else if (fsa_our_dc != NULL) {
         crm_node_t *dc_node = crm_get_peer(0, fsa_our_dc);
 
-        crm_info("Set DC to %s (%s)", crm_str(fsa_our_dc), crm_str(fsa_our_dc_version));
+        crm_info("Set DC to %s (%s)",
+                 fsa_our_dc, pcmk__s(fsa_our_dc_version, "unknown version"));
         pcmk__update_peer_expected(__func__, dc_node, CRMD_JOINSTATE_MEMBER);
 
     } else if (last_dc != NULL) {
-        crm_info("Unset DC. Was %s", crm_str(last_dc));
+        crm_info("Unset DC (was %s)", last_dc);
     }
 
     free(last_dc);
@@ -766,7 +759,7 @@ void crmd_peer_down(crm_node_t *peer, bool full)
 #define MIN_CIB_OP_TIMEOUT (30)
 
 unsigned int
-cib_op_timeout()
+cib_op_timeout(void)
 {
     static int env_timeout = -1;
     unsigned int calculated_timeout = 0;
