@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -12,7 +12,7 @@
 
 #  include <crm/crm.h>
 #  include <crm/common/xml.h>
-#  include <crm/cib/internal.h>     // CIB_OP_MODIFY
+#  include <crm/cib/internal.h>     // PCMK__CIB_REQUEST_MODIFY
 #  include <controld_fsa.h>         // fsa_cib_conn
 #  include <controld_alerts.h>
 
@@ -22,7 +22,7 @@
 #  define fsa_cib_update(section, data, options, call_id, user_name)	\
 	if(fsa_cib_conn != NULL) {					\
 	    call_id = cib_internal_op(                                  \
-		fsa_cib_conn, CIB_OP_MODIFY, NULL, section, data,	\
+        fsa_cib_conn, PCMK__CIB_REQUEST_MODIFY, NULL, section, data,    \
 		NULL, options, user_name);				\
 									\
 	} else {							\
@@ -68,10 +68,10 @@ enum node_update_flags {
 
 crm_exit_t crmd_exit(crm_exit_t exit_code);
 _Noreturn void crmd_fast_exit(crm_exit_t exit_code);
-void pe_subsystem_free(void);
+void controld_shutdown_schedulerd_ipc(void);
 void controld_stop_sched_timer(void);
 void controld_free_sched_timer(void);
-void controld_expect_sched_reply(xmlNode *msg);
+void controld_expect_sched_reply(char *ref);
 
 void fsa_dump_actions(uint64_t action, const char *text);
 void fsa_dump_inputs(int log_level, const char *text, long long input_register);
@@ -84,6 +84,7 @@ void populate_cib_nodes(enum node_update_flags flags, const char *source);
 void crm_update_quorum(gboolean quorum, gboolean force_update);
 void controld_close_attrd_ipc(void);
 void update_attrd(const char *host, const char *name, const char *value, const char *user_name, gboolean is_remote_node);
+void update_attrd_list(GList *attrs, uint32_t opts);
 void update_attrd_remote_node_removed(const char *host, const char *user_name);
 void update_attrd_clear_failures(const char *host, const char *rsc,
                                  const char *op, const char *interval_spec,
