@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2021 the Pacemaker project contributors
+ * Copyright 2012-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -31,7 +31,7 @@ int lrmd_call_id = 0;
 
 #ifdef PCMK__COMPILE_REMOTE
 /* whether shutdown request has been sent */
-static sig_atomic_t shutting_down = FALSE;
+static gboolean shutting_down = FALSE;
 
 /* timer for waiting for acknowledgment of shutdown request */
 static guint shutdown_ack_timer = 0;
@@ -43,7 +43,6 @@ static void
 stonith_connection_destroy_cb(stonith_t * st, stonith_event_t * e)
 {
     stonith_api->state = stonith_disconnected;
-    crm_err("Connection to fencer lost");
     stonith_connection_failed();
 }
 
@@ -150,7 +149,7 @@ lrmd_ipc_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
  * \internal
  * \brief Free a client connection, and exit if appropriate
  *
- * \param[in] client  Client connection to free
+ * \param[in,out] client  Client connection to free
  */
 void
 lrmd_client_destroy(pcmk__client_t *client)
@@ -214,7 +213,7 @@ lrmd_server_send_reply(pcmk__client_t *client, uint32_t id, xmlNode *reply)
 #endif
         default:
             crm_err("Could not send reply: unknown type for client %s "
-                    CRM_XS " flags=0x%llx",
+                    CRM_XS " flags=%#llx",
                     pcmk__client_name(client), client->flags);
     }
     return ENOTCONN;
@@ -243,7 +242,7 @@ lrmd_server_send_notify(pcmk__client_t *client, xmlNode *msg)
 #endif
         default:
             crm_err("Could not notify client %s with unknown transport "
-                    CRM_XS " flags=0x%llx",
+                    CRM_XS " flags=%#llx",
                     pcmk__client_name(client), client->flags);
     }
     return ENOTCONN;
@@ -272,7 +271,7 @@ lrmd_exit(gpointer data)
     }
 
 #ifdef PCMK__COMPILE_REMOTE
-    lrmd_tls_server_destroy();
+    execd_stop_tls_server();
     ipc_proxy_cleanup();
 #endif
 
@@ -287,6 +286,23 @@ lrmd_exit(gpointer data)
     return FALSE;
 }
 
+/*!
+ * \internal
+ * \brief Clean up and exit if shutdown has started
+ *
+ * \return Doesn't return
+ */
+void
+execd_exit_if_shutting_down(void)
+{
+#ifdef PCMK__COMPILE_REMOTE
+    if (shutting_down) {
+        crm_warn("exit because TLS connection was closed and 'shutting_down' set");
+        lrmd_exit(NULL);
+    }
+#endif
+}
+
 /*!
  * \internal
  * \brief Request cluster shutdown if appropriate, otherwise exit immediately
@@ -321,7 +337,7 @@ lrmd_shutdown(int nsig)
             shutting_down = TRUE;
 
             /* Stop accepting new proxy connections */
-            lrmd_tls_server_destroy();
+            execd_stop_tls_server();
 
             /* Older controller versions will never acknowledge our request, so
              * set a fairly short timeout to exit quickly in that case. If we
@@ -345,7 +361,8 @@ lrmd_shutdown(int nsig)
  * \internal
  * \brief Defuse short exit timer if shutting down
  */
-void handle_shutdown_ack()
+void
+handle_shutdown_ack(void)
 {
 #ifdef PCMK__COMPILE_REMOTE
     if (shutting_down) {
@@ -364,7 +381,8 @@ void handle_shutdown_ack()
  * \internal
  * \brief Make short exit timer fire immediately
  */
-void handle_shutdown_nack()
+void
+handle_shutdown_nack(void)
 {
 #ifdef PCMK__COMPILE_REMOTE
     if (shutting_down) {
@@ -476,12 +494,15 @@ main(int argc, char **argv, char **envp)
     }
 
     option = pcmk__env_option(PCMK__ENV_LOGFACILITY);
-    if (option && !pcmk__strcase_any_of(option, "none", "/dev/null", NULL)) {
+    if (!pcmk__str_eq(option, PCMK__VALUE_NONE,
+                      pcmk__str_casei|pcmk__str_null_matches)
+        && !pcmk__str_eq(option, "/dev/null", pcmk__str_none)) {
         setenv("HA_LOGFACILITY", option, 1);  /* Used by the ocf_log/ha_log OCF macro */
     }
 
     option = pcmk__env_option(PCMK__ENV_LOGFILE);
-    if(option && !pcmk__str_eq(option, "none", pcmk__str_casei)) {
+    if (!pcmk__str_eq(option, PCMK__VALUE_NONE,
+                      pcmk__str_casei|pcmk__str_null_matches)) {
         setenv("HA_LOGFILE", option, 1);      /* Used by the ocf_log/ha_log OCF macro */
 
         if (pcmk__env_option_enabled(crm_system_name, PCMK__ENV_DEBUG)) {
