@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,8 +7,11 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__COMMON_ACL__H
-#  define PCMK__COMMON_ACL__H
+#ifndef PCMK__CRM_COMMON_ACL__H
+#  define PCMK__CRM_COMMON_ACL__H
+
+#  include <libxml/tree.h> // xmlNode
+#  include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -20,11 +23,9 @@ extern "C" {
  * \ingroup core
  */
 
-#  include <libxml/tree.h> // xmlNode
-
-bool xml_acl_enabled(xmlNode *xml);
+bool xml_acl_enabled(const xmlNode *xml);
 void xml_acl_disable(xmlNode *xml);
-bool xml_acl_denied(xmlNode *xml);
+bool xml_acl_denied(const xmlNode *xml);
 bool xml_acl_filtered_copy(const char *user, xmlNode* acl_source, xmlNode *xml,
                            xmlNode **result);
 
@@ -34,4 +35,4 @@ bool pcmk_acl_required(const char *user);
 }
 #endif
 
-#endif // PCMK__COMMON_ACL__H
+#endif // PCMK__CRM_COMMON_ACL__H
