@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,6 +7,14 @@
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
+#ifndef PCMK__CRM_COMMON_LOGGING__H
+#  define PCMK__CRM_COMMON_LOGGING__H
+
+#  include <stdio.h>
+#  include <glib.h>
+#  include <qb/qblog.h>
+#  include <libxml/tree.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -17,13 +25,6 @@ extern "C" {
  * \ingroup core
  */
 
-#ifndef CRM_LOGGING__H
-#  define CRM_LOGGING__H
-
-#  include <stdio.h>
-#  include <glib.h>
-#  include <qb/qblog.h>
-#  include <libxml/tree.h>
 
 /* Define custom log priorities.
  *
@@ -99,7 +100,7 @@ enum xml_log_options
 
 void crm_enable_blackbox(int nsig);
 void crm_disable_blackbox(int nsig);
-void crm_write_blackbox(int nsig, struct qb_log_callsite *callsite);
+void crm_write_blackbox(int nsig, const struct qb_log_callsite *callsite);
 
 void crm_update_callsites(void);
 
@@ -112,7 +113,7 @@ void crm_log_deinit(void);
  * \param[in] argc    The number of command line parameters
  * \param[in] argv    The command line parameter values
  */
-void crm_log_preinit(const char *entity, int argc, char **argv);
+void crm_log_preinit(const char *entity, int argc, char *const *argv);
 gboolean crm_log_init(const char *entity, uint8_t level, gboolean daemon,
                       gboolean to_stderr, int argc, char **argv, gboolean quiet);
 
@@ -130,8 +131,9 @@ void crm_enable_stderr(int enable);
 
 gboolean crm_is_callsite_active(struct qb_log_callsite *cs, uint8_t level, uint32_t tags);
 
-void log_data_element(int log_level, const char *file, const char *function, int line,
-                      const char *prefix, xmlNode * data, int depth, gboolean formatted);
+void log_data_element(int log_level, const char *file, const char *function,
+                      int line, const char *prefix, const xmlNode *data,
+                      int depth, gboolean formatted);
 
 /* returns the old value */
 unsigned int set_crm_log_level(unsigned int level);
@@ -380,8 +382,6 @@ unsigned int get_crm_log_level(void);
         }                                                       \
     } while(0)
 
-#  define crm_str(x)    (const char*)(x?x:"<null>")
-
 #if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
 #include <crm/common/logging_compat.h>
 #endif
