@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2020 the Pacemaker project contributors
+ * Copyright 2006-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,8 +7,8 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef OPTIONS_INTERNAL__H
-#  define OPTIONS_INTERNAL__H
+#ifndef PCMK__OPTIONS_INTERNAL__H
+#  define PCMK__OPTIONS_INTERNAL__H
 
 #  ifndef PCMK__CONFIG_H
 #    define PCMK__CONFIG_H
@@ -58,7 +58,7 @@ typedef struct pcmk__cli_option_s {
 } pcmk__cli_option_t;
 
 void pcmk__set_cli_options(const char *short_options, const char *usage,
-                           pcmk__cli_option_t *long_options,
+                           const pcmk__cli_option_t *long_options,
                            const char *app_desc);
 int pcmk__next_cli_option(int argc, char **argv, int *index,
                           const char **longname);
@@ -94,12 +94,13 @@ typedef struct pcmk__cluster_option_s {
 } pcmk__cluster_option_t;
 
 const char *pcmk__cluster_option(GHashTable *options,
-                                 pcmk__cluster_option_t *option_list, int len,
-                                 const char *name);
+                                 const pcmk__cluster_option_t *option_list,
+                                 int len, const char *name);
 
-void pcmk__print_option_metadata(const char *name, const char *version,
-                                 const char *desc_short, const char *desc_long,
-                                 pcmk__cluster_option_t *option_list, int len);
+gchar *pcmk__format_option_metadata(const char *name, const char *desc_short,
+                                    const char *desc_long,
+                                    pcmk__cluster_option_t *option_list,
+                                    int len);
 
 void pcmk__validate_cluster_options(GHashTable *options,
                                     pcmk__cluster_option_t *option_list,
@@ -111,7 +112,7 @@ bool pcmk__valid_number(const char *value);
 bool pcmk__valid_positive_number(const char *value);
 bool pcmk__valid_quorum(const char *value);
 bool pcmk__valid_script(const char *value);
-bool pcmk__valid_utilization(const char *value);
+bool pcmk__valid_percentage(const char *value);
 
 // from watchdog.c
 long pcmk__get_sbd_timeout(void);
@@ -119,4 +120,42 @@ bool pcmk__get_sbd_sync_resource_startup(void);
 long pcmk__auto_watchdog_timeout(void);
 bool pcmk__valid_sbd_timeout(const char *value);
 
-#endif // OPTIONS_INTERNAL__H
+// Constants for environment variable names
+#define PCMK__ENV_BLACKBOX                  "blackbox"
+#define PCMK__ENV_CLUSTER_TYPE              "cluster_type"
+#define PCMK__ENV_DEBUG                     "debug"
+#define PCMK__ENV_LOGFACILITY               "logfacility"
+#define PCMK__ENV_LOGFILE                   "logfile"
+#define PCMK__ENV_LOGPRIORITY               "logpriority"
+#define PCMK__ENV_MCP                       "mcp"
+#define PCMK__ENV_NODE_START_STATE          "node_start_state"
+#define PCMK__ENV_PHYSICAL_HOST             "physical_host"
+#define PCMK__ENV_QUORUM_TYPE               "quorum_type"
+#define PCMK__ENV_SHUTDOWN_DELAY            "shutdown_delay"
+#define PCMK__ENV_STDERR                    "stderr"
+
+// Constants for cluster option names
+#define PCMK__OPT_NODE_HEALTH_BASE          "node-health-base"
+#define PCMK__OPT_NODE_HEALTH_GREEN         "node-health-green"
+#define PCMK__OPT_NODE_HEALTH_RED           "node-health-red"
+#define PCMK__OPT_NODE_HEALTH_STRATEGY      "node-health-strategy"
+#define PCMK__OPT_NODE_HEALTH_YELLOW        "node-health-yellow"
+
+// Constants for meta-attribute names
+#define PCMK__META_ALLOW_UNHEALTHY_NODES    "allow-unhealthy-nodes"
+
+// Constants for enumerated values for various options
+#define PCMK__VALUE_CUSTOM                  "custom"
+#define PCMK__VALUE_FENCING                 "fencing"
+#define PCMK__VALUE_GREEN                   "green"
+#define PCMK__VALUE_MIGRATE_ON_RED          "migrate-on-red"
+#define PCMK__VALUE_NONE                    "none"
+#define PCMK__VALUE_NOTHING                 "nothing"
+#define PCMK__VALUE_ONLY_GREEN              "only-green"
+#define PCMK__VALUE_PROGRESSIVE             "progressive"
+#define PCMK__VALUE_QUORUM                  "quorum"
+#define PCMK__VALUE_RED                     "red"
+#define PCMK__VALUE_UNFENCING               "unfencing"
+#define PCMK__VALUE_YELLOW                  "yellow"
+
+#endif // PCMK__OPTIONS_INTERNAL__H
