@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2020 the Pacemaker project contributors
+ * Copyright 2017-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -123,6 +123,29 @@ do {
     }                                                                           \
 } while (0)
 
+/* XML search strings for guest, remote and pacemaker_remote nodes */
+
+/* search string to find CIB resources entries for cluster nodes */
+#define PCMK__XP_MEMBER_NODE_CONFIG \
+    "//" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_NODES \
+    "/" XML_CIB_TAG_NODE "[not(@type) or @type='member']"
+
+/* search string to find CIB resources entries for guest nodes */
+#define PCMK__XP_GUEST_NODE_CONFIG \
+    "//" XML_TAG_CIB "//" XML_CIB_TAG_CONFIGURATION "//" XML_CIB_TAG_RESOURCE \
+    "//" XML_TAG_META_SETS "//" XML_CIB_TAG_NVPAIR \
+    "[@name='" XML_RSC_ATTR_REMOTE_NODE "']"
+
+/* search string to find CIB resources entries for remote nodes */
+#define PCMK__XP_REMOTE_NODE_CONFIG \
+    "//" XML_TAG_CIB "//" XML_CIB_TAG_CONFIGURATION "//" XML_CIB_TAG_RESOURCE \
+    "[@type='remote'][@provider='pacemaker']"
+
+/* search string to find CIB node status entries for pacemaker_remote nodes */
+#define PCMK__XP_REMOTE_NODE_STATUS \
+    "//" XML_TAG_CIB "//" XML_CIB_TAG_STATUS "//" XML_CIB_TAG_STATE \
+    "[@" XML_NODE_IS_REMOTE "='true']"
+
 enum pcmk__xml_artefact_ns {
     pcmk__xml_artefact_ns_legacy_rng = 1,
     pcmk__xml_artefact_ns_legacy_xslt,
@@ -133,9 +156,15 @@ enum pcmk__xml_artefact_ns {
 void pcmk__strip_xml_text(xmlNode *xml);
 const char *pcmk__xe_add_last_written(xmlNode *xe);
 
-xmlNode *pcmk__xe_match(xmlNode *parent, const char *node_name,
+xmlNode *pcmk__xe_match(const xmlNode *parent, const char *node_name,
                         const char *attr_n, const char *attr_v);
 
+void pcmk__xe_remove_matching_attrs(xmlNode *element,
+                                    bool (*match)(xmlAttrPtr, void *),
+                                    void *user_data);
+
+GString *pcmk__element_xpath(const xmlNode *xml);
+
 /*!
  * \internal
  * \brief Get the root directory to scan XML artefacts of given kind for
@@ -235,4 +264,106 @@ pcmk__xe_next(const xmlNode *child)
     return next;
 }
 
+/*!
+ * \internal
+ * \brief Like pcmk__xe_set_props, but takes a va_list instead of
+ *        arguments directly.
+ *
+ * \param[in,out] node   XML to add attributes to
+ * \param[in]     pairs  NULL-terminated list of name/value pairs to add
+ */
+void
+pcmk__xe_set_propv(xmlNodePtr node, va_list pairs);
+
+/*!
+ * \internal
+ * \brief Add a NULL-terminated list of name/value pairs to the given
+ *        XML node as properties.
+ *
+ * \param[in,out] node XML node to add properties to
+ * \param[in]     ...  NULL-terminated list of name/value pairs
+ *
+ * \note A NULL name terminates the arguments; a NULL value will be skipped.
+ */
+void
+pcmk__xe_set_props(xmlNodePtr node, ...)
+G_GNUC_NULL_TERMINATED;
+
+/*!
+ * \internal
+ * \brief Get first attribute of an XML element
+ *
+ * \param[in] xe  XML element to check
+ *
+ * \return First attribute of \p xe (or NULL if \p xe is NULL or has none)
+ */
+static inline xmlAttr *
+pcmk__xe_first_attr(const xmlNode *xe)
+{
+    return (xe == NULL)? NULL : xe->properties;
+}
+
+/*!
+ * \internal
+ * \brief Extract the ID attribute from an XML element
+ *
+ * \param[in] xpath String to search
+ * \param[in] node  Node to get the ID for
+ *
+ * \return ID attribute of \p node in xpath string \p xpath
+ */
+char *
+pcmk__xpath_node_id(const char *xpath, const char *node);
+
+/* internal XML-related utilities */
+
+enum xml_private_flags {
+     pcmk__xf_none        = 0x0000,
+     pcmk__xf_dirty       = 0x0001,
+     pcmk__xf_deleted     = 0x0002,
+     pcmk__xf_created     = 0x0004,
+     pcmk__xf_modified    = 0x0008,
+
+     pcmk__xf_tracking    = 0x0010,
+     pcmk__xf_processed   = 0x0020,
+     pcmk__xf_skip        = 0x0040,
+     pcmk__xf_moved       = 0x0080,
+
+     pcmk__xf_acl_enabled = 0x0100,
+     pcmk__xf_acl_read    = 0x0200,
+     pcmk__xf_acl_write   = 0x0400,
+     pcmk__xf_acl_deny    = 0x0800,
+
+     pcmk__xf_acl_create  = 0x1000,
+     pcmk__xf_acl_denied  = 0x2000,
+     pcmk__xf_lazy        = 0x4000,
+};
+
+void pcmk__set_xml_doc_flag(xmlNode *xml, enum xml_private_flags flag);
+
+/*!
+ * \internal
+ * \brief Iterate over child elements of \p xml
+ *
+ * This function iterates over the children of \p xml, performing the
+ * callback function \p handler on each node.  If the callback returns
+ * a value other than pcmk_rc_ok, the iteration stops and the value is
+ * returned.  It is therefore possible that not all children will be
+ * visited.
+ *
+ * \param[in,out] xml                 The starting XML node.  Can be NULL.
+ * \param[in]     child_element_name  The name that the node must match in order
+ *                                    for \p handler to be run.  If NULL, all
+ *                                    child elements will match.
+ * \param[in]     handler             The callback function.
+ * \param[in,out] userdata            User data to pass to the callback function.
+ *                                    Can be NULL.
+ *
+ * \return Standard Pacemaker return code
+ */
+int
+pcmk__xe_foreach_child(xmlNode *xml, const char *child_element_name,
+                       int (*handler)(xmlNode *xml, void *userdata),
+                       void *userdata);
+
 #endif // PCMK__XML_INTERNAL__H
