@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,8 +7,16 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PE_TYPES__H
-#  define PE_TYPES__H
+#ifndef PCMK__CRM_PENGINE_PE_TYPES__H
+#  define PCMK__CRM_PENGINE_PE_TYPES__H
+
+
+#  include <stdbool.h>              // bool
+#  include <sys/types.h>            // time_t
+#  include <libxml/tree.h>          // xmlNode
+#  include <glib.h>                 // gboolean, guint, GList, GHashTable
+#  include <crm/common/iso8601.h>
+#  include <crm/pengine/common.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -20,13 +28,6 @@ extern "C" {
  * \ingroup pengine
  */
 
-#  include <stdbool.h>              // bool
-#  include <sys/types.h>            // time_t
-#  include <glib.h>                 // gboolean, guint, GList, GHashTable
-#  include <crm/crm.h>              // GListPtr
-#  include <crm/common/iso8601.h>
-#  include <crm/pengine/common.h>
-
 typedef struct pe_node_s pe_node_t;
 typedef struct pe_action_s pe_action_t;
 typedef struct pe_resource_s pe_resource_t;
@@ -47,13 +48,14 @@ typedef struct resource_object_functions_s {
     /* parameter result must be free'd */
     char *(*parameter) (pe_resource_t*, pe_node_t*, gboolean, const char*,
                         pe_working_set_t*);
+    //! \deprecated will be removed in a future release
     void (*print) (pe_resource_t*, const char*, long, void*);
     gboolean (*active) (pe_resource_t*, gboolean);
     enum rsc_role_e (*state) (const pe_resource_t*, gboolean);
     pe_node_t *(*location) (const pe_resource_t*, GList**, int);
     void (*free) (pe_resource_t*);
     void (*count) (pe_resource_t*);
-    gboolean (*is_filtered) (pe_resource_t*, GListPtr, gboolean);
+    gboolean (*is_filtered) (pe_resource_t*, GList *, gboolean);
 } resource_object_functions_t;
 
 typedef struct resource_alloc_functions_s resource_alloc_functions_t;
@@ -67,15 +69,15 @@ enum pe_quorum_policy {
 };
 
 enum node_type {
-    node_ping,
+    node_ping,      //! \deprecated Do not use
     node_member,
     node_remote
 };
 
 //! \deprecated will be removed in a future release
 enum pe_restart {
-    pe_restart_restart,
-    pe_restart_ignore
+    pe_restart_restart, //! \deprecated will be removed in a future release
+    pe_restart_ignore   //! \deprecated will be removed in a future release
 };
 
 //! Determine behavior of pe_find_resource_with_flags()
@@ -104,7 +106,10 @@ enum pe_find {
 #  define pe_flag_stop_everything       0x00000400ULL
 
 #  define pe_flag_start_failure_fatal   0x00001000ULL
+
+//! \deprecated
 #  define pe_flag_remove_after_stop     0x00002000ULL
+
 #  define pe_flag_startup_fencing       0x00004000ULL
 #  define pe_flag_shutdown_lock         0x00008000ULL
 
@@ -114,6 +119,8 @@ enum pe_find {
 
 #  define pe_flag_quick_location        0x00100000ULL
 #  define pe_flag_sanitized             0x00200000ULL
+
+//! \deprecated
 #  define pe_flag_stdout                0x00400000ULL
 
 //! Don't count total, disabled and blocked resource instances
@@ -124,6 +131,15 @@ enum pe_find {
  */
 #  define pe_flag_no_compat             0x01000000ULL
 
+#  define pe_flag_show_scores           0x02000000ULL
+#  define pe_flag_show_utilization      0x04000000ULL
+
+/*!
+ * When scheduling, only unpack the CIB (including constraints), calculate
+ * as much cluster status as possible, and apply node health.
+ */
+#  define pe_flag_check_config          0x08000000ULL
+
 struct pe_working_set_s {
     xmlNode *input;
     crm_time_t *now;
@@ -145,14 +161,14 @@ struct pe_working_set_s {
     // Actions for which there can be only one (e.g. fence nodeX)
     GHashTable *singletons;
 
-    GListPtr nodes;
-    GListPtr resources;
-    GListPtr placement_constraints;
-    GListPtr ordering_constraints;
-    GListPtr colocation_constraints;
-    GListPtr ticket_constraints;
+    GList *nodes;
+    GList *resources;
+    GList *placement_constraints;
+    GList *ordering_constraints;
+    GList *colocation_constraints;
+    GList *ticket_constraints;
 
-    GListPtr actions;
+    GList *actions;
     xmlNode *failed;
     xmlNode *op_defaults;
     xmlNode *rsc_defaults;
@@ -179,6 +195,8 @@ struct pe_working_set_s {
     int ninstances;     // Total number of resource instances
     guint shutdown_lock;// How long (seconds) to lock resources to shutdown node
     int priority_fencing_delay; // Priority fencing delay
+
+    void *priv;
 };
 
 enum pe_check_parameters {
@@ -217,13 +235,14 @@ struct pe_node_shared_s {
 
     int num_resources;
     pe_resource_t *remote_rsc;
-    GListPtr running_rsc;       /* pe_resource_t* */
-    GListPtr allocated_rsc;     /* pe_resource_t* */
+    GList *running_rsc;       /* pe_resource_t* */
+    GList *allocated_rsc;     /* pe_resource_t* */
 
     GHashTable *attrs;          /* char* => char* */
     GHashTable *utilization;
     GHashTable *digest_cache;   //!< cache of calculated resource digests
     int priority; // calculated based on the priority of resources running on the node
+    pe_working_set_t *data_set; //!< Cluster that this node is part of
 };
 
 struct pe_node_s {
@@ -247,17 +266,24 @@ struct pe_node_s {
 #  define pe_rsc_provisional                0x00000100ULL
 #  define pe_rsc_allocating                 0x00000200ULL
 #  define pe_rsc_merging                    0x00000400ULL
+#  define pe_rsc_restarting                 0x00000800ULL
 
 #  define pe_rsc_stop                       0x00001000ULL
 #  define pe_rsc_reload                     0x00002000ULL
 #  define pe_rsc_allow_remote_remotes       0x00004000ULL
+#  define pe_rsc_critical                   0x00008000ULL
 
 #  define pe_rsc_failed                     0x00010000ULL
 #  define pe_rsc_runnable                   0x00040000ULL
 #  define pe_rsc_start_pending              0x00080000ULL
 
+//!< \deprecated Do not use
 #  define pe_rsc_starting                   0x00100000ULL
+
+//!< \deprecated Do not use
 #  define pe_rsc_stopping                   0x00200000ULL
+
+#  define pe_rsc_stop_unexpected            0x00400000ULL
 #  define pe_rsc_allow_migrate              0x00800000ULL
 
 #  define pe_rsc_failure_ignored            0x01000000ULL
@@ -268,13 +294,6 @@ struct pe_node_s {
 #  define pe_rsc_needs_fencing              0x20000000ULL
 #  define pe_rsc_needs_unfencing            0x40000000ULL
 
-enum pe_graph_flags {
-    pe_graph_none = 0x00000,
-    pe_graph_updated_first = 0x00001,
-    pe_graph_updated_then = 0x00002,
-    pe_graph_disable = 0x00004,
-};
-
 /* *INDENT-OFF* */
 enum pe_action_flags {
     pe_action_pseudo = 0x00001,
@@ -288,7 +307,9 @@ enum pe_action_flags {
 
     pe_action_dumped = 0x00100,
     pe_action_processed = 0x00200,
-    pe_action_clear = 0x00400,
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+    pe_action_clear = 0x00400, //! \deprecated Unused
+#endif
     pe_action_dangle = 0x00800,
 
     /* This action requires one or more of its dependencies to be runnable.
@@ -321,7 +342,6 @@ struct pe_resource_s {
 
     enum rsc_recovery_type recovery_type;
 
-    // @TODO only pe_restart_restart is of interest, so merge into flags
     enum pe_restart restart_type; //!< \deprecated will be removed in future release
 
     int priority;
@@ -340,17 +360,17 @@ struct pe_resource_s {
 
     //!@{
     //! This field should be treated as internal to Pacemaker
-    GListPtr rsc_cons_lhs;      // List of rsc_colocation_t*
-    GListPtr rsc_cons;          // List of rsc_colocation_t*
-    GListPtr rsc_location;      // List of pe__location_t*
-    GListPtr actions;           // List of pe_action_t*
-    GListPtr rsc_tickets;       // List of rsc_ticket*
+    GList *rsc_cons_lhs;      // List of pcmk__colocation_t*
+    GList *rsc_cons;          // List of pcmk__colocation_t*
+    GList *rsc_location;      // List of pe__location_t*
+    GList *actions;           // List of pe_action_t*
+    GList *rsc_tickets;       // List of rsc_ticket*
     //!@}
 
     pe_node_t *allocated_to;
     pe_node_t *partial_migration_target;
     pe_node_t *partial_migration_source;
-    GListPtr running_on;        /* pe_node_t*   */
+    GList *running_on;        /* pe_node_t*   */
     GHashTable *known_on;       /* pe_node_t*   */
     GHashTable *allowed_nodes;  /* pe_node_t*   */
 
@@ -358,32 +378,27 @@ struct pe_resource_s {
     enum rsc_role_e next_role;
 
     GHashTable *meta;
-    GHashTable *parameters;
+    GHashTable *parameters; //! \deprecated Use pe_rsc_params() instead
     GHashTable *utilization;
 
-    GListPtr children;          /* pe_resource_t*   */
-    GListPtr dangling_migrations;       /* pe_node_t*       */
+    GList *children;          /* pe_resource_t*   */
+    GList *dangling_migrations;       /* pe_node_t*       */
 
     pe_resource_t *container;
-    GListPtr fillers;
+    GList *fillers;
 
     pe_node_t *pending_node;    // Node on which pending_task is happening
     pe_node_t *lock_node;       // Resource is shutdown-locked to this node
     time_t lock_time;           // When shutdown lock started
 
-#if ENABLE_VERSIONED_ATTRS
-    xmlNode *versioned_parameters;
-#endif
+    /* Resource parameters may have node-attribute-based rules, which means the
+     * values can vary by node. This table is a cache of parameter name/value
+     * tables for each node (as needed). Use pe_rsc_params() to get the table
+     * for a given node.
+     */
+    GHashTable *parameter_cache; // Key = node name, value = parameters table
 };
 
-#if ENABLE_VERSIONED_ATTRS
-// Used as action->action_details if action->rsc is not NULL
-typedef struct pe_rsc_action_details_s {
-    xmlNode *versioned_parameters;
-    xmlNode *versioned_meta;
-} pe_rsc_action_details_t;
-#endif
-
 struct pe_action_s {
     int id;
     int priority;
@@ -425,8 +440,8 @@ struct pe_action_s {
      * to be considered runnable */ 
     int required_runnable_before;
 
-    GListPtr actions_before;    /* pe_action_wrapper_t* */
-    GListPtr actions_after;     /* pe_action_wrapper_t* */
+    GList *actions_before;    /* pe_action_wrapper_t* */
+    GList *actions_after;     /* pe_action_wrapper_t* */
 
     /* Some of the above fields could be moved to the details,
      * except for API backward compatibility.
@@ -444,7 +459,7 @@ typedef struct pe_ticket_s {
 
 typedef struct pe_tag_s {
     char *id;
-    GListPtr refs;
+    GList *refs;
 } pe_tag_t;
 
 //! Internal tracking for transition graph creation
@@ -468,7 +483,7 @@ enum pe_ordering {
 
     pe_order_implies_first         = 0x10,      /* If 'then' is required, ensure 'first' is too */
     pe_order_implies_then          = 0x20,      /* If 'first' is required, ensure 'then' is too */
-    pe_order_implies_first_master  = 0x40,      /* Imply 'first' is required when 'then' is required and then's rsc holds Master role. */
+    pe_order_promoted_implies_first = 0x40,     /* If 'then' is required and then's rsc is promoted, ensure 'first' becomes required too */
 
     /* first requires then to be both runnable and migrate runnable. */
     pe_order_implies_first_migratable  = 0x80,
@@ -486,7 +501,7 @@ enum pe_ordering {
                                                  */
 
     pe_order_restart               = 0x1000,    /* 'then' is runnable if 'first' is optional or runnable */
-    pe_order_stonith_stop          = 0x2000,    /* only applies if the action is non-pseudo */
+    pe_order_stonith_stop          = 0x2000,    //<! \deprecated Will be removed in future release
     pe_order_serialize_only        = 0x4000,    /* serialize */
     pe_order_same_node             = 0x8000,    /* applies only if 'first' and 'then' are on same node */
 
@@ -501,6 +516,11 @@ enum pe_ordering {
     pe_order_preserve              = 0x1000000, /* Hack for breaking user ordering constraints with container resources */
     pe_order_then_cancels_first    = 0x2000000, // if 'then' becomes required, 'first' becomes optional
     pe_order_trace                 = 0x4000000, /* test marker */
+
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+    // \deprecated Use pe_order_promoted_implies_first instead
+    pe_order_implies_first_master  = pe_order_promoted_implies_first,
+#endif
 };
 /* *INDENT-ON* */
 
@@ -510,30 +530,12 @@ typedef struct pe_action_wrapper_s {
     pe_action_t *action;
 } pe_action_wrapper_t;
 
-#ifndef PCMK__NO_COMPAT
-/* Everything here is deprecated and kept only for public API backward
- * compatibility. It will be moved to compatibility.h in a future release.
- */
-
-//!< \deprecated Use pe_action_t instead
-typedef struct pe_action_s action_t;
-//!< \deprecated Use pe_action_wrapper_t instead
-typedef struct pe_action_wrapper_s action_wrapper_t;
-//!< \deprecated Use pe_node_t instead
-typedef struct pe_node_s node_t;
-//!< \deprecated Use enum pe_quorum_policy instead
-typedef enum pe_quorum_policy no_quorum_policy_t;
-//!< \deprecated use pe_resource_t instead
-typedef struct pe_resource_s resource_t;
-//!< \deprecated Use pe_tag_t instead
-typedef struct pe_tag_s tag_t;
-//!< \deprecated Use pe_ticket_t instead
-typedef struct pe_ticket_s ticket_t;
-
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/pengine/pe_types_compat.h>
 #endif
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif // PE_TYPES__H
+#endif // PCMK__CRM_PENGINE_PE_TYPES__H
