@@ -18,15 +18,17 @@ extern "C" {
 #include <libxml/tree.h>            // xmlNode
 #include <crm/pengine/status.h>
 
-gboolean xml_contains_remote_node(xmlNode *xml);
-gboolean pe__is_remote_node(pe_node_t *node);
-gboolean pe__is_guest_node(pe_node_t *node);
-gboolean pe__is_guest_or_remote_node(pe_node_t *node);
-gboolean pe__resource_is_remote_conn(pe_resource_t *rsc, pe_working_set_t *data_set);
+bool xml_contains_remote_node(xmlNode *xml);
+bool pe__is_remote_node(const pe_node_t *node);
+bool pe__is_guest_node(const pe_node_t *node);
+bool pe__is_guest_or_remote_node(const pe_node_t *node);
+bool pe__is_bundle_node(const pe_node_t *node);
+bool pe__resource_is_remote_conn(const pe_resource_t *rsc,
+                                 const pe_working_set_t *data_set);
 pe_resource_t *pe__resource_contains_guest_node(const pe_working_set_t *data_set,
                                                 const pe_resource_t *rsc);
-void pe_foreach_guest_node(const pe_working_set_t *data_set, const node_t *host,
-                           void (*helper)(const node_t*, void*), void *user_data);
+void pe_foreach_guest_node(const pe_working_set_t *data_set, const pe_node_t *host,
+                           void (*helper)(const pe_node_t*, void*), void *user_data);
 xmlNode *pe_create_remote_xml(xmlNode *parent, const char *uname,
                               const char *container_id, const char *migrateable,
                               const char *is_managed, const char *start_timeout,
