@@ -42,6 +42,16 @@
 #  include <crm/common/output_internal.h>
 #  include <crm/common/xml_internal.h>
 #  include <crm/common/internal.h>
+#  include <locale.h>
+#  include <gettext.h>
+
+#define N_(String) (String)
+
+#ifdef ENABLE_NLS
+#  define _(String) gettext(String)
+#else
+#  define _(String) (String)
+#endif
 
 
 /*
@@ -66,6 +76,10 @@
 #define PCMK__XA_ATTR_VALUE             "attr_value"
 #define PCMK__XA_ATTR_VERSION           "attr_version"
 #define PCMK__XA_ATTR_WRITER            "attr_writer"
+#define PCMK__XA_CONFIG_ERRORS          "config-errors"
+#define PCMK__XA_CONFIG_WARNINGS        "config-warnings"
+#define PCMK__XA_GRAPH_ERRORS           "graph-errors"
+#define PCMK__XA_GRAPH_WARNINGS         "graph-warnings"
 #define PCMK__XA_MODE                   "mode"
 #define PCMK__XA_TASK                   "task"
 
@@ -95,11 +109,4 @@
 
 #define PCMK__CONTROLD_CMD_NODES        "list-nodes"
 
-/*
- * Environment variables used by Pacemaker
- */
-
-#define PCMK__ENV_PHYSICAL_HOST         "physical_host"
-
-
 #endif                          /* CRM_INTERNAL__H */
