@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -33,7 +33,8 @@ cib_process_query(const char *op, int options, const char *section, xmlNode * re
     xmlNode *obj_root = NULL;
     int result = pcmk_ok;
 
-    crm_trace("Processing \"%s\" event for section=%s", op, crm_str(section));
+    crm_trace("Processing %s for %s section",
+              op, pcmk__s(section, "unspecified"));
 
     if (options & cib_xpath) {
         return cib_process_xpath(op, options, section, req, input,
@@ -47,7 +48,7 @@ cib_process_query(const char *op, int options, const char *section, xmlNode * re
         section = NULL;
     }
 
-    obj_root = get_object_root(section, existing_cib);
+    obj_root = pcmk_find_cib_element(existing_cib, section);
 
     if (obj_root == NULL) {
         result = -ENXIO;
@@ -129,8 +130,8 @@ cib_process_bump(const char *op, int options, const char *section, xmlNode * req
 {
     int result = pcmk_ok;
 
-    crm_trace("Processing \"%s\" event for epoch=%s",
-              op, crm_str(crm_element_value(existing_cib, XML_ATTR_GENERATION)));
+    crm_trace("Processing %s for epoch='%s'", op,
+              pcmk__s(crm_element_value(existing_cib, XML_ATTR_GENERATION), ""));
 
     *answer = NULL;
     cib_update_counter(*result_cib, XML_ATTR_GENERATION, FALSE);
@@ -153,9 +154,11 @@ cib_update_counter(xmlNode * xml_obj, const char *field, gboolean reset)
         new_value = pcmk__itoa(++int_value);
     } else {
         new_value = strdup("1");
+        CRM_ASSERT(new_value != NULL);
     }
 
-    crm_trace("%s %d(%s)->%s", field, int_value, crm_str(old_value), crm_str(new_value));
+    crm_trace("Update %s from %s to %s",
+              field, pcmk__s(old_value, "unset"), new_value);
     crm_xml_add(xml_obj, field, new_value);
 
     free(new_value);
@@ -172,7 +175,8 @@ cib_process_replace(const char *op, int options, const char *section, xmlNode *
     const char *tag = NULL;
     int result = pcmk_ok;
 
-    crm_trace("Processing \"%s\" event for section=%s", op, crm_str(section));
+    crm_trace("Processing %s for %s section",
+              op, pcmk__s(section, "unspecified"));
 
     if (options & cib_xpath) {
         return cib_process_xpath(op, options, section, req, input,
@@ -265,7 +269,7 @@ cib_process_replace(const char *op, int options, const char *section, xmlNode *
         xmlNode *obj_root = NULL;
         gboolean ok = TRUE;
 
-        obj_root = get_object_root(section, *result_cib);
+        obj_root = pcmk_find_cib_element(*result_cib, section);
         ok = replace_xml_child(NULL, obj_root, input, FALSE);
         if (ok == FALSE) {
             crm_trace("No matching object to replace");
@@ -294,7 +298,7 @@ cib_process_delete(const char *op, int options, const char *section, xmlNode * r
         return -EINVAL;
     }
 
-    obj_root = get_object_root(section, *result_cib);
+    obj_root = pcmk_find_cib_element(*result_cib, section);
     if(pcmk__str_eq(crm_element_name(input), section, pcmk__str_casei)) {
         xmlNode *child = NULL;
         for (child = pcmk__xml_first_child(input); child;
@@ -329,20 +333,21 @@ cib_process_modify(const char *op, int options, const char *section, xmlNode * r
         return -EINVAL;
     }
 
-    obj_root = get_object_root(section, *result_cib);
+    obj_root = pcmk_find_cib_element(*result_cib, section);
     if (obj_root == NULL) {
         xmlNode *tmp_section = NULL;
-        const char *path = get_object_parent(section);
+        const char *path = pcmk_cib_parent_name_for(section);
 
         if (path == NULL) {
             return -EINVAL;
         }
 
         tmp_section = create_xml_node(NULL, section);
-        cib_process_xpath(CIB_OP_CREATE, 0, path, NULL, tmp_section, NULL, result_cib, answer);
+        cib_process_xpath(PCMK__CIB_REQUEST_CREATE, 0, path, NULL, tmp_section,
+                          NULL, result_cib, answer);
         free_xml(tmp_section);
 
-        obj_root = get_object_root(section, *result_cib);
+        obj_root = pcmk_find_cib_element(*result_cib, section);
     }
 
     CRM_CHECK(obj_root != NULL, return -EINVAL);
@@ -395,7 +400,9 @@ update_cib_object(xmlNode * parent, xmlNode * update)
     CRM_CHECK(object_name != NULL, return -EINVAL);
 
     object_id = ID(update);
-    crm_trace("Processing: <%s id=%s>", crm_str(object_name), crm_str(object_id));
+    crm_trace("Processing update for <%s%s%s%s>", object_name,
+              ((object_id == NULL)? "" : " id='"), pcmk__s(object_id, ""),
+              ((object_id == NULL)? "" : "'"));
 
     if (object_id == NULL) {
         /*  placeholder object */
@@ -409,7 +416,9 @@ update_cib_object(xmlNode * parent, xmlNode * update)
         target = create_xml_node(parent, object_name);
     }
 
-    crm_trace("Found node <%s id=%s> to update", crm_str(object_name), crm_str(object_id));
+    crm_trace("Found node <%s%s%s%s> to update", object_name,
+              ((object_id == NULL)? "" : " id='"), pcmk__s(object_id, ""),
+              ((object_id == NULL)? "" : "'"));
 
     replace = crm_element_value(update, XML_CIB_ATTR_REPLACE);
     if (replace != NULL) {
@@ -447,19 +456,32 @@ update_cib_object(xmlNode * parent, xmlNode * update)
 
     copy_in_properties(target, update);
 
-    crm_trace("Processing children of <%s id=%s>", crm_str(object_name), crm_str(object_id));
+    if (xml_acl_denied(target)) {
+        crm_notice("Cannot update <%s id=%s>", pcmk__s(object_name, "<null>"), pcmk__s(object_id, "<null>"));
+        return -EACCES;
+    }
+
+    crm_trace("Processing children of <%s%s%s%s>", object_name,
+              ((object_id == NULL)? "" : " id='"), pcmk__s(object_id, ""),
+              ((object_id == NULL)? "" : "'"));
 
     for (a_child = pcmk__xml_first_child(update); a_child != NULL;
          a_child = pcmk__xml_next(a_child)) {
         int tmp_result = 0;
 
-        crm_trace("Updating child <%s id=%s>", crm_element_name(a_child), ID(a_child));
+        crm_trace("Updating child <%s%s%s%s>", crm_element_name(a_child),
+                  ((ID(a_child) == NULL)? "" : " id='"),
+                  pcmk__s(ID(a_child), ""), ((ID(a_child) == NULL)? "" : "'"));
 
         tmp_result = update_cib_object(target, a_child);
 
         /*  only the first error is likely to be interesting */
         if (tmp_result != pcmk_ok) {
-            crm_err("Error updating child <%s id=%s>", crm_element_name(a_child), ID(a_child));
+            crm_err("Error updating child <%s%s%s%s>",
+                    crm_element_name(a_child),
+                    ((ID(a_child) == NULL)? "" : " id='"),
+                    pcmk__s(ID(a_child), ""),
+                    ((ID(a_child) == NULL)? "" : "'"));
 
             if (result == pcmk_ok) {
                 result = tmp_result;
@@ -467,7 +489,9 @@ update_cib_object(xmlNode * parent, xmlNode * update)
         }
     }
 
-    crm_trace("Finished with <%s id=%s>", crm_str(object_name), crm_str(object_id));
+    crm_trace("Finished handling update for <%s%s%s%s>", object_name,
+              ((object_id == NULL)? "" : " id='"), pcmk__s(object_id, ""),
+              ((object_id == NULL)? "" : "'"));
 
     return result;
 }
@@ -475,44 +499,36 @@ update_cib_object(xmlNode * parent, xmlNode * update)
 static int
 add_cib_object(xmlNode * parent, xmlNode * new_obj)
 {
-    int result = pcmk_ok;
     const char *object_name = NULL;
     const char *object_id = NULL;
     xmlNode *equiv_node = NULL;
 
-    if (new_obj != NULL) {
-        object_name = crm_element_name(new_obj);
+    if ((parent == NULL) || (new_obj == NULL)) {
+        return -EINVAL;
     }
-    object_id = crm_element_value(new_obj, XML_ATTR_ID);
 
-    crm_trace("Processing: <%s id=%s>", crm_str(object_name), crm_str(object_id));
+    object_name = crm_element_name(new_obj);
+    if (object_name == NULL) {
+        return -EINVAL;
+    }
 
-    if (new_obj == NULL || object_name == NULL) {
-        result = -EINVAL;
+    object_id = ID(new_obj);
 
-    } else if (parent == NULL) {
-        result = -EINVAL;
+    crm_trace("Processing creation of <%s%s%s%s>", object_name,
+              ((object_id == NULL)? "" : " id='"), pcmk__s(object_id, ""),
+              ((object_id == NULL)? "" : "'"));
 
-    } else if (object_id == NULL) {
-        /*  placeholder object */
+    if (object_id == NULL) {
         equiv_node = find_xml_node(parent, object_name, FALSE);
-
     } else {
         equiv_node = pcmk__xe_match(parent, object_name, XML_ATTR_ID,
                                     object_id);
     }
-
-    if (result != pcmk_ok) {
-        ;                       /* do nothing */
-
-    } else if (equiv_node != NULL) {
-        result = -EEXIST;
-
-    } else {
-        result = update_cib_object(parent, new_obj);
+    if (equiv_node != NULL) {
+        return -EEXIST;
     }
 
-    return result;
+    return update_cib_object(parent, new_obj);
 }
 
 int
@@ -523,7 +539,8 @@ cib_process_create(const char *op, int options, const char *section, xmlNode * r
     int result = pcmk_ok;
     xmlNode *update_section = NULL;
 
-    crm_trace("Processing \"%s\" event for section=%s", op, crm_str(section));
+    crm_trace("Processing %s for %s section",
+              op, pcmk__s(section, "unspecified"));
     if (pcmk__str_eq(XML_CIB_TAG_SECTION_ALL, section, pcmk__str_casei)) {
         section = NULL;
 
@@ -534,7 +551,7 @@ cib_process_create(const char *op, int options, const char *section, xmlNode * r
         section = NULL;
     }
 
-    CRM_CHECK(strcasecmp(CIB_OP_CREATE, op) == 0, return -EINVAL);
+    CRM_CHECK(strcmp(op, PCMK__CIB_REQUEST_CREATE) == 0, return -EINVAL);
 
     if (input == NULL) {
         crm_err("Cannot perform modification with no data");
@@ -548,7 +565,7 @@ cib_process_create(const char *op, int options, const char *section, xmlNode * r
 
     failed = create_xml_node(NULL, XML_TAG_FAILED);
 
-    update_section = get_object_root(section, *result_cib);
+    update_section = pcmk_find_cib_element(*result_cib, section);
     if (pcmk__str_eq(crm_element_name(input), section, pcmk__str_casei)) {
         xmlNode *a_child = NULL;
 
@@ -672,13 +689,14 @@ cib_config_changed(xmlNode * last, xmlNode * next, xmlNode ** diff)
 }
 
 int
-cib_process_xpath(const char *op, int options, const char *section, xmlNode * req, xmlNode * input,
-                  xmlNode * existing_cib, xmlNode ** result_cib, xmlNode ** answer)
+cib_process_xpath(const char *op, int options, const char *section,
+                  const xmlNode *req, xmlNode *input, xmlNode *existing_cib,
+                  xmlNode **result_cib, xmlNode **answer)
 {
     int lpc = 0;
     int max = 0;
     int rc = pcmk_ok;
-    gboolean is_query = pcmk__str_eq(op, CIB_OP_QUERY, pcmk__str_casei);
+    bool is_query = pcmk__str_eq(op, PCMK__CIB_REQUEST_QUERY, pcmk__str_none);
 
     xmlXPathObjectPtr xpathObj = NULL;
 
@@ -692,7 +710,8 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
 
     max = numXpathResults(xpathObj);
 
-    if (max < 1 && pcmk__str_eq(op, CIB_OP_DELETE, pcmk__str_casei)) {
+    if ((max < 1)
+        && pcmk__str_eq(op, PCMK__CIB_REQUEST_DELETE, pcmk__str_none)) {
         crm_debug("%s was already removed", section);
 
     } else if (max < 1) {
@@ -705,7 +724,8 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
         }
     }
 
-    if (pcmk__str_eq(op, CIB_OP_DELETE, pcmk__str_casei) && (options & cib_multiple)) {
+    if (pcmk_is_set(options, cib_multiple)
+        && pcmk__str_eq(op, PCMK__CIB_REQUEST_DELETE, pcmk__str_none)) {
         dedupXpathResults(xpathObj);
     }
 
@@ -721,7 +741,7 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
         crm_debug("Processing %s op for %s with %s", op, section, path);
         free(path);
 
-        if (pcmk__str_eq(op, CIB_OP_DELETE, pcmk__str_casei)) {
+        if (pcmk__str_eq(op, PCMK__CIB_REQUEST_DELETE, pcmk__str_none)) {
             if (match == *result_cib) {
                 /* Attempting to delete the whole "/cib" */
                 crm_warn("Cannot perform %s for %s: The xpath is addressing the whole /cib", op, section);
@@ -734,18 +754,18 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
                 break;
             }
 
-        } else if (pcmk__str_eq(op, CIB_OP_MODIFY, pcmk__str_casei)) {
+        } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_MODIFY, pcmk__str_none)) {
             if (update_xml_child(match, input) == FALSE) {
                 rc = -ENXIO;
             } else if ((options & cib_multiple) == 0) {
                 break;
             }
 
-        } else if (pcmk__str_eq(op, CIB_OP_CREATE, pcmk__str_casei)) {
+        } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_CREATE, pcmk__str_none)) {
             add_node_copy(match, input);
             break;
 
-        } else if (pcmk__str_eq(op, CIB_OP_QUERY, pcmk__str_casei)) {
+        } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_QUERY, pcmk__str_none)) {
 
             if (options & cib_no_children) {
                 const char *tag = TYPE(match);
@@ -793,7 +813,8 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
                 *answer = match;
             }
 
-        } else if (pcmk__str_eq(op, CIB_OP_REPLACE, pcmk__str_casei)) {
+        } else if (pcmk__str_eq(op, PCMK__CIB_REQUEST_REPLACE,
+                                pcmk__str_none)) {
             xmlNode *parent = match->parent;
 
             free_xml(match);
