@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -226,9 +226,10 @@ pcmk__write_series_sequence(const char *directory, const char *series,
  * \internal
  * \brief Change the owner and group of a file series' .last file
  *
- * \param[in] dir  Directory that contains series
- * \param[in] uid  User ID of desired file owner
- * \param[in] gid  Group ID of desired file group
+ * \param[in] directory  Directory that contains series
+ * \param[in] series     Series to change
+ * \param[in] uid        User ID of desired file owner
+ * \param[in] gid        Group ID of desired file group
  *
  * \return Standard Pacemaker return code
  * \note The caller must have the appropriate privileges.
@@ -260,7 +261,7 @@ pcmk__daemon_user_can_write(const char *target_name, struct stat *target_stat)
     sys_user = getpwnam(CRM_DAEMON_USER);
     if (sys_user == NULL) {
         crm_notice("Could not find user %s: %s",
-                   CRM_DAEMON_USER, pcmk_strerror(errno));
+                   CRM_DAEMON_USER, pcmk_rc_str(errno));
         return FALSE;
     }
     if (target_stat->st_uid != sys_user->pw_uid) {
@@ -288,7 +289,7 @@ pcmk__daemon_group_can_write(const char *target_name, struct stat *target_stat)
     sys_grp = getgrnam(CRM_DAEMON_GROUP);
     if (sys_grp == NULL) {
         crm_notice("Could not find group %s: %s",
-                   CRM_DAEMON_GROUP, pcmk_strerror(errno));
+                   CRM_DAEMON_GROUP, pcmk_rc_str(errno));
         return FALSE;
     }
 
@@ -341,7 +342,7 @@ pcmk__daemon_can_write(const char *dir, const char *file)
 
         s_res = stat(full_file, &buf);
         if (s_res < 0) {
-            crm_notice("%s not found: %s", target, pcmk_strerror(errno));
+            crm_notice("%s not found: %s", target, pcmk_rc_str(errno));
             free(full_file);
             full_file = NULL;
             target = NULL;
@@ -359,7 +360,7 @@ pcmk__daemon_can_write(const char *dir, const char *file)
         target = dir;
         s_res = stat(dir, &buf);
         if (s_res < 0) {
-            crm_err("%s not found: %s", dir, pcmk_strerror(errno));
+            crm_err("%s not found: %s", dir, pcmk_rc_str(errno));
             return false;
 
         } else if (S_ISDIR(buf.st_mode) == FALSE) {
@@ -537,7 +538,7 @@ pcmk__set_nonblocking(int fd)
  * \return Name of directory to be used for temporary files
  */
 const char *
-pcmk__get_tmpdir()
+pcmk__get_tmpdir(void)
 {
     const char *dir = getenv("TMPDIR");
 
@@ -578,14 +579,14 @@ pcmk__close_fds_in_child(bool all)
      * Use this if available, because it's more efficient than a shotgun
      * approach to closing descriptors.
      */
-#if SUPPORT_PROCFS
+#if HAVE_LINUX_PROCFS
     dir = opendir("/proc/self/fd");
     if (dir == NULL) {
         dir = opendir("/dev/fd");
     }
 #else
     dir = opendir("/dev/fd");
-#endif
+#endif // HAVE_LINUX_PROCFS
     if (dir != NULL) {
         struct dirent *entry;
         int dir_fd = dirfd(dir);
