@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -49,6 +49,15 @@ struct trigger_s {
 
 };
 
+struct mainloop_timer_s {
+        guint id;
+        guint period_ms;
+        bool repeat;
+        char *name;
+        GSourceFunc cb;
+        void *userdata;
+};
+
 static gboolean
 crm_trigger_prepare(GSource * source, gint * timeout)
 {
@@ -84,24 +93,36 @@ crm_trigger_check(GSource * source)
     return trig->trigger;
 }
 
+/*!
+ * \internal
+ * \brief GSource dispatch function for crm_trigger_t
+ *
+ * \param[in] source        crm_trigger_t being dispatched
+ * \param[in] callback      Callback passed at source creation
+ * \param[in,out] userdata  User data passed at source creation
+ *
+ * \return G_SOURCE_REMOVE to remove source, G_SOURCE_CONTINUE to keep it
+ */
 static gboolean
-crm_trigger_dispatch(GSource * source, GSourceFunc callback, gpointer userdata)
+crm_trigger_dispatch(GSource *source, GSourceFunc callback, gpointer userdata)
 {
-    int rc = TRUE;
+    gboolean rc = G_SOURCE_CONTINUE;
     crm_trigger_t *trig = (crm_trigger_t *) source;
 
     if (trig->running) {
         /* Wait until the existing job is complete before starting the next one */
-        return TRUE;
+        return G_SOURCE_CONTINUE;
     }
     trig->trigger = FALSE;
 
     if (callback) {
-        rc = callback(trig->user_data);
-        if (rc < 0) {
+        int callback_rc = callback(trig->user_data);
+
+        if (callback_rc < 0) {
             crm_trace("Trigger handler %p not yet complete", trig);
             trig->running = TRUE;
-            rc = TRUE;
+        } else if (callback_rc == 0) {
+            rc = G_SOURCE_REMOVE;
         }
     }
     return rc;
@@ -150,13 +171,21 @@ mainloop_trigger_complete(crm_trigger_t * trig)
     trig->running = FALSE;
 }
 
-/* If dispatch returns:
- *  -1: Job running but not complete
- *   0: Remove the trigger from mainloop
- *   1: Leave the trigger in mainloop
+/*!
+ * \brief Create a trigger to be used as a mainloop source
+ *
+ * \param[in] priority  Relative priority of source (lower number is higher priority)
+ * \param[in] dispatch  Trigger dispatch function (should return 0 to remove the
+ *                      trigger from the mainloop, -1 if the trigger should be
+ *                      kept but the job is still running and not complete, and
+ *                      1 if the trigger should be kept and the job is complete)
+ * \param[in] userdata  Pointer to pass to \p dispatch
+ *
+ * \return Newly allocated mainloop source for trigger
  */
 crm_trigger_t *
-mainloop_add_trigger(int priority, int (*dispatch) (gpointer user_data), gpointer userdata)
+mainloop_add_trigger(int priority, int (*dispatch) (gpointer user_data),
+                     gpointer userdata)
 {
     GSource *source = NULL;
 
@@ -222,7 +251,7 @@ static crm_signal_t *crm_signals[NSIG];
  * \param[in] userdata  (ignored)
  */
 static gboolean
-crm_signal_dispatch(GSource * source, GSourceFunc callback, gpointer userdata)
+crm_signal_dispatch(GSource *source, GSourceFunc callback, gpointer userdata)
 {
     crm_signal_t *sig = (crm_signal_t *) source;
 
@@ -285,7 +314,7 @@ crm_signal_handler(int sig, sighandler_t dispatch)
 
     if (sigemptyset(&mask) < 0) {
         crm_err("Could not set handler for signal %d: %s",
-                sig, pcmk_strerror(errno));
+                sig, pcmk_rc_str(errno));
         return SIG_ERR;
     }
 
@@ -296,7 +325,7 @@ crm_signal_handler(int sig, sighandler_t dispatch)
 
     if (sigaction(sig, &sa, &old) < 0) {
         crm_err("Could not set handler for signal %d: %s",
-                sig, pcmk_strerror(errno));
+                sig, pcmk_rc_str(errno));
         return SIG_ERR;
     }
     return old.sa_handler;
@@ -462,22 +491,11 @@ gio_poll_destroy(gpointer data)
 static gint
 conv_prio_libqb2glib(enum qb_loop_priority prio)
 {
-    gint ret = G_PRIORITY_DEFAULT;
     switch (prio) {
-        case QB_LOOP_LOW:
-            ret = G_PRIORITY_LOW;
-            break;
-        case QB_LOOP_HIGH:
-            ret = G_PRIORITY_HIGH;
-            break;
-        default:
-            crm_trace("Invalid libqb's loop priority %d, assuming QB_LOOP_MED",
-                      prio);
-            /* fall-through */
-        case QB_LOOP_MED:
-            break;
+        case QB_LOOP_LOW:   return G_PRIORITY_LOW;
+        case QB_LOOP_HIGH:  return G_PRIORITY_HIGH;
+        default:            return G_PRIORITY_DEFAULT; // QB_LOOP_MED
     }
-    return ret;
 }
 
 /*!
@@ -487,27 +505,16 @@ conv_prio_libqb2glib(enum qb_loop_priority prio)
  * \param[in] prio  libqb's poll priority (#QB_LOOP_MED assumed as fallback)
  *
  * \return  best matching rate limiting spec
+ * \note This is the inverse of libqb's qb_ipcs_request_rate_limit().
  */
 static enum qb_ipcs_rate_limit
 conv_libqb_prio2ratelimit(enum qb_loop_priority prio)
 {
-    /* this is an inversion of what libqb's qb_ipcs_request_rate_limit does */
-    enum qb_ipcs_rate_limit ret = QB_IPCS_RATE_NORMAL;
     switch (prio) {
-        case QB_LOOP_LOW:
-            ret = QB_IPCS_RATE_SLOW;
-            break;
-        case QB_LOOP_HIGH:
-            ret = QB_IPCS_RATE_FAST;
-            break;
-        default:
-            crm_trace("Invalid libqb's loop priority %d, assuming QB_LOOP_MED",
-                      prio);
-            /* fall-through */
-        case QB_LOOP_MED:
-            break;
+        case QB_LOOP_LOW:   return QB_IPCS_RATE_SLOW;
+        case QB_LOOP_HIGH:  return QB_IPCS_RATE_FAST;
+        default:            return QB_IPCS_RATE_NORMAL; // QB_LOOP_MED
     }
-    return ret;
 }
 
 static int32_t
@@ -669,16 +676,14 @@ mainloop_add_ipc_server_with_prio(const char *name, enum qb_ipc_type type,
         qb_ipcs_request_rate_limit(server, conv_libqb_prio2ratelimit(prio));
     }
 
-#ifdef HAVE_IPCS_GET_BUFFER_SIZE
     /* All clients should use at least ipc_buffer_max as their buffer size */
     qb_ipcs_enforce_buffer_size(server, crm_ipc_default_buffer_size());
-#endif
-
     qb_ipcs_poll_handlers_set(server, &gio_poll_funcs);
 
     rc = qb_ipcs_run(server);
     if (rc < 0) {
         crm_err("Could not start %s IPC server: %s (%d)", name, pcmk_strerror(rc), rc);
+        qb_ipcs_destroy(server);
         return NULL;
     }
 
@@ -708,43 +713,56 @@ struct mainloop_io_s {
 
 };
 
+/*!
+ * \internal
+ * \brief I/O watch callback function (GIOFunc)
+ *
+ * \param[in] gio        I/O channel being watched
+ * \param[in] condition  I/O condition satisfied
+ * \param[in] data       User data passed when source was created
+ *
+ * \return G_SOURCE_REMOVE to remove source, G_SOURCE_CONTINUE to keep it
+ */
 static gboolean
-mainloop_gio_callback(GIOChannel * gio, GIOCondition condition, gpointer data)
+mainloop_gio_callback(GIOChannel *gio, GIOCondition condition, gpointer data)
 {
-    gboolean keep = TRUE;
+    gboolean rc = G_SOURCE_CONTINUE;
     mainloop_io_t *client = data;
 
     CRM_ASSERT(client->fd == g_io_channel_unix_get_fd(gio));
 
     if (condition & G_IO_IN) {
         if (client->ipc) {
-            long rc = 0;
+            long read_rc = 0L;
             int max = 10;
 
             do {
-                rc = crm_ipc_read(client->ipc);
-                if (rc <= 0) {
-                    crm_trace("Message acquisition from %s[%p] failed: %s (%ld)",
-                              client->name, client, pcmk_strerror(rc), rc);
+                read_rc = crm_ipc_read(client->ipc);
+                if (read_rc <= 0) {
+                    crm_trace("Could not read IPC message from %s: %s (%ld)",
+                              client->name, pcmk_strerror(read_rc), read_rc);
 
                 } else if (client->dispatch_fn_ipc) {
                     const char *buffer = crm_ipc_buffer(client->ipc);
 
-                    crm_trace("New message from %s[%p] = %ld (I/O condition=%d)", client->name, client, rc, condition);
-                    if (client->dispatch_fn_ipc(buffer, rc, client->userdata) < 0) {
+                    crm_trace("New %ld-byte IPC message from %s "
+                              "after I/O condition %d",
+                              read_rc, client->name, (int) condition);
+                    if (client->dispatch_fn_ipc(buffer, read_rc, client->userdata) < 0) {
                         crm_trace("Connection to %s no longer required", client->name);
-                        keep = FALSE;
+                        rc = G_SOURCE_REMOVE;
                     }
                 }
 
-            } while (keep && rc > 0 && --max > 0);
+            } while ((rc == G_SOURCE_CONTINUE) && (read_rc > 0) && --max > 0);
 
         } else {
-            crm_trace("New message from %s[%p] %u", client->name, client, condition);
+            crm_trace("New I/O event for %s after I/O condition %d",
+                      client->name, (int) condition);
             if (client->dispatch_fn_io) {
                 if (client->dispatch_fn_io(client->userdata) < 0) {
                     crm_trace("Connection to %s no longer required", client->name);
-                    keep = FALSE;
+                    rc = G_SOURCE_REMOVE;
                 }
             }
         }
@@ -753,12 +771,12 @@ mainloop_gio_callback(GIOChannel * gio, GIOCondition condition, gpointer data)
     if (client->ipc && crm_ipc_connected(client->ipc) == FALSE) {
         crm_err("Connection to %s closed " CRM_XS "client=%p condition=%d",
                 client->name, client, condition);
-        keep = FALSE;
+        rc = G_SOURCE_REMOVE;
 
     } else if (condition & (G_IO_HUP | G_IO_NVAL | G_IO_ERR)) {
         crm_trace("The connection %s[%p] has been closed (I/O condition=%d)",
                   client->name, client, condition);
-        keep = FALSE;
+        rc = G_SOURCE_REMOVE;
 
     } else if ((condition & G_IO_IN) == 0) {
         /*
@@ -791,10 +809,10 @@ mainloop_gio_callback(GIOChannel * gio, GIOCondition condition, gpointer data)
         crm_err("Strange condition: %d", condition);
     }
 
-    /* keep == FALSE results in mainloop_gio_destroy() being called
+    /* G_SOURCE_REMOVE results in mainloop_gio_destroy() being called
      * just before the source is removed from mainloop
      */
-    return keep;
+    return rc;
 }
 
 static void
@@ -837,11 +855,11 @@ mainloop_gio_destroy(gpointer c)
 /*!
  * \brief Connect to IPC and add it as a main loop source
  *
- * \param[in]  ipc        IPC connection to add
- * \param[in]  priority   Event source priority to use for connection
- * \param[in]  userdata   Data to register with callbacks
- * \param[in]  callbacks  Dispatch and destroy callbacks for connection
- * \param[out] source     Newly allocated event source
+ * \param[in,out] ipc        IPC connection to add
+ * \param[in]     priority   Event source priority to use for connection
+ * \param[in]     userdata   Data to register with callbacks
+ * \param[in]     callbacks  Dispatch and destroy callbacks for connection
+ * \param[out]    source     Newly allocated event source
  *
  * \return Standard Pacemaker return code
  *
@@ -854,13 +872,15 @@ mainloop_gio_destroy(gpointer c)
  */
 int
 pcmk__add_mainloop_ipc(crm_ipc_t *ipc, int priority, void *userdata,
-                       struct ipc_client_callbacks *callbacks,
+                       const struct ipc_client_callbacks *callbacks,
                        mainloop_io_t **source)
 {
     CRM_CHECK((ipc != NULL) && (callbacks != NULL), return EINVAL);
 
     if (!crm_ipc_connect(ipc)) {
-        return ENOTCONN;
+        int rc = errno;
+        crm_debug("Connection to %s failed: %d", crm_ipc_name(ipc), errno);
+        return rc;
     }
     *source = mainloop_add_fd(crm_ipc_name(ipc), priority, crm_ipc_get_fd(ipc),
                               userdata, NULL);
@@ -876,6 +896,22 @@ pcmk__add_mainloop_ipc(crm_ipc_t *ipc, int priority, void *userdata,
     return pcmk_rc_ok;
 }
 
+/*!
+ * \brief Get period for mainloop timer
+ *
+ * \param[in]  timer      Timer
+ *
+ * \return Period in ms
+ */
+guint
+pcmk__mainloop_timer_get_period(const mainloop_timer_t *timer)
+{
+    if (timer) {
+        return timer->period_ms;
+    }
+    return 0;
+}
+
 mainloop_io_t *
 mainloop_add_ipc_client(const char *name, int priority, size_t max_size,
                         void *userdata, struct ipc_client_callbacks *callbacks)
@@ -891,6 +927,11 @@ mainloop_add_ipc_client(const char *name, int priority, size_t max_size,
                     name, pcmk_rc_str(rc));
         }
         crm_ipc_destroy(ipc);
+        if (rc > 0) {
+            errno = rc;
+        } else {
+            errno = ENOTCONN;
+        }
         return NULL;
     }
     return source;
@@ -969,7 +1010,7 @@ mainloop_del_fd(mainloop_io_t * client)
     }
 }
 
-static GListPtr child_list = NULL;
+static GList *child_list = NULL;
 
 pid_t
 mainloop_child_pid(mainloop_child_t * child)
@@ -1044,7 +1085,7 @@ child_timeout_callback(gpointer p)
 
     child->timerid = 0;
     if (child->timeout) {
-        crm_crit("%s process (PID %d) will not die!", child->desc, (int)child->pid);
+        crm_warn("%s process (PID %d) will not die!", child->desc, (int)child->pid);
         return FALSE;
     }
 
@@ -1055,7 +1096,7 @@ child_timeout_callback(gpointer p)
     }
 
     child->timeout = TRUE;
-    crm_warn("%s process (PID %d) timed out", child->desc, (int)child->pid);
+    crm_debug("%s process (PID %d) timed out", child->desc, (int)child->pid);
 
     child->timerid = g_timeout_add(5000, child_timeout_callback, child);
     return FALSE;
@@ -1090,7 +1131,7 @@ child_waitpid(mainloop_child_t *child, int flags)
         signo = SIGCHLD;
         exitcode = 1;
         crm_notice("Wait for child process %d (%s) interrupted: %s",
-                   child->pid, child->desc, pcmk_strerror(errno));
+                   child->pid, child->desc, pcmk_rc_str(errno));
 
     } else if (WIFEXITED(status)) {
         exitcode = WEXITSTATUS(status);
@@ -1154,7 +1195,7 @@ child_signal_init(gpointer p)
 gboolean
 mainloop_child_kill(pid_t pid)
 {
-    GListPtr iter;
+    GList *iter;
     mainloop_child_t *child = NULL;
     mainloop_child_t *match = NULL;
     /* It is impossible to block SIGKILL, this allows us to
@@ -1212,7 +1253,7 @@ mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *pr
                    void (*callback) (mainloop_child_t * p, pid_t pid, int core, int signo, int exitcode))
 {
     static bool need_init = TRUE;
-    mainloop_child_t *child = g_new(mainloop_child_t, 1);
+    mainloop_child_t *child = calloc(1, sizeof(mainloop_child_t));
 
     child->pid = pid;
     child->timerid = 0;
@@ -1220,10 +1261,7 @@ mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *pr
     child->privatedata = privatedata;
     child->callback = callback;
     child->flags = flags;
-
-    if(desc) {
-        child->desc = strdup(desc);
-    }
+    pcmk__str_update(&child->desc, desc);
 
     if (timeout) {
         child->timerid = g_timeout_add(timeout, child_timeout_callback, child);
@@ -1248,15 +1286,6 @@ mainloop_child_add(pid_t pid, int timeout, const char *desc, void *privatedata,
     mainloop_child_add_with_flags(pid, timeout, desc, privatedata, 0, callback);
 }
 
-struct mainloop_timer_s {
-        guint id;
-        guint period_ms;
-        bool repeat;
-        char *name;
-        GSourceFunc cb;
-        void *userdata;
-};
-
 static gboolean
 mainloop_timer_cb(gpointer user_data)
 {
@@ -1381,8 +1410,8 @@ drain_timeout_cb(gpointer user_data)
 /*!
  * \brief Drain some remaining main loop events then quit it
  *
- * \param[in] mloop  Main loop to drain and quit
- * \param[in] n      Drain up to this many pending events
+ * \param[in,out] mloop  Main loop to drain and quit
+ * \param[in]     n      Drain up to this many pending events
  */
 void
 pcmk_quit_main_loop(GMainLoop *mloop, unsigned int n)
@@ -1403,9 +1432,10 @@ pcmk_quit_main_loop(GMainLoop *mloop, unsigned int n)
 /*!
  * \brief Process main loop events while a certain condition is met
  *
- * \param[in] mloop     Main loop to process
- * \param[in] timer_ms  Don't process longer than this amount of time
- * \param[in] check     Function that returns TRUE if events should be processed
+ * \param[in,out] mloop     Main loop to process
+ * \param[in]     timer_ms  Don't process longer than this amount of time
+ * \param[in]     check     Function that returns true if events should be
+ *                          processed
  *
  * \note This function is intended to be called at shutdown if certain important
  *       events should not be missed. The caller would likely quit the main loop
@@ -1437,15 +1467,15 @@ pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms, bool (*check)(guint))
 }
 
 // Deprecated functions kept only for backward API compatibility
-gboolean crm_signal(int sig, void (*dispatch) (int sig));
+// LCOV_EXCL_START
+
+#include <crm/common/mainloop_compat.h>
 
-/*
- * \brief Use crm_signal_handler() instead
- * \deprecated
- */
 gboolean
 crm_signal(int sig, void (*dispatch) (int sig))
 {
     return crm_signal_handler(sig, dispatch) != SIG_ERR;
 }
 
+// LCOV_EXCL_STOP
+// End deprecated API
