@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -20,7 +20,6 @@
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
-#include <crm/common/iso8601_internal.h>
 #include "crmcommon_private.h"
 
 /*
@@ -54,8 +53,8 @@ pcmk__new_nvpair(const char *name, const char *value)
     nvpair = calloc(1, sizeof(pcmk_nvpair_t));
     CRM_ASSERT(nvpair);
 
-    nvpair->name = strdup(name);
-    nvpair->value = value? strdup(value) : NULL;
+    pcmk__str_update(&nvpair->name, name);
+    pcmk__str_update(&nvpair->value, value);
     return nvpair;
 }
 
@@ -63,7 +62,7 @@ pcmk__new_nvpair(const char *name, const char *value)
  * \internal
  * \brief Free a name/value pair
  *
- * \param[in] nvpair  Name/value pair to free
+ * \param[in,out] nvpair  Name/value pair to free
  */
 static void
 pcmk__free_nvpair(gpointer data)
@@ -97,7 +96,7 @@ pcmk_prepend_nvpair(GSList *nvpairs, const char *name, const char *value)
 /*!
  * \brief Free a list of name/value pairs
  *
- * \param[in] list  List to free
+ * \param[in,out] list  List to free
  */
 void
 pcmk_free_nvpairs(GSList *nvpairs)
@@ -159,11 +158,11 @@ pcmk_sort_nvpairs(GSList *list)
  *       \c pcmk_free_nvpairs().
  */
 GSList *
-pcmk_xml_attrs2nvpairs(xmlNode *xml)
+pcmk_xml_attrs2nvpairs(const xmlNode *xml)
 {
     GSList *result = NULL;
 
-    for (xmlAttrPtr iter = pcmk__first_xml_attr(xml); iter != NULL;
+    for (xmlAttrPtr iter = pcmk__xe_first_attr(xml); iter != NULL;
          iter = iter->next) {
 
         result = pcmk_prepend_nvpair(result,
@@ -195,7 +194,7 @@ pcmk__nvpair_add_xml_attr(gpointer data, gpointer user_data)
 /*!
  * \brief Add XML attributes based on a list of name/value pairs
  *
- * \param[in]     list  List of name/value pairs
+ * \param[in,out] list  List of name/value pairs
  * \param[in,out] xml   XML node to add attributes to
  */
 void
@@ -207,6 +206,7 @@ pcmk_nvpairs2xml_attrs(GSList *list, xmlNode *xml)
 // convenience function for name=value strings
 
 /*!
+ * \internal
  * \brief Extract the name and value from an input string formatted as "name=value".
  * If unable to extract them, they are returned as NULL.
  *
@@ -218,9 +218,9 @@ pcmk_nvpairs2xml_attrs(GSList *list, xmlNode *xml)
  *         and error code otherwise
  */
 int
-pcmk_scan_nvpair(const char *input, char **name, char **value)
+pcmk__scan_nvpair(const char *input, char **name, char **value)
 {
-#ifdef SSCANF_HAS_M
+#ifdef HAVE_SSCANF_M
     *name = NULL;
     *value = NULL;
     if (sscanf(input, "%m[^=]=%m[^\n]", name, value) <= 0) {
@@ -277,9 +277,12 @@ pcmk_scan_nvpair(const char *input, char **name, char **value)
  * \param[in]     name  The name of the nvpair.
  * \param[in]     value The value of the nvpair.
  * \param[in]     units Optional units for the value, or NULL.
+ *
+ * \return Newly allocated string with name/value pair
  */
 char *
-pcmk_format_nvpair(const char *name, const char *value, const char *units) {
+pcmk__format_nvpair(const char *name, const char *value, const char *units)
+{
     return crm_strdup_printf("%s=\"%s%s\"", name, value, units ? units : "");
 }
 
@@ -287,15 +290,18 @@ pcmk_format_nvpair(const char *name, const char *value, const char *units) {
  * \internal
  * \brief Format a name/time pair.
  *
- * See pcmk_format_nvpair() for more details.
+ * See pcmk__format_nvpair() for more details.
  *
  * \note The caller is responsible for freeing the return value after use.
  *
  * \param[in]     name       The name for the time.
  * \param[in]     epoch_time The time to format.
+ *
+ * \return Newly allocated string with name/value pair
  */
 char *
-pcmk_format_named_time(const char *name, time_t epoch_time) {
+pcmk__format_named_time(const char *name, time_t epoch_time)
+{
     const char *now_str = pcmk__epoch2str(&epoch_time);
 
     return crm_strdup_printf("%s=\"%s\"", name, now_str ? now_str : "");
@@ -325,18 +331,6 @@ crm_xml_add(xmlNode *node, const char *name, const char *value)
     if (value == NULL) {
         return NULL;
     }
-#if XML_PARANOIA_CHECKS
-    {
-        const char *old_value = NULL;
-
-        old_value = crm_element_value(node, name);
-
-        /* Could be re-setting the same value */
-        CRM_CHECK(old_value != value,
-                  crm_err("Cannot reset %s with crm_xml_add(%s)", name, value);
-                  return value);
-    }
-#endif
 
     if (pcmk__tracking_xml_changes(node, FALSE)) {
         const char *old = crm_element_value(node, name);
@@ -346,7 +340,7 @@ crm_xml_add(xmlNode *node, const char *name, const char *value)
         }
     }
 
-    if (dirty && (pcmk__check_acl(node, name, xpf_acl_create) == FALSE)) {
+    if (dirty && (pcmk__check_acl(node, name, pcmk__xf_acl_create) == FALSE)) {
         crm_trace("Cannot add %s=%s to %s", name, value, node->name);
         return NULL;
     }
@@ -385,7 +379,7 @@ crm_xml_replace(xmlNode *node, const char *name, const char *value)
     /* Could be re-setting the same value */
     CRM_CHECK(old_value != value, return value);
 
-    if (pcmk__check_acl(node, name, xpf_acl_write) == FALSE) {
+    if (pcmk__check_acl(node, name, pcmk__xf_acl_write) == FALSE) {
         /* Create a fake object linked to doc->_private instead? */
         crm_trace("Cannot replace %s=%s to %s", name, value, node->name);
         return NULL;
@@ -424,7 +418,7 @@ crm_xml_replace(xmlNode *node, const char *name, const char *value)
 const char *
 crm_xml_add_int(xmlNode *node, const char *name, int value)
 {
-    char *number = crm_itoa(value);
+    char *number = pcmk__itoa(value);
     const char *added = crm_xml_add(node, name, number);
 
     free(number);
@@ -549,9 +543,9 @@ crm_element_value(const xmlNode *data, const char *name)
  *
  * This is like \c crm_element_value() but getting the value as an integer.
  *
- * \param[in] data   XML node to check
- * \param[in] name   Attribute name to check
- * \param[in] dest   Where to store element value
+ * \param[in]  data  XML node to check
+ * \param[in]  name  Attribute name to check
+ * \param[out] dest  Where to store element value
  *
  * \return 0 on success, -1 otherwise
  */
@@ -563,9 +557,13 @@ crm_element_value_int(const xmlNode *data, const char *name, int *dest)
     CRM_CHECK(dest != NULL, return -1);
     value = crm_element_value(data, name);
     if (value) {
-        errno = 0;
-        *dest = crm_parse_int(value, NULL);
-        if (errno == 0) {
+        long long value_ll;
+
+        if ((pcmk__scan_ll(value, &value_ll, 0LL) != pcmk_rc_ok)
+            || (value_ll < INT_MIN) || (value_ll > INT_MAX)) {
+            *dest = PCMK__PARSE_INT_DEFAULT;
+        } else {
+            *dest = (int) value_ll;
             return 0;
         }
     }
@@ -577,9 +575,9 @@ crm_element_value_int(const xmlNode *data, const char *name, int *dest)
  *
  * This is like \c crm_element_value() but getting the value as a long long int.
  *
- * \param[in] data   XML node to check
- * \param[in] name   Attribute name to check
- * \param[in] dest   Where to store element value
+ * \param[in]  data  XML node to check
+ * \param[in]  name  Attribute name to check
+ * \param[out] dest  Where to store element value
  *
  * \return 0 on success, -1 otherwise
  */
@@ -590,12 +588,9 @@ crm_element_value_ll(const xmlNode *data, const char *name, long long *dest)
 
     CRM_CHECK(dest != NULL, return -1);
     value = crm_element_value(data, name);
-    if (value) {
-        errno = 0;
-        *dest = crm_parse_ll(value, NULL);
-        if (errno == 0) {
-            return 0;
-        }
+    if ((value != NULL)
+        && (pcmk__scan_ll(value, dest, PCMK__PARSE_INT_DEFAULT) == pcmk_rc_ok)) {
+        return 0;
     }
     return -1;
 }
@@ -619,18 +614,11 @@ crm_element_value_ms(const xmlNode *data, const char *name, guint *dest)
 
     CRM_CHECK(dest != NULL, return -1);
     *dest = 0;
-
     value = crm_element_value(data, name);
-    if (value == NULL) {
-        return pcmk_ok;
-    }
-
-    errno = 0;
-    value_ll = crm_parse_ll(value, NULL);
-    if ((errno != 0) || (value_ll < 0) || (value_ll > G_MAXUINT)) {
+    if ((pcmk__scan_ll(value, &value_ll, 0LL) != pcmk_rc_ok)
+        || (value_ll < 0) || (value_ll > G_MAXUINT)) {
         return -1;
     }
-
     *dest = (guint) value_ll;
     return pcmk_ok;
 }
@@ -726,11 +714,8 @@ char *
 crm_element_value_copy(const xmlNode *data, const char *name)
 {
     char *value_copy = NULL;
-    const char *value = crm_element_value(data, name);
 
-    if (value != NULL) {
-        value_copy = strdup(value);
-    }
+    pcmk__str_update(&value_copy, crm_element_value(data, name));
     return value_copy;
 }
 
@@ -743,9 +728,9 @@ crm_element_value_copy(const xmlNode *data, const char *name)
  * name starts with a digit, this will instead add a \<param name=NAME
  * value=VALUE/> child to the XML (for legacy compatibility with heartbeat).
  *
- * \param[in] key        Key of hash table entry
- * \param[in] value      Value of hash table entry
- * \param[in] user_data  XML node
+ * \param[in]     key        Key of hash table entry
+ * \param[in]     value      Value of hash table entry
+ * \param[in,out] user_data  XML node
  */
 void
 hash2smartfield(gpointer key, gpointer value, gpointer user_data)
@@ -777,9 +762,9 @@ hash2smartfield(gpointer key, gpointer value, gpointer user_data)
  * and value, with an XML node passed as user data, and adds an XML attribute
  * with the specified name and value if it does not already exist.
  *
- * \param[in] key        Key of hash table entry
- * \param[in] value      Value of hash table entry
- * \param[in] user_data  XML node
+ * \param[in]     key        Key of hash table entry
+ * \param[in]     value      Value of hash table entry
+ * \param[in,out] user_data  XML node
  */
 void
 hash2field(gpointer key, gpointer value, gpointer user_data)
@@ -805,9 +790,9 @@ hash2field(gpointer key, gpointer value, gpointer user_data)
  * with the meta-attribute version of the specified name and value if it does
  * not already exist and if the name does not appear to be cluster-internal.
  *
- * \param[in] key        Key of hash table entry
- * \param[in] value      Value of hash table entry
- * \param[in] user_data  XML node
+ * \param[in]     key        Key of hash table entry
+ * \param[in]     value      Value of hash table entry
+ * \param[in,out] user_data  XML node
  */
 void
 hash2metafield(gpointer key, gpointer value, gpointer user_data)
@@ -837,10 +822,10 @@ hash2metafield(gpointer key, gpointer value, gpointer user_data)
 /*!
  * \brief Create an XML name/value pair
  *
- * \param[in] parent  If not \c NULL, make new XML node a child of this one
- * \param[in] id      If not \c NULL, use this as ID (otherwise auto-generate)
- * \param[in] name    Name to use
- * \param[in] value   Value to use
+ * \param[in,out] parent  If not \c NULL, make new XML node a child of this one
+ * \param[in]     id      Set this as XML ID (or NULL to auto-generate)
+ * \param[in]     name    Name to use
+ * \param[in]     value   Value to use
  *
  * \return New XML object on success, \c NULL otherwise
  */
@@ -878,9 +863,9 @@ crm_create_nvpair_xml(xmlNode *parent, const char *id, const char *name,
  * and value, with an XML node passed as the user data, and adds an \c nvpair
  * XML element with the specified name and value.
  *
- * \param[in] key        Key of hash table entry
- * \param[in] value      Value of hash table entry
- * \param[in] user_data  XML node
+ * \param[in]     key        Key of hash table entry
+ * \param[in]     value      Value of hash table entry
+ * \param[in,out] user_data  XML node
  */
 void
 hash2nvpair(gpointer key, gpointer value, gpointer user_data)
@@ -908,12 +893,12 @@ hash2nvpair(gpointer key, gpointer value, gpointer user_data)
  *       \c g_hash_table_destroy().
  */
 GHashTable *
-xml2list(xmlNode *parent)
+xml2list(const xmlNode *parent)
 {
     xmlNode *child = NULL;
     xmlAttrPtr pIter = NULL;
     xmlNode *nvpair_list = NULL;
-    GHashTable *nvpair_hash = crm_str_table_new();
+    GHashTable *nvpair_hash = pcmk__strkey_table(free, free);
 
     CRM_CHECK(parent != NULL, return nvpair_hash);
 
@@ -925,7 +910,7 @@ xml2list(xmlNode *parent)
 
     crm_log_xml_trace(nvpair_list, "Unpacking");
 
-    for (pIter = pcmk__first_xml_attr(nvpair_list); pIter != NULL;
+    for (pIter = pcmk__xe_first_attr(nvpair_list); pIter != NULL;
          pIter = pIter->next) {
 
         const char *p_name = (const char *)pIter->name;
@@ -952,3 +937,73 @@ xml2list(xmlNode *parent)
 
     return nvpair_hash;
 }
+
+void
+pcmk__xe_set_bool_attr(xmlNodePtr node, const char *name, bool value)
+{
+    crm_xml_add(node, name, value ? XML_BOOLEAN_TRUE : XML_BOOLEAN_FALSE);
+}
+
+int
+pcmk__xe_get_bool_attr(const xmlNode *node, const char *name, bool *value)
+{
+    const char *xml_value = NULL;
+    int ret, rc;
+
+    if (node == NULL) {
+        return ENODATA;
+    } else if (name == NULL || value == NULL) {
+        return EINVAL;
+    }
+
+    xml_value = crm_element_value(node, name);
+
+    if (xml_value == NULL) {
+        return ENODATA;
+    }
+
+    rc = crm_str_to_boolean(xml_value, &ret);
+    if (rc == 1) {
+        *value = ret;
+        return pcmk_rc_ok;
+    } else {
+        return pcmk_rc_unknown_format;
+    }
+}
+
+bool
+pcmk__xe_attr_is_true(const xmlNode *node, const char *name)
+{
+    bool value = false;
+    int rc;
+
+    rc = pcmk__xe_get_bool_attr(node, name, &value);
+    return rc == pcmk_rc_ok && value == true;
+}
+
+// Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
+
+#include <crm/common/util_compat.h>
+
+int
+pcmk_scan_nvpair(const char *input, char **name, char **value)
+{
+    return pcmk__scan_nvpair(input, name, value);
+}
+
+char *
+pcmk_format_nvpair(const char *name, const char *value,
+                   const char *units)
+{
+    return pcmk__format_nvpair(name, value, units);
+}
+
+char *
+pcmk_format_named_time(const char *name, time_t epoch_time)
+{
+    return pcmk__format_named_time(name, epoch_time);
+}
+
+// LCOV_EXCL_STOP
+// End deprecated API
