@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,6 +14,8 @@
 #include <crm/msg_xml.h>
 #include <crm/common/xml_internal.h>
 
+#include "pe_status_private.h"
+
 void populate_hash(xmlNode * nvpair_list, GHashTable * hash, const char **attrs, int attrs_length);
 
 resource_object_functions_t resource_class_functions[] = {
@@ -79,7 +81,7 @@ get_resource_type(const char *name)
     } else if (pcmk__str_eq(name, XML_CIB_TAG_INCARNATION, pcmk__str_casei)) {
         return pe_clone;
 
-    } else if (pcmk__str_eq(name, XML_CIB_TAG_MASTER, pcmk__str_casei)) {
+    } else if (pcmk__str_eq(name, PCMK_XE_PROMOTABLE_LEGACY, pcmk__str_casei)) {
         // @COMPAT deprecated since 2.0.0
         return pe_clone;
 
@@ -96,6 +98,46 @@ dup_attr(gpointer key, gpointer value, gpointer user_data)
     add_hash_param(user_data, key, value);
 }
 
+static void
+expand_parents_fixed_nvpairs(pe_resource_t * rsc, pe_rule_eval_data_t * rule_data, GHashTable * meta_hash, pe_working_set_t * data_set)
+{
+    GHashTable *parent_orig_meta = pcmk__strkey_table(free, free);
+    pe_resource_t *p = rsc->parent;
+
+    if (p == NULL) {
+        return ;
+    }
+
+    /* Search all parent resources, get the fixed value of "meta_attributes" set only in the original xml, and stack it in the hash table. */
+    /* The fixed value of the lower parent resource takes precedence and is not overwritten. */
+    while(p != NULL) {
+        /* A hash table for comparison is generated, including the id-ref. */
+        pe__unpack_dataset_nvpairs(p->xml, XML_TAG_META_SETS,
+                               rule_data, parent_orig_meta, NULL, FALSE, data_set);
+        p = p->parent; 
+    }
+
+    /* If there is a fixed value of "meta_attributes" of the parent resource, it will be processed. */
+    if (parent_orig_meta != NULL) {
+        GHashTableIter iter;
+        char *key = NULL;
+        char *value = NULL;
+
+        g_hash_table_iter_init(&iter, parent_orig_meta);
+        while (g_hash_table_iter_next(&iter, (gpointer *) &key, (gpointer *) &value)) {
+            /* Parameters set in the original xml of the parent resource will also try to overwrite the child resource. */
+            /* Attributes that already exist in the child lease are not updated. */
+            dup_attr(key, value, meta_hash);
+        }
+    }
+
+    if (parent_orig_meta != NULL) {
+        g_hash_table_destroy(parent_orig_meta);
+    }
+    
+    return ;
+
+}
 void
 get_meta_attributes(GHashTable * meta_hash, pe_resource_t * rsc,
                     pe_node_t * node, pe_working_set_t * data_set)
@@ -119,33 +161,36 @@ get_meta_attributes(GHashTable * meta_hash, pe_resource_t * rsc,
         rule_data.node_hash = node->details->attrs;
     }
 
-    if (rsc->xml) {
-        xmlAttrPtr xIter = NULL;
+    for (xmlAttrPtr a = pcmk__xe_first_attr(rsc->xml); a != NULL; a = a->next) {
+        const char *prop_name = (const char *) a->name;
+        const char *prop_value = crm_element_value(rsc->xml, prop_name);
 
-        for (xIter = rsc->xml->properties; xIter; xIter = xIter->next) {
-            const char *prop_name = (const char *)xIter->name;
-            const char *prop_value = crm_element_value(rsc->xml, prop_name);
-
-            add_hash_param(meta_hash, prop_name, prop_value);
-        }
+        add_hash_param(meta_hash, prop_name, prop_value);
     }
 
     pe__unpack_dataset_nvpairs(rsc->xml, XML_TAG_META_SETS, &rule_data,
                                meta_hash, NULL, FALSE, data_set);
 
-    /* set anything else based on the parent */
+    /* Set the "meta_attributes" explicitly set in the parent resource to the hash table of the child resource. */
+    /* If it is already explicitly set as a child, it will not be overwritten. */
     if (rsc->parent != NULL) {
-        g_hash_table_foreach(rsc->parent->meta, dup_attr, meta_hash);
+        expand_parents_fixed_nvpairs(rsc, &rule_data, meta_hash, data_set);
     }
 
-    /* and finally check the defaults */
+    /* check the defaults */
     pe__unpack_dataset_nvpairs(data_set->rsc_defaults, XML_TAG_META_SETS,
                                &rule_data, meta_hash, NULL, FALSE, data_set);
+
+    /* If there is "meta_attributes" that the parent resource has not explicitly set, set a value that is not set from rsc_default either. */
+    /* The values already set up to this point will not be overwritten. */
+    if (rsc->parent) {
+        g_hash_table_foreach(rsc->parent->meta, dup_attr, meta_hash);
+    }
 }
 
 void
-get_rsc_attributes(GHashTable * meta_hash, pe_resource_t * rsc,
-                   pe_node_t * node, pe_working_set_t * data_set)
+get_rsc_attributes(GHashTable *meta_hash, const pe_resource_t *rsc,
+                   const pe_node_t *node, pe_working_set_t *data_set)
 {
     pe_rule_eval_data_t rule_data = {
         .node_hash = NULL,
@@ -174,36 +219,6 @@ get_rsc_attributes(GHashTable * meta_hash, pe_resource_t * rsc,
     }
 }
 
-#if ENABLE_VERSIONED_ATTRS
-void
-pe_get_versioned_attributes(xmlNode * meta_hash, pe_resource_t * rsc,
-                            pe_node_t * node, pe_working_set_t * data_set)
-{
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = (node == NULL)? NULL : node->details->attrs,
-        .role = RSC_ROLE_UNKNOWN,
-        .now = data_set->now,
-        .match_data = NULL,
-        .rsc_data = NULL,
-        .op_data = NULL
-    };
-
-    pe_eval_versioned_attributes(data_set->input, rsc->xml, XML_TAG_ATTR_SETS,
-                                 &rule_data, meta_hash, NULL);
-
-    /* set anything else based on the parent */
-    if (rsc->parent != NULL) {
-        pe_get_versioned_attributes(meta_hash, rsc->parent, node, data_set);
-
-    } else {
-        /* and finally check the defaults */
-        pe_eval_versioned_attributes(data_set->input, data_set->rsc_defaults,
-                                     XML_TAG_ATTR_SETS, &rule_data, meta_hash,
-                                     NULL);
-    }
-}
-#endif
-
 static char *
 template_op_key(xmlNode * op)
 {
@@ -211,8 +226,9 @@ template_op_key(xmlNode * op)
     const char *role = crm_element_value(op, "role");
     char *key = NULL;
 
-    if (pcmk__str_eq(role, RSC_ROLE_STARTED_S, pcmk__str_null_matches)
-        || pcmk__str_eq(role, RSC_ROLE_SLAVE_S, pcmk__str_none)) {
+    if ((role == NULL)
+        || pcmk__strcase_any_of(role, RSC_ROLE_STARTED_S, RSC_ROLE_UNPROMOTED_S,
+                                RSC_ROLE_UNPROMOTED_LEGACY_S, NULL)) {
         role = RSC_ROLE_UNKNOWN_S;
     }
 
@@ -291,9 +307,7 @@ unpack_template(xmlNode * xml_obj, xmlNode ** expanded_xml, pe_working_set_t * d
 
     if (template_ops && rsc_ops) {
         xmlNode *op = NULL;
-        GHashTable *rsc_ops_hash = g_hash_table_new_full(crm_str_hash,
-                                                         g_str_equal, free,
-                                                         NULL);
+        GHashTable *rsc_ops_hash = pcmk__strkey_table(free, NULL);
 
         for (op = pcmk__xe_first_child(rsc_ops); op != NULL;
              op = pcmk__xe_next(op)) {
@@ -381,7 +395,8 @@ detect_promotable(pe_resource_t *rsc)
     }
 
     // @COMPAT deprecated since 2.0.0
-    if (pcmk__str_eq(crm_element_name(rsc->xml), XML_CIB_TAG_MASTER, pcmk__str_casei)) {
+    if (pcmk__str_eq(crm_element_name(rsc->xml), PCMK_XE_PROMOTABLE_LEGACY,
+                     pcmk__str_casei)) {
         /* @TODO in some future version, pe_warn_once() here,
          *       then drop support in even later version
          */
@@ -392,50 +407,206 @@ detect_promotable(pe_resource_t *rsc)
     return FALSE;
 }
 
-gboolean
-common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
-              pe_resource_t * parent, pe_working_set_t * data_set)
+static void
+free_params_table(gpointer data)
+{
+    g_hash_table_destroy((GHashTable *) data);
+}
+
+/*!
+ * \brief Get a table of resource parameters
+ *
+ * \param[in,out] rsc       Resource to query
+ * \param[in]     node      Node for evaluating rules (NULL for defaults)
+ * \param[in]     data_set  Cluster working set
+ *
+ * \return Hash table containing resource parameter names and values
+ *         (or NULL if \p rsc or \p data_set is NULL)
+ * \note The returned table will be destroyed when the resource is freed, so
+ *       callers should not destroy it.
+ */
+GHashTable *
+pe_rsc_params(pe_resource_t *rsc, const pe_node_t *node,
+              pe_working_set_t *data_set)
+{
+    GHashTable *params_on_node = NULL;
+
+    /* A NULL node is used to request the resource's default parameters
+     * (not evaluated for node), but we always want something non-NULL
+     * as a hash table key.
+     */
+    const char *node_name = "";
+
+    // Sanity check
+    if ((rsc == NULL) || (data_set == NULL)) {
+        return NULL;
+    }
+    if ((node != NULL) && (node->details->uname != NULL)) {
+        node_name = node->details->uname;
+    }
+
+    // Find the parameter table for given node
+    if (rsc->parameter_cache == NULL) {
+        rsc->parameter_cache = pcmk__strikey_table(free, free_params_table);
+    } else {
+        params_on_node = g_hash_table_lookup(rsc->parameter_cache, node_name);
+    }
+
+    // If none exists yet, create one with parameters evaluated for node
+    if (params_on_node == NULL) {
+        params_on_node = pcmk__strkey_table(free, free);
+        get_rsc_attributes(params_on_node, rsc, node, data_set);
+        g_hash_table_insert(rsc->parameter_cache, strdup(node_name),
+                            params_on_node);
+    }
+    return params_on_node;
+}
+
+/*!
+ * \internal
+ * \brief Unpack a resource's "requires" meta-attribute
+ *
+ * \param[in] rsc         Resource being unpacked
+ * \param[in] value       Value of "requires" meta-attribute
+ * \param[in] is_default  Whether \p value was selected by default
+ */
+static void
+unpack_requires(pe_resource_t *rsc, const char *value, bool is_default)
+{
+    if (pcmk__str_eq(value, PCMK__VALUE_NOTHING, pcmk__str_casei)) {
+
+    } else if (pcmk__str_eq(value, PCMK__VALUE_QUORUM, pcmk__str_casei)) {
+        pe__set_resource_flags(rsc, pe_rsc_needs_quorum);
+
+    } else if (pcmk__str_eq(value, PCMK__VALUE_FENCING, pcmk__str_casei)) {
+        pe__set_resource_flags(rsc, pe_rsc_needs_fencing);
+        if (!pcmk_is_set(rsc->cluster->flags, pe_flag_stonith_enabled)) {
+            pcmk__config_warn("%s requires fencing but fencing is disabled",
+                              rsc->id);
+        }
+
+    } else if (pcmk__str_eq(value, PCMK__VALUE_UNFENCING, pcmk__str_casei)) {
+        if (pcmk_is_set(rsc->flags, pe_rsc_fence_device)) {
+            pcmk__config_warn("Resetting \"" XML_RSC_ATTR_REQUIRES "\" for %s "
+                              "to \"" PCMK__VALUE_QUORUM "\" because fencing "
+                              "devices cannot require unfencing", rsc->id);
+            unpack_requires(rsc, PCMK__VALUE_QUORUM, true);
+            return;
+
+        } else if (!pcmk_is_set(rsc->cluster->flags, pe_flag_stonith_enabled)) {
+            pcmk__config_warn("Resetting \"" XML_RSC_ATTR_REQUIRES "\" for %s "
+                              "to \"" PCMK__VALUE_QUORUM "\" because fencing "
+                              "is disabled", rsc->id);
+            unpack_requires(rsc, PCMK__VALUE_QUORUM, true);
+            return;
+
+        } else {
+            pe__set_resource_flags(rsc,
+                                   pe_rsc_needs_fencing|pe_rsc_needs_unfencing);
+        }
+
+    } else {
+        const char *orig_value = value;
+
+        if (pcmk_is_set(rsc->flags, pe_rsc_fence_device)) {
+            value = PCMK__VALUE_QUORUM;
+
+        } else if ((rsc->variant == pe_native)
+                   && xml_contains_remote_node(rsc->xml)) {
+            value = PCMK__VALUE_QUORUM;
+
+        } else if (pcmk_is_set(rsc->cluster->flags, pe_flag_enable_unfencing)) {
+            value = PCMK__VALUE_UNFENCING;
+
+        } else if (pcmk_is_set(rsc->cluster->flags, pe_flag_stonith_enabled)) {
+            value = PCMK__VALUE_FENCING;
+
+        } else if (rsc->cluster->no_quorum_policy == no_quorum_ignore) {
+            value = PCMK__VALUE_NOTHING;
+
+        } else {
+            value = PCMK__VALUE_QUORUM;
+        }
+
+        if (orig_value != NULL) {
+            pcmk__config_err("Resetting '" XML_RSC_ATTR_REQUIRES "' for %s "
+                             "to '%s' because '%s' is not valid",
+                              rsc->id, value, orig_value);
+        }
+        unpack_requires(rsc, value, true);
+        return;
+    }
+
+    pe_rsc_trace(rsc, "\tRequired to start: %s%s", value,
+                 (is_default? " (default)" : ""));
+}
+
+/*!
+ * \internal
+ * \brief Unpack configuration XML for a given resource
+ *
+ * Unpack the XML object containing a resource's configuration into a new
+ * \c pe_resource_t object.
+ *
+ * \param[in]     xml_obj   XML node containing the resource's configuration
+ * \param[out]    rsc       Where to store the unpacked resource information
+ * \param[in]     parent    Resource's parent, if any
+ * \param[in,out] data_set  Cluster working set
+ *
+ * \return Standard Pacemaker return code
+ * \note If pcmk_rc_ok is returned, \p *rsc is guaranteed to be non-NULL, and
+ *       the caller is responsible for freeing it using its variant-specific
+ *       free() method. Otherwise, \p *rsc is guaranteed to be NULL.
+ */
+int
+pe__unpack_resource(xmlNode *xml_obj, pe_resource_t **rsc,
+                    pe_resource_t *parent, pe_working_set_t *data_set)
 {
-    bool isdefault = FALSE;
     xmlNode *expanded_xml = NULL;
     xmlNode *ops = NULL;
     const char *value = NULL;
-    const char *rclass = NULL; /* Look for this after any templates have been expanded */
-    const char *id = crm_element_value(xml_obj, XML_ATTR_ID);
-    bool guest_node = FALSE;
-    bool remote_node = FALSE;
-    bool has_versioned_params = FALSE;
+    const char *id = NULL;
+    bool guest_node = false;
+    bool remote_node = false;
 
     pe_rule_eval_data_t rule_data = {
         .node_hash = NULL,
         .role = RSC_ROLE_UNKNOWN,
-        .now = data_set->now,
+        .now = NULL,
         .match_data = NULL,
         .rsc_data = NULL,
         .op_data = NULL
     };
 
-    crm_log_xml_trace(xml_obj, "Processing resource input...");
+    CRM_CHECK(rsc != NULL, return EINVAL);
+    CRM_CHECK((xml_obj != NULL) && (data_set != NULL),
+              *rsc = NULL;
+              return EINVAL);
 
+    rule_data.now = data_set->now;
+
+    crm_log_xml_trace(xml_obj, "[raw XML]");
+
+    id = crm_element_value(xml_obj, XML_ATTR_ID);
     if (id == NULL) {
-        pe_err("Must specify id tag in <resource>");
-        return FALSE;
-
-    } else if (rsc == NULL) {
-        pe_err("Nowhere to unpack resource into");
-        return FALSE;
-
+        pe_err("Ignoring <%s> configuration without " XML_ATTR_ID,
+               crm_element_name(xml_obj));
+        return pcmk_rc_unpack_error;
     }
 
     if (unpack_template(xml_obj, &expanded_xml, data_set) == FALSE) {
-        return FALSE;
+        return pcmk_rc_unpack_error;
     }
 
     *rsc = calloc(1, sizeof(pe_resource_t));
+    if (*rsc == NULL) {
+        crm_crit("Unable to allocate memory for resource '%s'", id);
+        return ENOMEM;
+    }
     (*rsc)->cluster = data_set;
 
     if (expanded_xml) {
-        crm_log_xml_trace(expanded_xml, "Expanded resource...");
+        crm_log_xml_trace(expanded_xml, "[expanded XML]");
         (*rsc)->xml = expanded_xml;
         (*rsc)->orig_xml = xml_obj;
 
@@ -445,7 +616,7 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
     }
 
     /* Do not use xml_obj from here on, use (*rsc)->xml in case templates are involved */
-    rclass = crm_element_value((*rsc)->xml, XML_AGENT_ATTR_CLASS);
+
     (*rsc)->parent = parent;
 
     ops = find_xml_node((*rsc)->xml, "operations", FALSE);
@@ -453,24 +624,16 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
 
     (*rsc)->variant = get_resource_type(crm_element_name((*rsc)->xml));
     if ((*rsc)->variant == pe_unknown) {
-        pe_err("Unknown resource type: %s", crm_element_name((*rsc)->xml));
-        free(*rsc);
-        return FALSE;
+        pe_err("Ignoring resource '%s' of unknown type '%s'",
+               id, crm_element_name((*rsc)->xml));
+        common_free(*rsc);
+        *rsc = NULL;
+        return pcmk_rc_unpack_error;
     }
 
-    (*rsc)->parameters = crm_str_table_new();
-
-#if ENABLE_VERSIONED_ATTRS
-    (*rsc)->versioned_parameters = create_xml_node(NULL, XML_TAG_RSC_VER_ATTRS);
-#endif
-
-    (*rsc)->meta = crm_str_table_new();
-
-    (*rsc)->allowed_nodes =
-        g_hash_table_new_full(crm_str_hash, g_str_equal, NULL, free);
-
-    (*rsc)->known_on = g_hash_table_new_full(crm_str_hash, g_str_equal, NULL,
-                                             free);
+    (*rsc)->meta = pcmk__strkey_table(free, free);
+    (*rsc)->allowed_nodes = pcmk__strkey_table(NULL, free);
+    (*rsc)->known_on = pcmk__strkey_table(NULL, free);
 
     value = crm_element_value((*rsc)->xml, XML_RSC_ATTR_INCARNATION);
     if (value) {
@@ -482,13 +645,9 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
     }
 
     (*rsc)->fns = &resource_class_functions[(*rsc)->variant];
-    pe_rsc_trace((*rsc), "Unpacking resource...");
 
     get_meta_attributes((*rsc)->meta, *rsc, NULL, data_set);
-    get_rsc_attributes((*rsc)->parameters, *rsc, NULL, data_set);
-#if ENABLE_VERSIONED_ATTRS
-    pe_get_versioned_attributes((*rsc)->versioned_parameters, *rsc, NULL, data_set);
-#endif
+    (*rsc)->parameters = pe_rsc_params(*rsc, NULL, data_set); // \deprecated
 
     (*rsc)->flags = 0;
     pe__set_resource_flags(*rsc, pe_rsc_runnable|pe_rsc_provisional);
@@ -509,7 +668,12 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
     (*rsc)->failure_timeout = 0;
 
     value = g_hash_table_lookup((*rsc)->meta, XML_CIB_ATTR_PRIORITY);
-    (*rsc)->priority = crm_parse_int(value, "0");
+    (*rsc)->priority = char2score(value);
+
+    value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_CRITICAL);
+    if ((value == NULL) || crm_is_true(value)) {
+        pe__set_resource_flags(*rsc, pe_rsc_critical);
+    }
 
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_NOTIFY);
     if (crm_is_true(value)) {
@@ -519,28 +683,23 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
     if (xml_contains_remote_node((*rsc)->xml)) {
         (*rsc)->is_remote_node = TRUE;
         if (g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_CONTAINER)) {
-            guest_node = TRUE;
+            guest_node = true;
         } else {
-            remote_node = TRUE;
+            remote_node = true;
         }
     }
 
     value = g_hash_table_lookup((*rsc)->meta, XML_OP_ATTR_ALLOW_MIGRATE);
-#if ENABLE_VERSIONED_ATTRS
-    has_versioned_params = xml_has_children((*rsc)->versioned_parameters);
-#endif
-    if (crm_is_true(value) && has_versioned_params) {
-        pe_rsc_trace((*rsc), "Migration is disabled for resources with versioned parameters");
-    } else if (crm_is_true(value)) {
+    if (crm_is_true(value)) {
         pe__set_resource_flags(*rsc, pe_rsc_allow_migrate);
-    } else if ((value == NULL) && remote_node && !has_versioned_params) {
+    } else if ((value == NULL) && remote_node) {
         /* By default, we want remote nodes to be able
          * to float around the cluster without having to stop all the
          * resources within the remote-node before moving. Allowing
          * migration support enables this feature. If this ever causes
          * problems, migration support can be explicitly turned off with
          * allow-migrate=false.
-         * We don't support migration for versioned resources, though. */
+         */
         pe__set_resource_flags(*rsc, pe_rsc_allow_migrate);
     }
 
@@ -575,32 +734,46 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
         pe__set_resource_flags(*rsc, pe_rsc_unique);
     }
 
-    pe_rsc_trace((*rsc), "Options for %s", (*rsc)->id);
-
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_RESTART);
     if (pcmk__str_eq(value, "restart", pcmk__str_casei)) {
         (*rsc)->restart_type = pe_restart_restart;
-        pe_rsc_trace((*rsc), "\tDependency restart handling: restart");
+        pe_rsc_trace((*rsc), "%s dependency restart handling: restart",
+                     (*rsc)->id);
         pe_warn_once(pe_wo_restart_type,
                      "Support for restart-type is deprecated and will be removed in a future release");
 
     } else {
         (*rsc)->restart_type = pe_restart_ignore;
-        pe_rsc_trace((*rsc), "\tDependency restart handling: ignore");
+        pe_rsc_trace((*rsc), "%s dependency restart handling: ignore",
+                     (*rsc)->id);
     }
 
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_MULTIPLE);
     if (pcmk__str_eq(value, "stop_only", pcmk__str_casei)) {
         (*rsc)->recovery_type = recovery_stop_only;
-        pe_rsc_trace((*rsc), "\tMultiple running resource recovery: stop only");
+        pe_rsc_trace((*rsc), "%s multiple running resource recovery: stop only",
+                     (*rsc)->id);
 
     } else if (pcmk__str_eq(value, "block", pcmk__str_casei)) {
         (*rsc)->recovery_type = recovery_block;
-        pe_rsc_trace((*rsc), "\tMultiple running resource recovery: block");
+        pe_rsc_trace((*rsc), "%s multiple running resource recovery: block",
+                     (*rsc)->id);
 
-    } else {
+    } else if (pcmk__str_eq(value, "stop_unexpected", pcmk__str_casei)) {
+        (*rsc)->recovery_type = recovery_stop_unexpected;
+        pe_rsc_trace((*rsc), "%s multiple running resource recovery: "
+                             "stop unexpected instances",
+                     (*rsc)->id);
+
+    } else { // "stop_start"
+        if (!pcmk__str_eq(value, "stop_start",
+                          pcmk__str_casei|pcmk__str_null_matches)) {
+            pe_warn("%s is not a valid value for " XML_RSC_ATTR_MULTIPLE
+                    ", using default of \"stop_start\"", value);
+        }
         (*rsc)->recovery_type = recovery_stop_start;
-        pe_rsc_trace((*rsc), "\tMultiple running resource recovery: stop/start");
+        pe_rsc_trace((*rsc), "%s multiple running resource recovery: "
+                             "stop/start", (*rsc)->id);
     }
 
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_STICKINESS);
@@ -623,85 +796,15 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
         }
     }
 
-    if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
+    if (pcmk__str_eq(crm_element_value((*rsc)->xml, XML_AGENT_ATTR_CLASS),
+                     PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
         pe__set_working_set_flags(data_set, pe_flag_have_stonith_resource);
         pe__set_resource_flags(*rsc, pe_rsc_fence_device);
     }
 
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_REQUIRES);
+    unpack_requires(*rsc, value, false);
 
-  handle_requires_pref:
-    if (pcmk__str_eq(value, "nothing", pcmk__str_casei)) {
-
-    } else if (pcmk__str_eq(value, "quorum", pcmk__str_casei)) {
-        pe__set_resource_flags(*rsc, pe_rsc_needs_quorum);
-
-    } else if (pcmk__str_eq(value, "unfencing", pcmk__str_casei)) {
-        if (pcmk_is_set((*rsc)->flags, pe_rsc_fence_device)) {
-            pcmk__config_warn("Resetting '" XML_RSC_ATTR_REQUIRES "' for %s "
-                              "to 'quorum' because fencing devices cannot "
-                              "require unfencing", (*rsc)->id);
-            value = "quorum";
-            isdefault = TRUE;
-            goto handle_requires_pref;
-
-        } else if (!pcmk_is_set(data_set->flags, pe_flag_stonith_enabled)) {
-            pcmk__config_warn("Resetting '" XML_RSC_ATTR_REQUIRES "' for %s "
-                              "to 'quorum' because fencing is disabled",
-                              (*rsc)->id);
-            value = "quorum";
-            isdefault = TRUE;
-            goto handle_requires_pref;
-
-        } else {
-            pe__set_resource_flags(*rsc, pe_rsc_needs_fencing
-                                           |pe_rsc_needs_unfencing);
-        }
-
-    } else if (pcmk__str_eq(value, "fencing", pcmk__str_casei)) {
-        pe__set_resource_flags(*rsc, pe_rsc_needs_fencing);
-        if (!pcmk_is_set(data_set->flags, pe_flag_stonith_enabled)) {
-            pcmk__config_warn("%s requires fencing but fencing is disabled",
-                              (*rsc)->id);
-        }
-
-    } else {
-        const char *orig_value = value;
-
-        isdefault = TRUE;
-        if (pcmk_is_set((*rsc)->flags, pe_rsc_fence_device)) {
-            value = "quorum";
-
-        } else if (((*rsc)->variant == pe_native)
-                   && pcmk__str_eq(crm_element_value((*rsc)->xml, XML_AGENT_ATTR_CLASS), PCMK_RESOURCE_CLASS_OCF, pcmk__str_casei)
-                   && pcmk__str_eq(crm_element_value((*rsc)->xml, XML_AGENT_ATTR_PROVIDER), "pacemaker", pcmk__str_casei)
-                   && pcmk__str_eq(crm_element_value((*rsc)->xml, XML_ATTR_TYPE), "remote", pcmk__str_casei)
-            ) {
-            value = "quorum";
-
-        } else if (pcmk_is_set(data_set->flags, pe_flag_enable_unfencing)) {
-            value = "unfencing";
-
-        } else if (pcmk_is_set(data_set->flags, pe_flag_stonith_enabled)) {
-            value = "fencing";
-
-        } else if (data_set->no_quorum_policy == no_quorum_ignore) {
-            value = "nothing";
-
-        } else {
-            value = "quorum";
-        }
-
-        if (orig_value != NULL) {
-            pcmk__config_err("Resetting '" XML_RSC_ATTR_REQUIRES "' for %s "
-                             "to '%s' because '%s' is not valid",
-                              (*rsc)->id, value, orig_value);
-        }
-
-        goto handle_requires_pref;
-    }
-
-    pe_rsc_trace((*rsc), "\tRequired to start: %s%s", value, isdefault?" (default)":"");
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_FAIL_TIMEOUT);
     if (value != NULL) {
         // Stored as seconds
@@ -709,7 +812,15 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
     }
 
     if (remote_node) {
-        value = g_hash_table_lookup((*rsc)->parameters, XML_REMOTE_ATTR_RECONNECT_INTERVAL);
+        GHashTable *params = pe_rsc_params(*rsc, NULL, data_set);
+
+        /* Grabbing the value now means that any rules based on node attributes
+         * will evaluate to false, so such rules should not be used with
+         * reconnect_interval.
+         *
+         * @TODO Evaluate per node before using
+         */
+        value = g_hash_table_lookup(params, XML_REMOTE_ATTR_RECONNECT_INTERVAL);
         if (value) {
             /* reconnect delay works by setting failure_timeout and preventing the
              * connection from starting until the failure is cleared. */
@@ -721,11 +832,13 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
     }
 
     get_target_role(*rsc, &((*rsc)->next_role));
-    pe_rsc_trace((*rsc), "\tDesired next state: %s",
+    pe_rsc_trace((*rsc), "%s desired next state: %s", (*rsc)->id,
                  (*rsc)->next_role != RSC_ROLE_UNKNOWN ? role2text((*rsc)->next_role) : "default");
 
     if ((*rsc)->fns->unpack(*rsc, data_set) == FALSE) {
-        return FALSE;
+        (*rsc)->fns->free(*rsc);
+        *rsc = NULL;
+        return pcmk_rc_unpack_error;
     }
 
     if (pcmk_is_set(data_set->flags, pe_flag_symmetric_cluster)) {
@@ -738,22 +851,22 @@ common_unpack(xmlNode * xml_obj, pe_resource_t ** rsc,
         resource_location(*rsc, NULL, 0, "remote_connection_default", data_set);
     }
 
-    pe_rsc_trace((*rsc), "\tAction notification: %s",
+    pe_rsc_trace((*rsc), "%s action notification: %s", (*rsc)->id,
                  pcmk_is_set((*rsc)->flags, pe_rsc_notify)? "required" : "not required");
 
-    (*rsc)->utilization = crm_str_table_new();
+    (*rsc)->utilization = pcmk__strkey_table(free, free);
 
     pe__unpack_dataset_nvpairs((*rsc)->xml, XML_TAG_UTILIZATION, &rule_data,
                                (*rsc)->utilization, NULL, FALSE, data_set);
 
-/* 	data_set->resources = g_list_append(data_set->resources, (*rsc)); */
-
     if (expanded_xml) {
         if (add_template_rsc(xml_obj, data_set) == FALSE) {
-            return FALSE;
+            (*rsc)->fns->free(*rsc);
+            *rsc = NULL;
+            return pcmk_rc_unpack_error;
         }
     }
-    return TRUE;
+    return pcmk_rc_ok;
 }
 
 void
@@ -764,11 +877,11 @@ common_update_score(pe_resource_t * rsc, const char *id, int score)
     node = pe_hash_table_lookup(rsc->allowed_nodes, id);
     if (node != NULL) {
         pe_rsc_trace(rsc, "Updating score for %s on %s: %d + %d", rsc->id, id, node->weight, score);
-        node->weight = pe__add_scores(node->weight, score);
+        node->weight = pcmk__add_scores(node->weight, score);
     }
 
     if (rsc->children) {
-        GListPtr gIter = rsc->children;
+        GList *gIter = rsc->children;
 
         for (; gIter != NULL; gIter = gIter->next) {
             pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
@@ -823,14 +936,9 @@ common_free(pe_resource_t * rsc)
     g_list_free(rsc->rsc_tickets);
     g_list_free(rsc->dangling_migrations);
 
-    if (rsc->parameters != NULL) {
-        g_hash_table_destroy(rsc->parameters);
+    if (rsc->parameter_cache != NULL) {
+        g_hash_table_destroy(rsc->parameter_cache);
     }
-#if ENABLE_VERSIONED_ATTRS
-    if (rsc->versioned_parameters != NULL) {
-        free_xml(rsc->versioned_parameters);
-    }
-#endif
     if (rsc->meta != NULL) {
         g_hash_table_destroy(rsc->meta);
     }
@@ -999,3 +1107,22 @@ pe__count_common(pe_resource_t *rsc)
         }
     }
 }
+
+/*!
+ * \internal
+ * \brief Update a resource's next role
+ *
+ * \param[in,out] rsc   Resource to be updated
+ * \param[in]     role  Resource's new next role
+ * \param[in]     why   Human-friendly reason why role is changing (for logs)
+ */
+void
+pe__set_next_role(pe_resource_t *rsc, enum rsc_role_e role, const char *why)
+{
+    CRM_ASSERT((rsc != NULL) && (why != NULL));
+    if (rsc->next_role != role) {
+        pe_rsc_trace(rsc, "Resetting next role for %s from %s to %s (%s)",
+                     rsc->id, role2text(rsc->next_role), role2text(role), why);
+        rsc->next_role = role;
+    }
+}
