@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2021 the Pacemaker project contributors
+ * Copyright 2008-2022 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -264,7 +264,7 @@ pe_get_failcount(pe_node_t *node, pe_resource_t *rsc, time_t *last_failure,
     g_hash_table_iter_init(&iter, node->details->attrs);
     while (g_hash_table_iter_next(&iter, (gpointer *) &key, (gpointer *) &value)) {
         if (regexec(&failcount_re, key, 0, NULL, 0) == 0) {
-            failcount = pe__add_scores(failcount, char2score(value));
+            failcount = pcmk__add_scores(failcount, char2score(value));
         } else if (regexec(&lastfailure_re, key, 0, NULL, 0) == 0) {
             long long last_ll;
 
@@ -298,7 +298,7 @@ pe_get_failcount(pe_node_t *node, pe_resource_t *rsc, time_t *last_failure,
 
         if (now > (last + rsc->failure_timeout)) {
             crm_debug("Failcount for %s on %s expired after %ds",
-                      rsc->id, node->details->uname, rsc->failure_timeout);
+                      rsc->id, pe__node_name(node), rsc->failure_timeout);
             failcount = 0;
         }
     }
@@ -331,22 +331,18 @@ pe_get_failcount(pe_node_t *node, pe_resource_t *rsc, time_t *last_failure,
         }
 
         if (failcount > 0) {
-            char *score = score2char(failcount);
-
-            crm_info("Container %s and the resources within it have failed %s times on %s",
-                     rsc->id, score, node->details->uname);
-            free(score);
+            crm_info("Container %s and the resources within it "
+                     "have failed %s time%s on %s",
+                     rsc->id, pcmk_readable_score(failcount),
+                     pcmk__plural_s(failcount), pe__node_name(node));
         }
 
     } else if (failcount > 0) {
-        char *score = score2char(failcount);
-
-        crm_info("%s has failed %s times on %s",
-                 rsc->id, score, node->details->uname);
-        free(score);
+        crm_info("%s has failed %s time%s on %s",
+                 rsc->id, pcmk_readable_score(failcount),
+                 pcmk__plural_s(failcount), pe__node_name(node));
     }
 
-
     return failcount;
 }
 
@@ -374,6 +370,6 @@ pe__clear_failcount(pe_resource_t *rsc, pe_node_t *node,
                           data_set);
     add_hash_param(clear->meta, XML_ATTR_TE_NOWAIT, XML_BOOLEAN_TRUE);
     crm_notice("Clearing failure of %s on %s because %s " CRM_XS " %s",
-               rsc->id, node->details->uname, reason, clear->uuid);
+               rsc->id, pe__node_name(node), reason, clear->uuid);
     return clear;
 }
