@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2021 the Pacemaker project contributors
+ * Copyright 2018-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,6 +14,15 @@
  * declared with G_GNUC_INTERNAL for efficiency.
  */
 
+#if defined(PCMK__UNIT_TESTING)
+#undef G_GNUC_INTERNAL
+#define G_GNUC_INTERNAL
+#endif
+
+/*!
+ * \internal
+ * \deprecated This macro will be removed in a future release
+ */
 #  define status_print(fmt, args...)           \
    if(options & pe_print_html) {           \
        FILE *stream = print_data;      \
@@ -37,6 +46,13 @@ G_GNUC_INTERNAL
 void pe__force_anon(const char *standard, pe_resource_t *rsc, const char *rid,
                     pe_working_set_t *data_set);
 
+G_GNUC_INTERNAL
+gint pe__cmp_rsc_priority(gconstpointer a, gconstpointer b);
+
+G_GNUC_INTERNAL
+gboolean pe__unpack_resource(xmlNode *xml_obj, pe_resource_t **rsc,
+                             pe_resource_t *parent, pe_working_set_t *data_set);
+
 G_GNUC_INTERNAL
 gboolean unpack_remote_nodes(xmlNode *xml_resources, pe_working_set_t *data_set);
 
@@ -61,4 +77,7 @@ op_digest_cache_t *pe__compare_fencing_digest(pe_resource_t *rsc,
                                               pe_node_t *node,
                                               pe_working_set_t *data_set);
 
+G_GNUC_INTERNAL
+void pe__unpack_node_health_scores(pe_working_set_t *data_set);
+
 #endif  // PE_STATUS_PRIVATE__H
