@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,12 +15,14 @@
 #include <crm/common/ipc.h>
 #include <crm/cib/internal.h>
 
+#include <pacemaker-internal.h>
+
 static int message_timeout_ms = 30;
 static int command_options = 0;
 static int request_id = 0;
 static int bump_log_num = 0;
 
-static const char *host = NULL;
+static char *host = NULL;
 static const char *cib_user = NULL;
 static const char *cib_action = NULL;
 static const char *obj_type = NULL;
@@ -120,6 +122,19 @@ static pcmk__cli_option_t long_options[] = {
         "md5-sum-versioned", no_argument, NULL, '6',
         "Calculate an on-the-wire versioned CIB digest", pcmk__option_default
     },
+    {
+        "show-access", optional_argument, NULL, 'S',
+        "Whether to use syntax highlighting for ACLs "
+            "(with -Q/--query and -U/--user)",
+        pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\n\tThat amounts to one of \"color\" (default for terminal),"
+            " \"text\" (otherwise), \"namespace\", or \"auto\""
+            " (per former defaults).",
+        pcmk__option_default
+    },
     {
         "blank", no_argument, NULL, '-',
         NULL, pcmk__option_hidden
@@ -333,6 +348,16 @@ static pcmk__cli_option_t long_options[] = {
         "-spacer-", no_argument, NULL, '-',
         " cibadmin --replace --xml-file $HOME/local.xml", pcmk__option_example
     },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Assuming terminal, render configuration in color (green for writable, blue for readable, red for denied) to visualize permissions for user tony:",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " cibadmin --show-access=color --query --user tony | less -r",
+        pcmk__option_example
+    },
     {
         "-spacer-", no_argument, NULL, '-',
         "SEE ALSO:", pcmk__option_default
@@ -375,7 +400,7 @@ print_xml_output(xmlNode * xml)
 
     } else {
         buffer = dump_xml_formatted(xml);
-        fprintf(stdout, "%s", crm_str(buffer));
+        fprintf(stdout, "%s", pcmk__s(buffer, "<null>\n"));
         free(buffer);
     }
 }
@@ -384,7 +409,7 @@ print_xml_output(xmlNode * xml)
 static void
 report_schema_unchanged(void)
 {
-    const char *err = pcmk_strerror(pcmk_err_schema_unchanged);
+    const char *err = pcmk_rc_str(pcmk_rc_schema_unchanged);
 
     crm_info("Upgrade unnecessary: %s\n", err);
     printf("Upgrade unnecessary: %s\n", err);
@@ -404,6 +429,15 @@ main(int argc, char **argv)
     gboolean admin_input_stdin = FALSE;
     xmlNode *output = NULL;
     xmlNode *input = NULL;
+    char *username = NULL;
+    const char *acl_cred = NULL;
+    enum acl_eval_how {
+        acl_eval_unused,
+        acl_eval_auto,
+        acl_eval_namespace,
+        acl_eval_text,
+        acl_eval_color,
+    } acl_eval_how = acl_eval_unused;
 
     int option_index = 0;
 
@@ -438,33 +472,57 @@ main(int argc, char **argv)
                                       cib_xpath_address);
                 break;
             case 'u':
-                cib_action = CIB_OP_UPGRADE;
+                cib_action = PCMK__CIB_REQUEST_UPGRADE;
                 dangerous_cmd = TRUE;
                 break;
             case 'E':
-                cib_action = CIB_OP_ERASE;
+                cib_action = PCMK__CIB_REQUEST_ERASE;
                 dangerous_cmd = TRUE;
                 break;
+            case 'S':
+                if (optarg != NULL) {
+                    if (!strcmp(optarg, "auto")) {
+                        acl_eval_how = acl_eval_auto;
+                    } else if (!strcmp(optarg, "namespace")) {
+                        acl_eval_how = acl_eval_namespace;
+                    } else if (!strcmp(optarg, "text")) {
+                        acl_eval_how = acl_eval_text;
+                    } else if (!strcmp(optarg, "color")) {
+                        acl_eval_how = acl_eval_color;
+                    } else {
+                        fprintf(stderr, "Unrecognized value for --show-access: \"%s\"\n",
+                               optarg);
+                        ++argerr;
+                    }
+                } else {
+                    acl_eval_how = acl_eval_auto;
+                }
+                /* XXX this is a workaround until we unify happy paths for
+                       both a/sync handling; the respective extra code is
+                       only in sync path now, but does it matter at all for
+                       query-like request wrt. what blackbox users observe? */
+                command_options |= cib_sync_call;
+                break;
             case 'Q':
-                cib_action = CIB_OP_QUERY;
+                cib_action = PCMK__CIB_REQUEST_QUERY;
                 break;
             case 'P':
-                cib_action = CIB_OP_APPLY_DIFF;
+                cib_action = PCMK__CIB_REQUEST_APPLY_PATCH;
                 break;
             case 'U':
                 cib_user = optarg;
                 break;
             case 'M':
-                cib_action = CIB_OP_MODIFY;
+                cib_action = PCMK__CIB_REQUEST_MODIFY;
                 break;
             case 'R':
-                cib_action = CIB_OP_REPLACE;
+                cib_action = PCMK__CIB_REQUEST_REPLACE;
                 break;
             case 'C':
-                cib_action = CIB_OP_CREATE;
+                cib_action = PCMK__CIB_REQUEST_CREATE;
                 break;
             case 'D':
-                cib_action = CIB_OP_DELETE;
+                cib_action = PCMK__CIB_REQUEST_DELETE;
                 break;
             case '5':
                 cib_action = "md5-sum";
@@ -481,7 +539,7 @@ main(int argc, char **argv)
                                       cib_no_children);
                 break;
             case 'B':
-                cib_action = CIB_OP_BUMP;
+                cib_action = PCMK__CIB_REQUEST_BUMP;
                 crm_log_args(argc, argv);
                 break;
             case 'V':
@@ -514,14 +572,14 @@ main(int argc, char **argv)
                 break;
             case 'N':
             case 'h':
-                host = strdup(optarg);
+                pcmk__str_update(&host, optarg);
                 break;
             case 'l':
                 cib__set_call_options(command_options, crm_system_name,
                                       cib_scope_local);
                 break;
             case 'd':
-                cib_action = CIB_OP_DELETE;
+                cib_action = PCMK__CIB_REQUEST_DELETE;
                 cib__set_call_options(command_options, crm_system_name,
                                       cib_multiple);
                 dangerous_cmd = TRUE;
@@ -547,7 +605,7 @@ main(int argc, char **argv)
                     crm_xml_add(output, XML_ATTR_VALIDATION, argv[optind]);
                 }
                 admin_input_xml = dump_xml_formatted(output);
-                fprintf(stdout, "%s\n", crm_str(admin_input_xml));
+                fprintf(stdout, "%s", pcmk__s(admin_input_xml, "<null>\n"));
                 crm_exit(CRM_EX_OK);
                 break;
             default:
@@ -597,6 +655,39 @@ main(int argc, char **argv)
     } else if (admin_input_stdin) {
         source = "STDIN";
         input = stdin2xml();
+
+    } else if (acl_eval_how != acl_eval_unused) {
+        username = pcmk__uid2username(geteuid());
+        if (pcmk_acl_required(username)) {
+            if (force_flag) {
+                fprintf(stderr, "The supplied command can provide skewed"
+                                 " result since it is run under user that also"
+                                 " gets guarded per ACLs on their own right."
+                                 " Continuing since --force flag was"
+                                 " provided.\n");
+
+            } else {
+                fprintf(stderr, "The supplied command can provide skewed"
+                                 " result since it is run under user that also"
+                                 " gets guarded per ACLs in their own right."
+                                 " To accept the risk of such a possible"
+                                 " distortion (without even knowing it at this"
+                                 " time), use the --force flag.\n");
+                crm_exit(CRM_EX_UNSAFE);
+            }
+
+        }
+        free(username);
+        username = NULL;
+
+        if (cib_user == NULL) {
+            fprintf(stderr, "The supplied command requires -U user specified.\n");
+            crm_exit(CRM_EX_USAGE);
+        }
+
+        /* we already stopped/warned ACL-controlled users about consequences */
+        acl_cred = cib_user;
+        cib_user = NULL;
     }
 
     if (input != NULL) {
@@ -617,7 +708,7 @@ main(int argc, char **argv)
 
         digest = calculate_on_disk_digest(input);
         fprintf(stderr, "Digest: ");
-        fprintf(stdout, "%s\n", crm_str(digest));
+        fprintf(stdout, "%s\n", pcmk__s(digest, "<null>"));
         free(digest);
         free_xml(input);
         crm_exit(CRM_EX_OK);
@@ -634,7 +725,7 @@ main(int argc, char **argv)
         version = crm_element_value(input, XML_ATTR_CRM_VERSION);
         digest = calculate_xml_versioned_digest(input, FALSE, TRUE, version);
         fprintf(stderr, "Versioned (%s) digest: ", version);
-        fprintf(stdout, "%s\n", crm_str(digest));
+        fprintf(stdout, "%s\n", pcmk__s(digest, "<null>"));
         free(digest);
         free_xml(input);
         crm_exit(CRM_EX_OK);
@@ -645,7 +736,7 @@ main(int argc, char **argv)
         crm_err("Init failed, could not perform requested operations");
         fprintf(stderr, "Init failed, could not perform requested operations\n");
         free_xml(input);
-        crm_exit(crm_errno2exit(rc));
+        crm_exit(pcmk_rc2exitc(pcmk_legacy2rc(rc)));
     }
 
     rc = do_work(input, command_options, &output);
@@ -666,20 +757,23 @@ main(int argc, char **argv)
         g_main_loop_run(mainloop);
 
     } else if ((rc == -pcmk_err_schema_unchanged)
-               && pcmk__str_eq(cib_action, CIB_OP_UPGRADE, pcmk__str_none)) {
+               && pcmk__str_eq(cib_action, PCMK__CIB_REQUEST_UPGRADE,
+                               pcmk__str_none)) {
         report_schema_unchanged();
 
     } else if (rc < 0) {
-        crm_err("Call failed: %s", pcmk_strerror(rc));
-        fprintf(stderr, "Call failed: %s\n", pcmk_strerror(rc));
+        rc = pcmk_legacy2rc(rc);
+        crm_err("Call failed: %s", pcmk_rc_str(rc));
+        fprintf(stderr, "Call failed: %s\n", pcmk_rc_str(rc));
 
-        if (rc == -pcmk_err_schema_validation) {
-            if (pcmk__str_eq(cib_action, CIB_OP_UPGRADE, pcmk__str_none)) {
+        if (rc == pcmk_rc_schema_validation) {
+            if (pcmk__str_eq(cib_action, PCMK__CIB_REQUEST_UPGRADE,
+                             pcmk__str_none)) {
                 xmlNode *obj = NULL;
-                int version = 0, rc = 0;
+                int version = 0;
 
-                rc = the_cib->cmds->query(the_cib, NULL, &obj, command_options);
-                if (rc == pcmk_ok) {
+                if (the_cib->cmds->query(the_cib, NULL, &obj,
+                                         command_options) == pcmk_ok) {
                     update_validation(&obj, &version, 0, TRUE, FALSE);
                 }
 
@@ -687,7 +781,48 @@ main(int argc, char **argv)
                 validate_xml_verbose(output);
             }
         }
-        exit_code = crm_errno2exit(rc);
+        exit_code = pcmk_rc2exitc(rc);
+    }
+
+    if (output != NULL && acl_eval_how != acl_eval_unused) {
+        xmlDoc *acl_evaled_doc;
+        rc = pcmk__acl_annotate_permissions(acl_cred, output->doc, &acl_evaled_doc);
+        if (rc == pcmk_rc_ok) {
+            enum pcmk__acl_render_how how;
+            xmlChar *rendered = NULL;
+            free_xml(output);
+            switch(acl_eval_how) {
+                case acl_eval_text:
+                    how = pcmk__acl_render_text;
+                    break;
+                case acl_eval_color:
+                    how = pcmk__acl_render_color;
+                    break;
+                case acl_eval_namespace:
+                    how = pcmk__acl_render_namespace;
+                    break;
+                default:
+                    if (/*acl_eval_auto*/ isatty(STDOUT_FILENO)) {
+                        how = pcmk__acl_render_color;
+                    } else {
+                        how = pcmk__acl_render_text;
+                    }
+                    break;
+            }
+
+            if (!pcmk__acl_evaled_render(acl_evaled_doc, how,
+                                            &rendered)) {
+                printf("%s\n", (char *) rendered);
+                free(rendered);
+            } else {
+                fprintf(stderr, "Could not render evaluated access\n");
+                crm_exit(CRM_EX_CONFIG);
+            }
+            output = NULL;
+        } else {
+            fprintf(stderr, "Could not evaluate access per request (%s, error: %s)\n", acl_cred, pcmk_rc_str(rc));
+            crm_exit(CRM_EX_CONFIG);
+        }
     }
 
     if (output != NULL) {
@@ -698,12 +833,12 @@ main(int argc, char **argv)
     crm_trace("%s exiting normally", crm_system_name);
 
     free_xml(input);
-    rc = the_cib->cmds->signoff(the_cib);
+    rc = cib__clean_up_connection(&the_cib);
     if (exit_code == CRM_EX_OK) {
-        exit_code = crm_errno2exit(rc);
+        exit_code = pcmk_rc2exitc(rc);
     }
-    cib_delete(the_cib);
 
+    free(host);
     crm_exit(exit_code);
 }
 
@@ -712,9 +847,9 @@ do_work(xmlNode * input, int call_options, xmlNode ** output)
 {
     /* construct the request */
     the_cib->call_timeout = message_timeout_ms;
-    if (strcasecmp(CIB_OP_REPLACE, cib_action) == 0
+    if ((strcmp(cib_action, PCMK__CIB_REQUEST_REPLACE) == 0)
         && pcmk__str_eq(crm_element_name(input), XML_TAG_CIB, pcmk__str_casei)) {
-        xmlNode *status = get_object_root(XML_CIB_TAG_STATUS, input);
+        xmlNode *status = pcmk_find_cib_element(input, XML_CIB_TAG_STATUS);
 
         if (status == NULL) {
             create_xml_node(input, XML_CIB_TAG_STATUS);
@@ -750,17 +885,20 @@ do_init(void)
 void
 cibadmin_op_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *user_data)
 {
-    exit_code = crm_errno2exit(rc);
+    rc = pcmk_legacy2rc(rc);
+    exit_code = pcmk_rc2exitc(rc);
 
-    if (rc == -pcmk_err_schema_unchanged) {
+    if (rc == pcmk_rc_schema_unchanged) {
         report_schema_unchanged();
 
-    } else if (rc != pcmk_ok) {
-        crm_warn("Call %s failed (%d): %s", cib_action, rc, pcmk_strerror(rc));
-        fprintf(stderr, "Call %s failed (%d): %s\n", cib_action, rc, pcmk_strerror(rc));
+    } else if (rc != pcmk_rc_ok) {
+        crm_warn("Call %s failed: %s " CRM_XS " rc=%d",
+                 cib_action, pcmk_rc_str(rc), rc);
+        fprintf(stderr, "Call %s failed: %s\n", cib_action, pcmk_rc_str(rc));
         print_xml_output(output);
 
-    } else if (pcmk__str_eq(cib_action, CIB_OP_QUERY, pcmk__str_casei) && output == NULL) {
+    } else if (pcmk__str_eq(cib_action, PCMK__CIB_REQUEST_QUERY, pcmk__str_none)
+               && (output == NULL)) {
         crm_err("Query returned no output");
         crm_log_xml_err(msg, "no output");
 
