@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -20,6 +20,7 @@
 #include <crm/common/mainloop.h>
 #include <crm/msg_xml.h>
 #include <crm/cib.h>
+#include <crm/cib/internal.h>
 #include <crm/common/ipc_controld.h>
 #include <crm/common/attrd_internal.h>
 
@@ -127,7 +128,7 @@ remove_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError *
 
     options.command = 'R';
     options.dangerous_cmd = TRUE;
-    options.target_uname = strdup(optarg);
+    pcmk__str_update(&options.target_uname, optarg);
     return TRUE;
 }
 
@@ -363,8 +364,7 @@ cib_remove_node(long id, const char *name)
                 name, id, pcmk_strerror(rc));
     }
 
-    cib->cmds->signoff(cib);
-    cib_delete(cib);
+    cib__clean_up_connection(&cib);
     return rc;
 }
 
@@ -429,11 +429,8 @@ tools_remove_node_cache(const char *node_name, long nodeid, const char *target)
         crm_xml_add(cmd, F_ORIG, crm_system_name);
 
         crm_xml_add(cmd, PCMK__XA_TASK, PCMK__ATTRD_CMD_PEER_REMOVE);
-        crm_xml_add(cmd, PCMK__XA_ATTR_NODE_NAME, node_name);
 
-        if (nodeid > 0) {
-            crm_xml_add_int(cmd, PCMK__XA_ATTR_NODE_ID, (int) nodeid);
-        }
+        pcmk__xe_add_node(cmd, node_name, nodeid);
 
     } else { // Fencer or pacemakerd
         cmd = create_request(CRM_OP_RM_NODE_CACHE, NULL, NULL, target,
