@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2021 the Pacemaker project contributors
+ * Copyright 2004-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,6 +8,10 @@
  */
 
 #include <crm_internal.h>
+
+#include <stdint.h>
+#include <stdbool.h>
+
 #include <crm/crm.h>
 
 #include <crm/msg_xml.h>
@@ -31,13 +35,15 @@ typedef struct node_info_s {
 enum resource_check_flags {
     rsc_remain_stopped  = (1 << 0),
     rsc_unpromotable    = (1 << 1),
-    rsc_unmanaged       = (1 << 2)
+    rsc_unmanaged       = (1 << 2),
+    rsc_locked          = (1 << 3),
+    rsc_node_health     = (1 << 4),
 };
 
 typedef struct resource_checks_s {
-    pe_resource_t *rsc;
-    unsigned int flags;
-    const char *lock_node;
+    pe_resource_t *rsc;     // Resource being checked
+    uint32_t flags;         // Group of enum resource_check_flags
+    const char *lock_node;  // Node that resource is shutdown-locked to, if any
 } resource_checks_t;
 
 resource_checks_t *cli_check_resource(pe_resource_t *rsc, char *role_s, char *managed);
@@ -63,7 +69,8 @@ int cli_resource_print_operations(const char *rsc_id, const char *host_uname,
                                   bool active, pe_working_set_t * data_set);
 
 /* runtime */
-int cli_resource_check(pcmk__output_t *out, cib_t * cib, pe_resource_t *rsc);
+int cli_resource_check(pcmk__output_t *out, pe_resource_t *rsc,
+                       pe_node_t *node);
 int cli_resource_fail(pcmk_ipc_api_t *controld_api, const char *host_uname,
                       const char *rsc_id, pe_working_set_t *data_set);
 GList *cli_resource_search(pe_resource_t *rsc, const char *requested_name,
@@ -75,7 +82,7 @@ int cli_resource_delete(pcmk_ipc_api_t *controld_api, const char *host_uname,
 int cli_cleanup_all(pcmk_ipc_api_t *controld_api, const char *node_name,
                     const char *operation, const char *interval_spec,
                     pe_working_set_t *data_set);
-int cli_resource_restart(pcmk__output_t *out, pe_resource_t *rsc, const char *host,
+int cli_resource_restart(pcmk__output_t *out, pe_resource_t *rsc, pe_node_t *node,
                          const char *move_lifetime, int timeout_ms, cib_t *cib,
                          int cib_options, gboolean promoted_role_only, gboolean force);
 int cli_resource_move(pe_resource_t *rsc, const char *rsc_id, const char *host_name,
