@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2020 the Pacemaker project contributors
+ * Copyright 2009-2022 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -229,6 +229,7 @@ static GOptionEntry addl_entries[] = {
 
 static pcmk__supported_format_t formats[] = {
     PCMK__SUPPORTED_FORMAT_HTML,
+    PCMK__SUPPORTED_FORMAT_NONE,
     PCMK__SUPPORTED_FORMAT_TEXT,
     PCMK__SUPPORTED_FORMAT_XML,
     { NULL, NULL, NULL }
@@ -277,7 +278,7 @@ add_stonith_params(const gchar *option_name, const gchar *optarg, gpointer data,
 
     crm_info("Scanning: -o %s", optarg);
 
-    rc = pcmk_scan_nvpair(optarg, &name, &value);
+    rc = pcmk__scan_nvpair(optarg, &name, &value);
 
     if (rc != 2) {
         rc = pcmk_legacy2rc(rc);
@@ -330,51 +331,76 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
     return context;
 }
 
+// \return Standard Pacemaker return code
+static int
+request_fencing(stonith_t *st, const char *target, const char *command,
+                GError **error)
+{
+    char *reason = NULL;
+    int rc = pcmk__request_fencing(st, target, command, name,
+                                   options.timeout * 1000,
+                                   options.tolerance * 1000,
+                                   options.delay, &reason);
+
+    if (rc != pcmk_rc_ok) {
+        const char *rc_str = pcmk_rc_str(rc);
+        const char *what = (strcmp(command, "on") == 0)? "unfence" : "fence";
+
+        // If reason is identical to return code string, don't display it twice
+        if (pcmk__str_eq(rc_str, reason, pcmk__str_none)) {
+            free(reason);
+            reason = NULL;
+        }
+
+        g_set_error(error, PCMK__RC_ERROR, rc,
+                    "Couldn't %s %s: %s%s%s%s",
+                    what, target, rc_str,
+                    ((reason == NULL)? "" : " ("),
+                    ((reason == NULL)? "" : reason),
+                    ((reason == NULL)? "" : ")"));
+    }
+    free(reason);
+    return rc;
+}
+
 int
 main(int argc, char **argv)
 {
     int rc = 0;
+    crm_exit_t exit_code = CRM_EX_OK;
     bool no_connect = false;
     bool required_agent = false;
 
     char *target = NULL;
     const char *device = NULL;
-
-    crm_exit_t exit_code = CRM_EX_OK;
     stonith_t *st = NULL;
 
-    pcmk__output_t *out = NULL;
-    pcmk__common_args_t *args = pcmk__new_common_args(SUMMARY);
-
     GError *error = NULL;
-    GOptionContext *context = NULL;
+
+    pcmk__output_t *out = NULL;
+
     GOptionGroup *output_group = NULL;
-    gchar **processed_args = NULL;
+    pcmk__common_args_t *args = pcmk__new_common_args(SUMMARY);
+    gchar **processed_args = pcmk__cmdline_preproc(argv, "adehilorstvyBCDFHQRTU");
+    GOptionContext *context = build_arg_context(args, &output_group);
 
-    context = build_arg_context(args, &output_group);
     pcmk__register_formats(output_group, formats);
-
-    crm_log_cli_init("stonith_admin");
-
-    name = strdup(crm_system_name);
-
-    processed_args = pcmk__cmdline_preproc(argv, "adehilorstvBCDFHQRTU");
-
     if (!g_option_context_parse_strv(context, &processed_args, &error)) {
-        fprintf(stderr, "%s: %s\n", g_get_prgname(), error->message);
         exit_code = CRM_EX_USAGE;
         goto done;
     }
 
-    for (int i = 0; i < args->verbosity; i++) {
-        crm_bump_log_level(argc, argv);
+    pcmk__cli_init_logging("stonith_admin", args->verbosity);
+
+    if (name == NULL) {
+        name = strdup(crm_system_name);
     }
 
     rc = pcmk__output_new(&out, args->output_ty, args->output_dest, argv);
     if (rc != pcmk_rc_ok) {
-        fprintf(stderr, "Error creating output format %s: %s\n",
-                args->output_ty, pcmk_rc_str(rc));
         exit_code = CRM_EX_ERROR;
+        g_set_error(&error, PCMK__EXITC_ERROR, exit_code, "Error creating output format %s: %s",
+                    args->output_ty, pcmk_rc_str(rc));
         goto done;
     }
 
@@ -479,7 +505,7 @@ main(int argc, char **argv)
         target = options.unregister_level;
     }
 
-    if (optind > argc || action == 0) {
+    if (action == 0) {
         char *help = g_option_context_get_help(context, TRUE, NULL);
 
         out->err(out, "%s", help);
@@ -549,25 +575,42 @@ main(int argc, char **argv)
             rc = st->cmds->register_device(st, st_opts, device, NULL, options.agent,
                                            options.params);
             rc = pcmk_legacy2rc(rc);
+            if (rc != pcmk_rc_ok) {
+                out->err(out, "Can't register device %s using agent %s: %s",
+                         device, options.agent, pcmk_rc_str(rc));
+            }
             break;
 
         case 'D':
             rc = st->cmds->remove_device(st, st_opts, device);
             rc = pcmk_legacy2rc(rc);
+            if (rc != pcmk_rc_ok) {
+                out->err(out, "Can't unregister device %s: %s",
+                         device, pcmk_rc_str(rc));
+            }
             break;
 
         case 'd':
             rc = pcmk__fence_unregister_level(st, target, options.fence_level);
+            if (rc != pcmk_rc_ok) {
+                out->err(out, "Can't unregister topology level %d for %s: %s",
+                         options.fence_level, target, pcmk_rc_str(rc));
+            }
             break;
 
         case 'r':
             rc = pcmk__fence_register_level(st, target, options.fence_level, options.devices);
+            if (rc != pcmk_rc_ok) {
+                out->err(out, "Can't register topology level %d for %s: %s",
+                         options.fence_level, target, pcmk_rc_str(rc));
+            }
             break;
 
         case 'M':
             rc = pcmk__fence_metadata(out, st, options.agent, options.timeout*1000);
             if (rc != pcmk_rc_ok) {
-                out->err(out, "Can't get fence agent meta-data: %s", pcmk_strerror(rc));
+                out->err(out, "Can't get fence agent meta-data: %s",
+                         pcmk_rc_str(rc));
             }
 
             break;
@@ -578,18 +621,15 @@ main(int argc, char **argv)
             break;
 
         case 'B':
-            rc = pcmk__fence_action(st, target, "reboot", name, options.timeout*1000,
-                                    options.tolerance*1000, options.delay);
+            rc = request_fencing(st, target, "reboot", &error);
             break;
 
         case 'F':
-            rc = pcmk__fence_action(st, target, "off", name, options.timeout*1000,
-                                    options.tolerance*1000, options.delay);
+            rc = request_fencing(st, target, "off", &error);
             break;
 
         case 'U':
-            rc = pcmk__fence_action(st, target, "on", name, options.timeout*1000,
-                                    options.tolerance*1000, options.delay);
+            rc = request_fencing(st, target, "on", &error);
             break;
 
         case 'h':
@@ -613,9 +653,10 @@ main(int argc, char **argv)
 
   done:
     g_strfreev(processed_args);
-    g_clear_error(&error);
     pcmk__free_arg_context(context);
 
+    pcmk__output_and_clear_error(error, out);
+
     if (out != NULL) {
         out->finish(out, exit_code, true, NULL);
         pcmk__output_free(out);
